<?
/*
Stronghold CMS - Database Connector
Copyright (C) 2008 Scott Barnett

This program is free software; you can redistribute it and/or
modify it under the terms of Version 3 of the GNU General Public
License as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

-----------------------------------------------------------------

Adding as much transparency as we can to the database connections.
Conditional if statements control the rest through the application
for database specific tweaks (otherwise known as hacks).

Specifically choosing not to use pear, because this gives us
more control over the connections.

*/

//******************************************************************************************************************
// PostgreSQL Database Connector

class pg{
	
	var $db; //database object
	
	function __destruct(){ pg_close($this->db); }
	
	function pg($server,$database,$username,$password){
		if ($server==NULL || $database==NULL || $username==NULL || $password==NULL){ return (false); }

		$s="dbname=".$database;
		$s.=" user=".$username;
		$s.=" password=".$password;
		$s.=" host=".$server;

		$this->db = pg_connect($s);
		if (!$this->db){ return (false); }

		return (true);
	}

	function query($sql){
		$result=NULL;
		//ob_start();
		$result = pg_exec($this->db,$sql);
		if (!$result){
			echo ("<br>\n");
			echo ($sql."<br><br>\n");
			//$error_msg=ob_get_contents();
			error_handler($error_msg,"Postgres");
		}
		//ob_end_flush(); //end flush screws with http header redirects

		$rs=$this->recordset($result);
		$result=NULL;
		return ($rs);
	}
	
	function recordset($result){
		$ri=0;
		$rs=array();

		while ($row=pg_fetch_assoc($result)){
			while( list($key, $value) = each($row) ){ $rs[$ri][$key]=$value; }
			$ri++;
		}
		return ($rs);
	}
	
	function boolean($bool){
		if ($bool=="t"){ return (true); } else { return (false); }
	}
	
	function next_val($sequence){
		$sql="SELECT NEXTVAL('".$sequence."') AS next_id";
		$rs=$this->query($sql);
		if (count($rs)!=1){ return (false); }
		return ($rs[0]["next_id"]);
	}
}

//******************************************************************************************************************
// Microsoft SQL Server Database Connector

class ms{

	var $db; //database object

	//default constructor
	function ms($server,$database,$username,$password){
		if ($server==NULL || $database==NULL || $username==NULL || $password==NULL){ return (false); }

		$this->db = mssql_connect($server,$username,$password);
		mssql_select_db($database,$this->db);
		return (true);
	}
	
	function query($sql){
		$result=NULL;
		ob_start();
		
		$result = mssql_query($sql);
		if (!$result){
			$error=mssql_get_last_message();
			
			echo ("<br>\n");
			echo ($sql."<br><br>\n");
			if($error){ echo ($error."<br><br>\n"); }
			
			$error_msg=ob_get_contents();
			error_handler($error_msg,"MsSQL");
		}
		//ob_end_flush(); //end flush screws with http header redirects

		$rs=$this->recordset($result);
		$result=NULL;
		return $rs;
	}
	
	function recordset($result){
		$ri=0;
		$rs=array();

		while (@$row=mssql_fetch_assoc($result)){
			while( list($key, $value) = each($row) ){ $rs[$ri][$key]=$value; }
			$ri++;
		}
		return $rs;
	}
	
	function insert_id(){
		$sql="SELECT @@IDENTITY as last_insert_id";
		$rs=$this->recordset(mssql_query($sql));
		$id=$rs[0]["last_insert_id"];
		if ($id){ return ($id); }
		return (false);		
	}

	function next_val($sequence){
		$db_common_prefix=$GLOBALS["CONFIG"]["db_common_prefix"];		
		//do this as one transaction so it's atomic
		$sql="UPDATE ".$db_common_prefix."seq SET seq_val=seq_val+1 WHERE seq_name='".$sequence."';".
			"SELECT seq_val FROM ".$db_common_prefix."seq WHERE seq_name='".$sequence."'";
		$rs=$this->query($sql);

		if (count($rs)!=1){ return (false); }
		return ($rs[0]["seq_val"]);
	}
	
	function convert_time($s){
		//convert an sql server time to unix epoch format
		//2006-02-21 12:12:00
		
		if (strlen($s)==19){
			$year=$s[0].$s[1].$s[2].$s[3];
			$month=$s[5].$s[6];
			$day=$s[8].$s[9];
			$hour=$s[11].$s[12];
			$min=$s[14].$s[15];
			$sec=$s[17].$s[18];
			//echo($hour." ".$min." ".$sec." ".$month." ".$day." ".$year);
			$r=mktime($hour,$min,$sec,$month,$day,$year);
		} else {
			$r=NULL;
		}
		return ($r);
	}
	
	function convert_date($d) {
		//convert an sql server date to unix epoch format
		// 2006-06-07 - mssql
		// 0123456789012345678
		$d = trim($d);
		
		if (strlen($d)==10){
			$day=$d[8].$d[9];
			$month=$d[5].$d[6];
			$year=$d[0].$d[1].$d[2].$d[3];
			$r=mktime(0,0,0,$month,$day,$year);
		} else {
			$r=NULL;
		}
		return ($r);
	}
}

//******************************************************************************************************************
// MySQL Database Connector

class my{
	
	var $db; //database object
		
	//default constructor
	
	function my($server,$database,$username,$password){
		if ($server==NULL || $database==NULL || $username==NULL || $password==NULL){ return (false); }

		$this->db = mysql_connect($server,$username,$password);
		mysql_select_db($database);

		return (true);
	}
	
	function query($sql){
		$result = mysql_query($sql,$this->db);
		if (!$result) { die('Invalid query: ' . mysql_error()); }
		$rs=$this->recordset($result);
		$result=NULL;
		return $rs;
	}
	
	function recordset($result){
		$ri=0;
		$rs=array();

		while ($row=@mysql_fetch_assoc($result)){
			while( list($key, $value) = each($row) ){ $rs[$ri][$key]=$value; }
			$ri++;
		}
		return $rs;
	}
}

//******************************************************************************************************************
// Oracle Database Connector (this doesn't work yet! I don't have a testing server)

class ora{
	
	var $db; //database object
		
	//default constructor
	
	function ora($server,$database,$username,$password){
		if ($server==NULL || $database==NULL || $username==NULL || $password==NULL){ return (false); }

		$this->db = ori_connect($username,$password);

		return (true);
	}

	
	function query($sql){
		$result = query($sql,$this->db);
		if (!$result) { die(); }
		$rs=$this->recordset($result);
		$result=NULL;
		return $rs;
	}
	
	function recordset($result){
		$ri=0;
		$rs=array();

		while ($row=@ori_fetch_assoc($result)){
			while( list($key, $value) = each($row) ){ $rs[$ri][$key]=$value; }
			$ri++;
		}
		return $rs;
	}
}

//******************************************************************************************************************
// Microsoft Access Database Connector (yuk)

class acc{
	

	var $_acc_db=NULL;
	var $_acc_database=NULL;


	//default constructor
	function acc($database="cic"){
		if ($database=="cic"){ $this->_acc_database="CIC"; }
		if ($database=="sc"){ $this->_acc_database="SocialClub"; }

		$this->_acc_db = odbc_connect ($this->_acc_database, 'root', '') or die( "Could    Not Connect to ODBC Database!" );
		return (1);
	}
	
	function query($sql){
		$result = odbc_exec($this->_acc_db, $sql) or die (odbc_errormsg()); 
		if (!$result){ echo ($sql."<br>\n"); }
		$rs=$this->recordset($result);
		$result=NULL;
		return $rs;		
	}
	
	function recordset($result){
		$ri=0;
		$rs=array();

		while($row = odbc_fetch_array($result)){
			while( list($key, $value) = each($row) ){ $rs[$ri][$key]=$value; }
			$ri++;
		}

		return $rs;
	}
}

//establish a database connection for a given configuration
function database_connector($config){
	if (!is_array($config)){ return (false); }

	if ($config["db_type"]=="postgres"){
		$db=new pg($config["db_server"],$config["db_database"],$config["db_username"],$config["db_password"]);
		return ($db);
	} elseif ($config["db_type"]=="mysql"){
		//untested
		//$db=new my($config["db_server"],$config["db_database"],$config["db_username"],$config["db_password"]);
		//return ($db);
	} elseif ($CONFIG["db_type"]=="mssql"){
		//untested
		//$db=new ms($config["db_server"],$config["db_database"],$config["db_username"],$config["db_password"]);
		//return ($db);
	} elseif ($CONFIG["db_type"]=="oracle"){
		//untested
		//$db=new or($config["db_server"],$config["db_database"],$config["db_username"],$config["db_password"]);
		//return ($db);
	}
	return (false);
}


//******************************************************************************************************************
// Generic Database Functions

//return the maximum value for a given column in a table
function max_value($db,$column,$table){
	$sql="SELECT MAX (".$column.") AS max FROM ".$table;
	$rs=$db->query($sql);
	return ($rs[0]["max"]);
}


function error_handler($error,$type){


	$error.="SERVER: ".prepare_str($_SERVER["HTTP_HOST"])."<br>\n";
	$error.="SCRIPT: ".prepare_str($_SERVER["PHP_SELF"])."<br>\n";
	$error.="TIME: ".date("d/m/Y H:i:s")."<br>\n";
	$error.="SESSION: <br>\n";
	//print_r($_SESSION);
	foreach($_SESSION as $key => $value){ $error.=$key." - ".$value."<br>\n"; }
	$error.="<br>\n";
	
	foreach ($GLOBALS["CONFIG"]["error_reporting"] as $id => $address){
		html_mail($address,"DB Error Handler <".$GLOBALS["CONFIG"]["system_email"].">",$type." Error",$error);
	}
}


//*****************************************************************************************
// Array Manipulation Functions


// Array juggling function
// calling with recordset and a key  makes $rs[0]["key"] into $rs[key]=array(values)
// calling with a recordset, a key, and a value makes $rs[0]["key"] into $rs[key]=value
function key_array($rs,$key,$value=NULL){
	if ($value==NULL){
		$rv=array();
		if (count($rs)!=0){
			if (array_key_exists($key,$rs[0])){
				for ($i=0; $i<count($rs); $i++){
					$r=$rs[$i];
					foreach ($r AS $rkey => $rvalue){
						if ($rkey!=$key){ $rv[ $r[$key] ][$rkey]=$rvalue; }
					}
				}
			}
		}
		return $rv;
	} else {
		$ret=array();
		foreach ($rs as $k => $v){ $ret[($v[$key])]=$v[$value]; }
		return ($ret);
	}
}



// multi dimensional array sorting
function aasort(&$array, $args) { 
	// example - aasort($your_array, array("+A", "-B", "+C"));
    foreach($args as $arg) { 
        $order_field = substr($arg, 1, strlen($arg)); 
        foreach($array as $array_row) { 
            $sort_array[$order_field][] = $array_row[$order_field]; 
        } 
        $sort_rule .= '$sort_array['.$order_field.'], '.($arg[0] == "+" ? SORT_ASC : SORT_DESC).','; 
    } 
    eval ("array_multisort($sort_rule".' &$array);'); 
}


//*****************************************************************************************
// String Manipulation Functions


//validate an email address
function vemail ($address) { 
	if (strlen($address)<4){ return (false); } //an email's always going to have at least 4 characters

	$bits=explode("@",$address);
	if (count($bits)!=2){ return (false); } //no @ sign or more than 1
	if (count($bits[0])==0 || count($bits[1])==0){ return (false); }
	


	if ($slength>2){
		for ($i=0; $i<$slength; $i++){
			if ($address[$i]=='@'){ //if we find an @
				//if it's not the first or last char
				if ((($i+1)!=$slength) && ($i!=0)) { 
					$valid=true;
				}
			}
		}
	}
	return (true);
}

// Function for taking a blob of text and inserting HTML into it
// I presume this function will grow
function decode($str){
	$str=str_replace("\n","<br>\n",$str);
	$str=str_replace("`","'",$str);
	return ($str);
}

//implode sql fields for an UPDATE
function implode_fields($fields){
        if (!is_array($fields)){ return (false); }
        $options="";
        foreach ($fields as $key => $val){ $options.=$key."=".$val.","; }
        $options=substr($options,0,strlen($options)-1);;
        return ($options);
}

//String Protection
function prepare_str($str){
	$str=prepare_txt($str);
	$illegal=array("\n","<",">");
	$legal=  array(" ", " "," ");
	$str=str_replace($illegal,$legal,$str);
	return ((string)$str);
}

//Text area protection
function prepare_txt($str){
	// 32 to 126 is our allowed character set array
	//for ($i=32; $i<127; $i++){ echo (chr($i)); }
	//  !"#$%&'()*+,-./0123456789:;<=>?@ABCDEFGHIJKLMNOPQRSTUVWXYZ[\]^_`abcdefghijklmnopqrstuvwxyz{|}~
	//carriage return is 10
	$illegal=array("'","","","","\\","\r");
	$legal=  array("`","\"","\"","`","/","");
	$str=str_replace($illegal,$legal,$str);

	for ($i=0; $i<strlen($str); $i++){
		//echo (ord($str[$i])."-".$str[$i].",");
		$o=ord($str[$i]);
		if (!($o>=32 && $o<=126) && ($o!=10)){ $str[$i]=" "; } //remove illegal characters
	}
	
	return ((string)$str);

}

//Integer protection
function prepare_int($str){
	// 45 to 57 is our allowed character set array
	//for ($i=45; $i<58; $i++){ echo (chr($i)); }
	if ($str==NULL){ return (NULL); }
	$str=(string)$str;

	for ($i=0; $i<strlen($str); $i++){
		$o=ord($str[$i]);
		if (!($o>=45 && $o<=57)){ $str[$i]=" "; } //remove illegal characters
	}
	$str=str_replace("/","",$str); //remove / characters
	$str=str_replace(" ","",$str); //remove spaces
	
	return ((string)$str); //return as a string, int won't handle decimals
}


//*****************************************************************************************
// Object Generation Functions

//print out a 2 dimensional array as a table
function ptable($rs){
	$r=NULL;
	
	if ($rs!=NULL){
		$firstrow=$rs[0];
	
		$r.="<table border='1' class='dtable'>\n";
		$r.="<tr>\n";
		while (list($key, $value) = each($firstrow)){ $r.="<th>".$key."</th>\n"; }
		$r.="</tr>\n";
		
		while (list($i, $row) = each($rs)) {
			$r.="<tr>\n";
			while (list($key, $value) = each($row)){ $r.="<td>".$value."</td>\n"; }
			$r.="</tr>\n";
		}
		$r.="</table>\n";
	} else {
		$r.="<p><i>No Data!!!</i></p>";
	}
	return ($r);
}

//random password generator
function randomstring($length=10){
	$salt = "abcdefghijklmnopqrstuvwxyz0123456789"; 
	$slength=strlen($salt)-1;

	//sead the random number generator
	srand((double)microtime()*1000000); 
	
	$password="";
	
	for ($i=0; $i<$length; $i++){
		$n=rand(0,$slength);
		$c=substr($salt,$n,1);
		$password.=$c;
	}
	
	return ($password); 
} 

//*****************************************************************************************
// Form Field Functions

//returns a select box given a formatted array
//  $label = the html name
//  $arr = a key'd array for selection options
//  $default = default value
//  $use_key = compare "default" to the array key, not to the array value
//  $allow_null = allow null values
//  $attributes = a javascript hook
function generic_select($label,$arr,$default,$use_key=true,$allow_null=true,$attributes=NULL){
	$r="<select name='".$label."' ".$attributes.">\n";
	if ($allow_null){ $r.="<option></option>\n"; }
	
	foreach($arr as $key => $value){
		$r.="<option";
		if ($use_key){
			$r.=" value='".$key."'";
			if ($default==$key){ $r.=" selected"; }
		} else {
			if ($default==$value){ $r.=" selected"; }
		}
		$r.=">".$value."</option>\n";
	}
	$r.="</select>\n";
	return ($r);
}

// return a select box for a generic jump menu (requires $javascript["jump_menu"]=1;)
function generic_jump($arr,$location,$default,$new=true,$nulls=true){
	if (!is_array($arr)){ return (false); }

	$r="<form name=\"form1\"><select name=\"menu1\" onChange=\"MM_jumpMenu('parent',this,0)\">\n";
	if ($nulls){ $r.="<option value=''></option>\n"; }
	if ($default=="new") { $selected=" selected"; } else { $selected=NULL; }
	if ($new){ $r.="<option value='".$location."new'".$selected.">--- New Item ---</option>\n"; }
	
	foreach ($arr as $key => $value){
		if ($default==$key) { $selected=" selected"; } else { $selected=NULL; }
		$r.="<option value='".$location.$key."'".$selected.">".$value."</option>\n";
	}
	$r.="</select></form>\n";

	return ($r);
}

//Date select field
function select_date($day,$month,$year,$prefix=NULL,$disabled=false){
//echo $day." ".$month." ".$year;

	if ($disabled == true){ $r= "<select name='".$prefix."day' disabled=\"true\">\n"; }
	else { $r= "<select name='".$prefix."day'>\n"; }
	
	$r.="<option value=''>--</option>\n";

	for ($i=1; $i<32; $i++){
		$r.="<option";
		if ($day==$i){ $r.=" selected"; }
		$r.=" value='".sprintf("%02d",$i)."'>".$i."</option>\n";
	}
	
	$r.="</select> / ";
	
	if ($disabled == true) {  $r.= "<select name='".$prefix."month' disabled=\"true\">\n"; }
	else { $r.= "<select name='".$prefix."month'>\n"; }
	
	$r.="<option value=''>--</option>\n";
	$r.="<option value='01'"; if ($month=="01") { $r.=" selected"; } $r.=">Jan</option>\n";
	$r.="<option value='02'"; if ($month=="02") { $r.=" selected"; } $r.=">Feb</option>\n";
	$r.="<option value='03'"; if ($month=="03") { $r.=" selected"; } $r.=">Mar</option>\n";
	$r.="<option value='04'"; if ($month=="04") { $r.=" selected"; } $r.=">Apr</option>\n";
	$r.="<option value='05'"; if ($month=="05") { $r.=" selected"; } $r.=">May</option>\n";
	$r.="<option value='06'"; if ($month=="06") { $r.=" selected"; } $r.=">Jun</option>\n";
	$r.="<option value='07'"; if ($month=="07") { $r.=" selected"; } $r.=">Jul</option>\n";
	$r.="<option value='08'"; if ($month=="08") { $r.=" selected"; } $r.=">Aug</option>\n";
	$r.="<option value='09'"; if ($month=="09") { $r.=" selected"; } $r.=">Sep</option>\n";
	$r.="<option value='10'"; if ($month=="10") { $r.=" selected"; } $r.=">Oct</option>\n";
	$r.="<option value='11'"; if ($month=="11") { $r.=" selected"; } $r.=">Nov</option>\n";
	$r.="<option value='12'"; if ($month=="12") { $r.=" selected"; } $r.=">Dec</option>\n";
	$r.="</select> / ";
	
	if ($disabled == true) { $r.= "<select name='".$prefix."year' disabled=\"true\">\n"; }
	else { $r.= "<select name='".$prefix."year'>\n"; }

	$r.="<option value=''>----</option>\n";
	
	for ($i=1980; $i<2011; $i++){
		$r.="<option";
		if ($year==$i) { $r.=" selected"; }
		$r.=">".$i."</option>\n";
	}
	$r.="</select>\n";
		
	return ($r);
}

function select_time($hour,$min,$prefix=NULL){
//echo $hour."  ".$min;
	if (! is_null($hour)){ $hour=(int)$hour; }
	if (! is_null($min)){ $min=(int)$min; }


	$r="<select name='".$prefix."hour'>\n";
	$r.="<option value=''>--</option>\n";
	for ($i=0; $i<24; $i++){
		$r.="<option";
		if ($hour===$i){ $r.=" selected"; }
		$r.=">".sprintf("%02d",$i)."</option>\n";
	}
	$r.="</select> : <select name='".$prefix."min'>\n";
	$r.="<option value=''>--</option>\n";
	for ($i=0; $i<60; $i++){
		$r.="<option";
		if ($min===$i){ $r.=" selected"; }
		$r.=">".sprintf("%02d",$i)."</option>\n";
	}
	$r.="</select>\n";
	
	return $r;
}


function text_date($day,$month,$year,$prefix=NULL,$no_note=0){
	$r="<input type='text' name='".$prefix."day' value='".$day."' size='2'> / ";
	$r.="<input type='text' name='".$prefix."month' value='".$month."' size='2'> / ";
	$r.="<input type='text' name='".$prefix."year' value='".$year."' size='4' maxlength='4'> ";
	if (!$no_note){ $r.="<span class='footnote'>DD/MM/YYYY</span>"; }
	return ($r);
}

function text_time($hour,$min,$prefix=NULL,$no_note=0){
	$r="<input type='text' name='".$prefix."hour' value='".$hour."' size='2' maxlength='2'> : ";
	$r.="<input type='text' name='".$prefix."min' value='".$min."' size='2' maxlength='2'> ";
	if (!$no_note){ $r.="<span class='footnote'>HH:MM (24HR)</span>"; }
	return ($r);
}

//*****************************************************************************************
// Email Related Functions

// send HTML email
function html_mail($to,$from,$subject,$html,$cc=NULL){
	//requires Pear - mail and mail mime
	
	require_once("Mail.php");
	require_once("Mail/mime.php");
	
	// send the email
	$mime = new Mail_mime();
	$mime->setHTMLBody($html);
	if ($cc){ $mime->addCc($cc); }
	$body = $mime->get();
	$headers = $mime->headers(array(
		"From" => $from,
		"Subject" => $subject
	));
										 
	$mail =& Mail::factory("mail");
	$mail->send($to,$headers,$body);

}


//*****************************************************************************************
// SQL Related Functions

function sql_string($var,$nulls=false){
	if ($nulls && $var==NULL){ return ("NULL"); }
	return ("'".$var."'");
}

function sql_boolean($var,$nulls=false){
	if ($nulls && $var==NULL){ return ("NULL"); }
	if ($var===1 || $var===true || $var==="1"){ return (1); }
	return (0);
}

function sql_int($var,$nulls=false){
	if ($nulls && $var==NULL){ return ("NULL"); }
	return ($var);
}

function sql_bool($var){
	$r="false";
	if ($var==1){ $r="true"; }
	if ($var==="t"){ $r="true"; }
	if ($var==="true"){ $r="true"; }
	return ($r);
}

// compensate for nulls
function sql_null($var){
	if ($var==NULL){ return ("NULL"); } else { return ($var); }
}

// compensate for string nulls
function sql_snull($var){
	if ($var==NULL){ return ("NULL"); } else { return ("'".$var."'"); }
}

//*****************************************************************************************
// Debugging Functions

// spits out an array dump with pre tags wrapped around it
// this is a debugging function only!
function pre($array){
	echo ("<div align='left'><pre>\n");
	print_r($array);
	echo ("</pre></div>\n");
}

//return a directory listing, return false on failure
// requires the full path name
function file_ls($folder){
	if ($folder==NULL){ return (false); }
	if (!is_dir($folder)){ return (false); }
	
	$dh=opendir($folder);
	if (!$dh){ return (false); }

	$list=array();
	while (($file = readdir($dh)) !== false) {
		if ($file[0]!="."){ $list[]=$file; } //ignore dir references and hidden files
	}
	closedir($dh);
	
	return ($list);
}

?>
