<?
/*
Stronghold CMS - Extended System
Copyright (C) 2008 Scott Barnett

This program is free software; you can redistribute it and/or
modify it under the terms of Version 3 of the GNU General Public
License as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

-----------------------------------------------------------------

The extended class is for functions not utilised at the CMS generic
run-time level. This is generally data manipluation-type requests.
For performance reasons, we want to define these type of occasionally
used functions seperately.

*/


//*********************************************************************************************
// GROUP Functions

//create a group, return false or the group_id
function group_create($db,$db_table_prefix,$group_name){
	if ($group_name==NULL){ fatal ("Group Creation Fault: No group_name specified"); }
	
	//create the group in the cms
	$group_id=$db->next_val($db_table_prefix."groups_group_id_seq"); //grab a new group id
	if (!$group_id){ return (false); }
	
	$insert=array();
	$insert["group_id"]=$group_id;
	$insert["title"]="'".$group_name."'";
	$sql="INSERT INTO ".$db_table_prefix."groups (".implode(",",array_keys($insert)).") VALUES (".implode(",",$insert).")";
	$db->query($sql);	

	return ($group_id);
}

function group_modify($db,$db_table_prefix,$group_id,$group_name){
	if ($group_id==NULL){ fatal ("Group Creation Fault: No group_id specified"); }
	if ($group_name==NULL){ fatal ("Group Creation Fault: No group_name specified"); }

	$sql="UPDATE ".$db_table_prefix."groups SET title='".$group_name."' WHERE group_id=".$group_id;
	$db->query($sql);
	
	return (true);
}

function group_add_user($db,$db_table_prefix,$group_id,$user_id,$level){
	if ($group_id==NULL){ return (false); }
	if ($user_id==NULL){ return (false); }
	if ($level==NULL){ return (false); }

	//make sure the entry doesn't already exist
	$sql="SELECT user_id FROM ".$db_table_prefix."group_members WHERE group_id=".$group_id." AND user_id=".$user_id;
	$rs=$db->query($sql);
	if (count($rs)!=0){ return (false); }

	//setup their permissions to the cms
	$insert=array();
	$insert["group_id"]=$group_id;
	$insert["user_id"]=$user_id;
	$insert["level"]=$level;
	$sql="INSERT INTO ".$db_table_prefix."group_members (".implode(",",array_keys($insert)).") VALUES (".implode(",",$insert).")";
	$db->query($sql);

	return (true);
}

function group_remove_user($db,$db_table_prefix,$group_id,$user_id){
	if ($group_id==NULL){ fatal("Group User Removal Fault: No group_id specified"); }
	if ($user_id==NULL){ fatal("Group User Removal Fault: No user_id specified"); }

	//make sure the entry doesn't already exist
	$sql="SELECT user_id FROM ".$db_table_prefix."group_members WHERE group_id=".$group_id." AND user_id=".$user_id;
	$rs=$db->query($sql);
	if (count($rs)==1){ return (false); }

	//setup their permissions to the cms
	$insert=array();
	$insert["group_id"]=$group_id;
	$insert["user_id"]=$user_id;
	$insert["level"]=$level;
	$sql="INSERT INTO ".$db_table_prefix."group_members (".implode(",",array_keys($insert)).") VALUES (".implode(",",$insert).")";
	echo ($sql."\n");
	$db->query($sql);

	return (true);
}

//*********************************************************************************************
// USER Functions

//creates a use.r and return the user_id on success, false on failure
function user_create($db,$db_common_prefix,$fields){
	if ($fields["username"]==NULL){ fatal ("User Creation Fault: No username specified"); }
	if ($fields["firstname"]==NULL){ fatal ("User Creation Fault: No firstname specified"); }
	if ($fields["surname"]==NULL){ fatal ("User Creation Fault: No surname specified"); }
	if ($fields["email"]==NULL){ fatal ("User Creation Fault: No email specified"); }
	if ($fields["access"]==NULL){ fatal ("User Creation Fault: No access specified"); }
	if ($fields["active"]==NULL){ fatal ("User Creation Fault: No active specified"); }
	if ($fields["editor_size"]==NULL){ fatal ("User Creation Fault: No editor_size specified"); }

	//check the user doesn't already exist 
	$sql="SELECT user_id FROM ".$db_common_prefix."users WHERE UPPER(username)='".strtoupper($fields["username"])."'";
	$rs=$db->query($sql);
	if (count($rs)!=0){ return (false); }

	//insert the user into common
	$user_id=$db->next_val($db_common_prefix."users_user_id_seq"); //grab a new user id
	if (!$user_id){ return (false); }

	$insert=array();
	$insert["user_id"]=$user_id;
	$insert["username"]="'".$fields["username"]."'";
	$insert["password"]=sql_snull($fields["password"]); //allow nulls
	$insert["firstname"]="'".$fields["firstname"]."'";
	$insert["surname"]="'".$fields["surname"]."'";
	$insert["email"]="'".$fields["email"]."'";
	$insert["access"]=$fields["access"];
	$insert["active"]=$fields["active"];
	$insert["editor_size"]="'".$fields["editor_size"]."'";
	$sql="INSERT INTO ".$db_common_prefix."users (".implode(",",array_keys($insert)).") VALUES (".implode(",",$insert).")";
	//echo ($sql."\n");
	$db->query($sql);

	return ($user_id);
}

function user_modify($db,$db_common_prefix,$user_id,$fields){
	if ($user_id==NULL){ fatal ("User Modification Fault: No user_id specified"); }
	if (!is_array($fields)){ fatal ("User Modification Fault: fields must be an array"); }
	if ($fields["username"]==NULL){ fatal ("User Modification Fault: No username specified"); }
	if ($fields["firstname"]==NULL){ fatal ("User Modification Fault: No firstname specified"); }
	if ($fields["surname"]==NULL){ fatal ("User Modification Fault: No surname specified"); }
	if ($fields["email"]==NULL){ fatal ("User Modification Fault: No email specified"); }
	if ($fields["access"]==NULL){ fatal ("User Modification Fault: No access level specified"); }
	if ($fields["editor_size"]==NULL){ fatal ("User Modification Fault: No editor_size specified"); }
	
	$update=array();
	$update["username"]="'".$fields["username"]."'";
	$update["firstname"]="'".$fields["firstname"]."'";
	$update["surname"]="'".$fields["surname"]."'";
	$update["email"]="'".$fields["email"]."'";
	$update["access"]=$fields["access"];
	$update["editor_size"]="'".$fields["editor_size"]."'";
	$sql="UPDATE ".$db_common_prefix."users SET ".implode_fields($update)." WHERE user_id=".$user_id;
	//echo ($sql."\n");
	$db->query($sql);
	return (true);
}
/*
function create_user($db,$db_common_prefix,$username,$firstname,$surname,$email,$access){
	if ($db_common_prefix==NULL || $username==NULL || $firstname==NULL || $surname==NULL || $email==NULL || $access==NULL){ return ("User creation failed."); }
	
	$sql="SELECT user_id FROM ".$db_common_prefix."users WHERE UPPER(username)='".strtoupper($username)."'";
	$rs=$db->query($sql);
	if (count($rs)!=0){ return ("User '".$username."' already exists."); }
	
	$map=array();
	$map["username"]="'".$username."'";
	$map["firstname"]="'".$firstname."'";
	$map["surname"]="'".$surname."'";
	$map["email"]="'".$email."'";
	$map["access"]=$access;
	$map["active"]=1;
	$sql="INSERT INTO ".$db_common_prefix."users (".implode(",",array_keys($map)).") VALUES (".implode(",",$map).")";
	$db->query($sql);
	
	return (true);
}
*/

//*********************************************************************************************
// WORKFLOW Functions

//add a group to a workflow, returns true or false
function workflow_add_group($db,$db_table_prefix,$workflow_id,$group_id){
	if ($workflow_id==NULL){ return (false); }
	if ($group_id==NULL){ return (false); }

	$sql="INSERT INTO ".$db_table_prefix."workflow (workflow_id,group_id) VALUES (".$workflow_id.",".$group_id.")";
	//echo ($sql."\n");
	$db->query($sql);	

	return (true);
}


?>
