<?
/*
Stronghold CMS - Resource Class
Copyright (C) 2008 Scott Barnett

This program is free software; you can redistribute it and/or
modify it under the terms of Version 3 of the GNU General Public
License as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
*/

class resource{
	var $db;
	var $db_table_prefix;
	var $new_resource_id; //set when a resource is created
	var $new_version_id; //set when a version is created

	//default constructor
	function resource($db,$db_table_prefix){
		$this->db=$db;
		$this->db_table_prefix=$db_table_prefix;
		return (true);
	}

	//approve a resource, returns true on success
	function approve($resource_id,$version_id,$group_id=NULL,$user_id=NULL){
		if ($resource_id==NULL){ fatal ("Resource Class Fault - No resource id specified for version approval"); }
		if ($version_id==NULL){ fatal ("Resource Class Fault - No version id specified for version approval"); }

		if (php_sapi_name()=="cli" && $user_id==NULL){ fatal ("Resource Class Fault - No user_id specified for command line execution"); } //running from the console, user_id must be specified

		//determine the user_id
		if ($user_id==NULL){ $user_id=$_SESSION[$GLOBALS["CONFIG"]["system_session"]."userid"]; }
		if ($user_id==NULL){ fatal ("Resource Class Fault - No user_id specified"); }

		//retrieve the version of the resource (and workflow)		
		$res=$this->retrieve_version($resource_id,$version_id);
		if (!$res){ fatal ("Resource Class Fault - Could not load resource ".$resource_id.".".$version_id." for approval"); }
		$workflow=retrieve_workflow($this->db,$this->db_table_prefix,$res["workflow_id"]);

		if ($group_id==NULL){ foreach ($_SESSION[$GLOBALS["CONFIG"]["system_session"]."access"] AS $group_id => $value){ /* do nothing */ } } //get the group id (dodgy)

		$now=time();
	
		//if not already approved by this group && if their group is part of the workflow
		if (! in_array($group_id,$res["approvals"]) && in_array($group_id,$workflow)){ 
			$insert=array();
			$insert["resource_id"]=$resource_id;
			$insert["version_id"]=$version_id;
			$insert["group_id"]=$group_id;
			$insert["timestamp"]=$now;
			$insert["user_id"]=$user_id;
			$sql="INSERT INTO ".$this->db_table_prefix."resource_approvals (".implode(",",array_keys($insert)).") VALUES (".implode(",",$insert).")";
			$this->db->query($sql);
			
			//log the approval
			$this->log_edit($user_id,"APPROVE",$resource_id,$version_id);
	
			$res["approvals"][]=$group_id; //add the approval to the array
			
			$approved=true;
			foreach ($workflow as $key => $group_id){ if (! in_array($group_id,$res["approvals"])){ $approved=false; } }
			//pre($page["workflows"]); pre($page["approvals"]); exit();
			
			if ($approved){ //if everyone has approved the resource
				//update the resource
				$sql="UPDATE ".$this->db_table_prefix."resources SET current_version=".$version_id." WHERE resource_id=".$resource_id;
				$this->db->query($sql);
				
				//update the version
				$sql="UPDATE ".$this->db_table_prefix."resource_versions SET status=3 WHERE resource_id=".$resource_id." AND version_id=".$version_id;
				$this->db->query($sql);
				
				//log the event
				$this->log_edit($user_id,"LIVE",$resource_id,$version_id);
			}
			//unlock_module($db,$db_table_prefix,$page["page_id"],$page["version_id"],$user_id);
		}
		return (true);
	}

	//cancel a resource
	function cancel($resource_id,$version_id){
		if ($resource_id==NULL){ fail ("Resource Cancellation Fault: No resource_id specified."); }
		if ($version_id==NULL){ fail ("Resource Cancellation Fault: No version_id specified."); }
	
		$sql="UPDATE ".$GLOBALS["CONFIG"]["db_table_prefix"]."resource_versions SET".
			" status=4".
			" WHERE resource_id=".$resource_id." AND version_id=".$version_id;
		$this->db->query($sql);
		
		return (true);
	}
	
	//common pre flight checks for a resource creation, returns error or true if ok
	function create_preflight($filename,$node_id,&$ext,&$path){
		if ($node_id==NULL){ fatal ("Resource Class Fault - No node id specified for pre flight check"); }
		if ($filename==NULL){ fatal ("Resource Class Fault - No filename specified for pre flight check"); }

		//check it's a file we support
		$ext=file_type($filename);
		if (!$ext){ return("Could not determine file type."); }

		// we don't fault on long filenames, we just trim them :)
		//if (strlen($filename)>50){ return("Filename is too long -> ".$filename); }

		//make sure the parent exists
		$path=$this->node_path($node_id);
		if (!$path){ return("Could not locate the parent node."); }
				
		return (true);
	}
	
	//create a new resource, return true if ok, an error message if it fails
	function create($node_id,$filename,$notes,$method="post"){
		if ($node_id==NULL){ fatal ("Resource Class Fault - No node id specified for create resource"); }
		if ($filename==NULL){ fatal ("Resource Class Fault - No filename specified for create resource"); }
		if (!file_type($filename)){ return ("Could not determine file type -> ".$filename); }
		
		$now=time();

		//the filename is a full directory path in shell mode
		if ($method=="shell"){ $function_filename=substr($filename,strrpos($filename,"/")+1); }
		else { $function_filename=$filename; }
		$function_filename=trim_filename($function_filename,100);
		
		//do the common checks
		$check=$this->create_preflight($function_filename,$node_id,$ext,$path);
		if ($check!==true){ return ($check); }

		//make sure the file doesn't exist in the same folder (only check this for create, of course it exists in new_version... duh)
		$sql="SELECT * FROM ".$this->db_table_prefix."resource_versions AS v, ".$this->db_table_prefix."resources AS r".
			" WHERE v.node_id=".$node_id.
			" AND v.filename='".$function_filename."'".
			" AND r.resource_id=v.resource_id".
			" AND r.current_version=v.version_id".
			" AND v.status!=4";
		$rs=$this->db->query($sql);
		//echo ($sql); echo ptable($rs); exit();
		if (count($rs)!=0){ return ("File already exists -> ".$path."/".$function_filename); }

		//get the resource id
		$resource_id=$this->db->next_val($this->db_table_prefix."resources_resource_id_seq");
		if (!$resource_id){ fatal("Resource Class Fault - Could not locate resource id"); }
		$this->new_resource_id=$resource_id; //cache the new resource id
		
		//create the folder to store the file in
		$folder=$_SERVER["DOCUMENT_ROOT"]."/_datastore/".$resource_id;
		if (is_dir($folder)){ fatal("Resource Class Fault - Cannot create resource folder, it already exists"); }
		if (!mkdir($folder)){ fatal("Resource Class Fault - Resource folder creation failed"); }
		
		//insert the entry in "resources"
		$insert=array();
		$insert["resource_id"]=$resource_id;
		$insert["current_version"]=1;
		//$insert["current_path"]="'".$path."/".$function_filename."'";
		$insert["time_created"]=$now;
		$insert["workflow_id"]=1;
		/* NOTE: When creating, it actually does an INSERT here and then an UPDATE with new_version() */
		$sql="INSERT INTO ".$this->db_table_prefix."resources (".implode(",",array_keys($insert)).") VALUES (".implode(",",$insert).")";
		$this->db->query($sql);
		//echo ($sql); exit();

		//create the new version of the resource
		$result=$this->new_version($resource_id,$node_id,$filename,$notes,$now,$method,$ext,$path);
		
		return ($result);
	}
	
	function log_edit($user_id,$event,$page_id,$version_id){
		if ($user_id==NULL){ fatal("Resource Logging Fault: Missing variable [user_id] - event: ".$event); }
		if ($event==NULL){ fatal("Resource Logging Fault: Missing variable [event] - event: ".$event); }
		if ($page_id==NULL){ fatal("Resource Logging Fault: Missing variable [page_id] - event: ".$event); }
		if ($version_id==NULL){ fatal("Resource Logging Fault: Missing variable [version_id] - event: ".$event); }
		
		$event_id=NULL;
		if ($event=="UPDATE"){ $event_id=1; }
		elseif ($event=="ARCHIVE"){ $event_id=2; }
		elseif ($event=="APPROVE"){ $event_id=3; }
		elseif ($event=="LIVE"){ $event_id=4; }
		elseif ($event=="REV_NOTE"){ $event_id=5; }
		elseif ($event=="SUBMIT"){ $event_id=6; }
		elseif ($event=="CREATE"){ $event_id=7; }
		elseif ($event=="CANCEL"){ $event_id=8; }
		if ($event_id==NULL){ fatal ("Undefined log event."); }
		
		$now=time();
		if ($event_id==4){ $now=$now+1; }//dodgy? make the live event appear after the update event
		$sql="INSERT INTO  ".$this->db_table_prefix."resource_log (resource_id,version_id,user_id,event,timestamp";
		$sql.=") VALUES (";
		$sql.=$page_id.",".$version_id.",".$user_id.",".$event_id.",".$now.");";
		$this->db->query($sql);
	
		return (true);	
	}

	
	//create a new version of an existing resource, returns true on success, an error message if it fails
	function new_version($resource_id,$node_id,$filename,$notes,$time_created,$method,$ext=NULL,$path=NULL){
		if ($resource_id==NULL){ fatal ("Resource Class Fault - No resource id specified for new version"); }
		if ($node_id==NULL){ fatal ("Resource Class Fault - No node id specified for new version"); }
		if ($filename==NULL){ fatal ("Resource Class Fault - No filename specified for new version"); }
		if ($time_created==NULL){ fatal ("Resource Class Fault - No creation time specified for new version"); }
		if ($method==NULL){ fatal ("Resource Class Fault - No method specified for new version"); }

		//the filename is a full directory path in shell mode
		if ($method=="shell"){ $actual_filename=substr($filename,strrpos($filename,"/")+1); } //strip the first slash
		else { $actual_filename=$filename; }
		$actual_filename=trim_filename($actual_filename,100);

		//do the common checks
		if ($ext==NULL || $path==NULL){ //if called from create(), the checks have already been done
			$check=$this->create_preflight($actual_filename,$node_id,$ext,$path);
			if ($check!==true){ return ($check); }	
		}

		//work out the next version id
		$sql="SELECT MAX(version_id) AS max_version_id FROM ".$this->db_table_prefix."resource_versions WHERE resource_id=".$resource_id;
		$rs=$this->db->query($sql);
		$version_id=$rs[0]["max_version_id"]+1;
		if (!$version_id){ fatal ("Resource Manager Fault - Could not find next version_id"); }
		$this->new_version_id=$version_id; //cache the new version id

		//store the file in the datastore
		$this->store_file($resource_id,$version_id,$method,$filename);
		$md5=md5_file($GLOBALS["CONFIG"]["system_datastore"]."/".$resource_id."/".$resource_id."-".$version_id);
		
		//insert the entry in "resource_versions"
		$insert=array();
		$insert["resource_id"]=$resource_id;
		$insert["version_id"]=$version_id;
		$insert["node_id"]="'".$node_id."'";
		$insert["filename"]="'".$actual_filename."'";
		//$insert["path"]="'".$path."'";
		$insert["file_type"]="'".$ext."'";
		$insert["time_created"]=$time_created;
		$insert["status"]=1;
		$insert["md5_sum"]="'".$md5."'";
		$insert["notes"]="'".$notes."'";
		$sql="INSERT INTO ".$this->db_table_prefix."resource_versions (".implode(",",array_keys($insert)).") VALUES (".implode(",",$insert).")";
		//echo ($sql); exit();
		$this->db->query($sql);
		
		//log it
		$this->log_edit($_SESSION[$GLOBALS["CONFIG"]["system_session"]."userid"],"CREATE",$resource_id,$version_id);
		
		//update the parent resource with the new information
		//$sql="UPDATE ".$this->db_table_prefix."resources SET current_version=".$version_id.",current_path='".$path."/".$actual_filename."' WHERE resource_id=".$resource_id;
		$sql="UPDATE ".$this->db_table_prefix."resources SET current_version=".$version_id." WHERE resource_id=".$resource_id;
		$this->db->query($sql);
		
		return (true);
	}
	
	//retrieve all the details of a resource and return the resource or false on failure
	//if $include_path
	function retrieve($resource_id,$include_path=false){
		if ($resource_id==NULL){ return (false); }
		
		//load the header
		//$sql="SELECT current_version,current_path,time_created,workflow_id FROM ".$this->db_table_prefix."resources WHERE resource_id=".$resource_id;
		$sql="SELECT current_version,time_created,workflow_id FROM ".$this->db_table_prefix."resources WHERE resource_id=".$resource_id;
		$rs=$this->db->query($sql);
		if (count($rs)!=1){ return (false); }
		$resource=$rs[0];
		
		//load the versions
		$sql="SELECT v.version_id,v.filename,v.file_type,v.resource_name,v.node_id,v.md5_sum,v.status,v.resource_name,r.ref_val as status_text".
			" FROM ".$this->db_table_prefix."resource_versions AS v,".$GLOBALS["CONFIG"]["db_common_prefix"]."refs AS r".
			" WHERE v.resource_id=".$resource_id.
			" AND r.ref='status'".
			" AND r.ref_id=v.status".
			" ORDER BY v.version_id DESC";
			//echo ($sql);
		$rs=$this->db->query($sql);
		$resource["versions"]=key_array($rs,"version_id");
		
		//load the full resource path for the current version (the stored procedure generates multiple queries, only do it if we "need" it)
		if ($include_path){
			$node=$resource["versions"][$resource["current_version"]]["node_id"];

			//$sql="SELECT tree_path(".$node.",'".$GLOBALS["CONFIG"]["db_table_prefix"]."resource_tree') AS tree_path";
			$sql="SELECT ".tree_path_sql($GLOBALS["CONFIG"]["db_type"],$GLOBALS["CONFIG"]["db_table_prefix"],$node)." AS tree_path";
			$rs=$this->db->query($sql);
			$resource["current_path"]=$rs[0]["tree_path"];
		}

		//load the workflow
		$resource["workflows"]=retrieve_workflow($this->db,$this->db_table_prefix,$resource["workflow_id"]);

		//initialise an array of approvals for each workflow
		foreach (array_keys($resource["versions"]) as $id=> $res_id){ //don't use resource_id
			foreach ($resource["workflows"] as $id => $group_id){ $resource["versions"][$res_id]["approvals"]=array();	}
		}

		//load the approvals for the resource versions
		$sql="SELECT version_id,group_id FROM ".$this->db_table_prefix."resource_approvals WHERE resource_id=".$resource_id." ORDER BY version_id DESC";
		$rs=$this->db->query($sql);
		foreach ($rs as $id => $r){ $resource["versions"][$r["version_id"]]["approvals"][]=$r["group_id"]; }
	
		return ($resource);
	}
	
	//retrieve a resource id, given a resource name and type, returns the resource id on success, false on failure
	function retrieve_resource_id($name,$type){
		if (!$name){ return (false); }
		if (!$type){ return (false); }
		
		$sql="SELECT r.resource_id ".
			" FROM ".$this->db_table_prefix."resources AS r,".$this->db_table_prefix."resource_versions AS v".
			" WHERE r.resource_id=v.resource_id".
			" AND r.current_version=v.version_id ".
			" AND v.file_type='".$type."'".
			" AND v.resource_name='".$name."'";
		$rs=$this->db->query($sql);
		//echo ($sql); echo ptable($rs);
		if (count($rs)!=1){ return (false); }
		
		return ($rs[0]["resource_id"]);
	}
	
	//retrieve a specific version of a resource and return it or false on failure
	function retrieve_version($resource_id,$version_id){
		if ($resource_id==NULL || $version_id==NULL){ return (false); }
		
		$sql="SELECT r.*,v.node_id,v.filename,".tree_path_sql($GLOBALS["CONFIG"]["db_type"],$GLOBALS["CONFIG"]["db_table_prefix"],"v.node_id")." AS path,".
			"v.file_type,v.resource_name,v.time_created,v.md5_sum,v.notes ".
			" FROM ".$this->db_table_prefix."resource_versions AS v, ".$this->db_table_prefix."resources AS r".
			" WHERE v.resource_id=".$resource_id.
			" AND r.resource_id=v.resource_id".
			" AND v.version_id=".$version_id;
		$rs=$this->db->query($sql);
		if (count($rs)!=1){ return (false); }
		$version=$rs[0];
		
		//load the approvals
		$sql="SELECT group_id".
			" FROM ".$this->db_table_prefix."resource_approvals".
			" WHERE resource_id=".$resource_id.
			" AND version_id=".$version_id;
		$rs=$this->db->query($sql);
		$version["approvals"]=array();
		foreach ($rs as $id => $r){ $version["approvals"][]=$r["group_id"]; }
		
		return ($version);
	}
	
	//store a file, return true on success, error message on failure
	function store_file($resource_id,$version_id,$method,$filename=NULL){ //filename only specified for shell imports
		if ($resource_id==NULL){ fatal ("Resource Class Fault - No resource id specified for resource storage"); }
		if ($version_id==NULL){ fatal ("Resource Class Fault - No version id specified for resource storage"); }
		if ($method!="post" && $method!="shell"){ fatal ("Resource Class Fault - Unknown storage method"); }
	
		//work out where to store it
		if (!$GLOBALS["CONFIG"]["system_datastore"]){ fatal ("The [system_datastore] variable has not been set in config.php"); }
		if (!is_dir($GLOBALS["CONFIG"]["system_datastore"])){ fatal ("Could not locate the system datastore folder -> ".$GLOBALS["CONFIG"]["system_datastore"]); }
		$new_resource_path=$GLOBALS["CONFIG"]["system_datastore"]."/".$resource_id."/".$resource_id."-".$version_id;
		if (file_exists($new_resource_path)){ fatal ("Resource Manager Fault - File already exists, ".$new_resource_path); } //this should never ever happen!!!

		if ($method=="shell"){
			//inserted from the shell (don't use fatal errors, just echo and exit)
			if ($filename==NULL){ echo ("FATAL: No filename specified\n"); exit(); }
			
			if (!file_exists($filename)){ echo ("FATAL: Could not locate file (check for bad characters) -> ".$filename."\n"); exit(); }
			//echo ($filename."---".$new_resource_path);
			$result=copy($filename,$new_resource_path);
			if (!file_exists($new_resource_path)){ echo ("FATAL: File storage failed (check your permissions)\n"); exit(); }
			if (filesize($new_resource_path)==0){ echo ("FATAL: File storage failed, zero file size\n"); exit(); }
		} else { //method == post
			//inserted from the resource manager
			$result=move_uploaded_file($_FILES["new_resource"]["tmp_name"],$new_resource_path);
			if ($result==false){ fatal ("Resource Manager Fault - Moving uploaded file for new resource failed"); }
			if (!file_exists($new_resource_path)){ fatal ("Resource Manager Fault - File for new resource unavailable"); }
		}
		return (true);
	}
	
	//update a resource, returns true on success, an error if it fails
	function update($resource_id,$version_id,$node_id,$filename,$resource_name,$notes,$time_created){
		if ($resource_id==NULL){ fatal ("Resource Class Fault - No resource id specified for version update"); }
		if ($version_id==NULL){ fatal ("Resource Class Fault - No version id specified for version update"); }
		if ($node_id==NULL){ fatal ("Resource Class Fault - No node id specified for version update"); }
		if ($filename==NULL){ fatal ("Resource Class Fault - No filename specified for version update"); }
		if ($time_created==NULL){ fatal ("Resource Class Fault - No creation time specified for version update"); }

		//do the common checks
		$check=$this->create_preflight($filename,$node_id,$ext,$path);
		if ($check!==true){ return ($check); }

		//update is meta data only, doesn't update the file, they must create a new version (by design)
				
		//update the entry in "resource_versions"
		$update=array();
		$update["node_id"]="'".$node_id."'";
		$update["filename"]="'".$filename."'";
		$update["resource_name"]=sql_snull($resource_name);
		//$update["path"]="'".$path."'";
		$update["file_type"]="'".$ext."'";
		$update["time_created"]=$time_created;
		$update["notes"]="'".$notes."'";
		$sql_options=array();
		foreach ($update as $key => $val){ $sql_options[]=$key."=".$val; }
		$sql="UPDATE ".$this->db_table_prefix."resource_versions SET ".implode(",",$sql_options)." WHERE resource_id=".$resource_id." AND version_id=".$version_id;
		$this->db->query($sql);
		
		//log it
		$this->log_edit($_SESSION[$GLOBALS["CONFIG"]["system_session"]."userid"],"UPDATE",$resource_id,$version_id);
		
		return (true);
	}
	
	//retrive the node information for a given node
	function retrieve_node($node_id){
		if ($node_id==NULL){ return (false); }
		
		$sql="SELECT node_name,node_parent FROM ".$this->db_table_prefix."resource_tree WHERE node_id=".$node_id;
		$rs=$this->db->query($sql);
		if (count($rs)!=1){ return (false); }

		return ($rs[0]);
	}

	// return the children for a given node
	function child_nodes($node_id){
		if ($node_id==NULL){ return (false); }
		//folders
		$sql="SELECT node_id,node_name FROM ".$this->db_table_prefix."resource_tree WHERE node_parent=".$node_id." ORDER BY node_name ASC";
		$rs=$this->db->query($sql);
		$folders=key_array($rs,"node_id");
		
		//files
		$sql="SELECT v.*".
			" FROM ".$this->db_table_prefix."resources AS r,".$this->db_table_prefix."resource_versions AS v".
			" WHERE r.resource_id=v.resource_id".
			" AND r.current_version=v.version_id".
			" AND v.node_id=".$node_id.
			" AND r.archive_date IS NULL".
			" ORDER BY v.filename ASC";
			//echo ($sql); exit();
		$rs=$this->db->query($sql);
		$files=key_array($rs,"resource_id");		

		return (array("folders"=>$folders,"files"=>$files));
	}
	
	// create a node, returns true on success, an error on fail
	function node_create($name,$parent,&$node_id){
		if ($name==NULL){ return ("Node name not specified"); }
		if ($parent===NULL){ return ("Parent node not specified"); }
	
		$node_id=$this->db->next_val($this->db_table_prefix."resource_tree_node_id_seq");
		
		$insert=array();
		$insert["node_id"]=$node_id;
		$insert["node_name"]="'".$name."'";
		$insert["node_parent"]=$parent;
		$sql="INSERT INTO ".$this->db_table_prefix."resource_tree (".implode(",",array_keys($insert)).") VALUES (".implode(",",$insert).")";
		$this->db->query($sql);
		
		return (true);
	}
	
	//return a select box for the node tree (recursive)
	function node_select($label,$default){
		//load the nodes
		$sql="SELECT node_id as id,node_parent as parent,node_name as title FROM ".$this->db_table_prefix."resource_tree";
		$rs=$this->db->query($sql);
		//echo (ptable($rs));
		//echo ("!!!!".$default."!!!!");
		
		//data objects
		$info=key_array($rs,"id");
		$map=get_children(0,key_array($rs,"id","parent"));		
		//pre($map);
		
		$r="<select name='".$label."'>\n";
		$r.="<option value=''> - Please Select - </option>\n";
		$r.=$this->node_select_options($map,$info,$default);
		$r.="</select>\n";

		return ($r);
	}
	
	//return the node_id for a given path
	function path_node($path){
		$path=explode("/",$path);
		unset ($path[0]); //drop the first token, it's rubbish
		if (count($path)<1){ return (false); }
		
		$parent=0; //the start of the tree
		foreach ($path as $id => $p){
			$sql="SELECT node_id FROM ".$this->db_table_prefix."resource_tree WHERE node_name='".$p."' AND node_parent=".$parent;
			$rs=$this->db->query($sql);
			if (count($rs)!=1){ return (false); }
			$parent=$rs[0]["node_id"];
		}
		
		return ($parent);		
	}
	
	//return the options for a select box, but displaying the node depth (recursive)
	function node_select_options($branch,$info,$default,$path=""){
		$options=NULL;
		//echo ("!!!!".$default."!!!!");
		
		
		foreach ($branch AS $id => $leaf){
			$branch_path=$path."/".$info[$id]["title"]; //add the folder to the path
		//echo ($branch_path."<br>\n");

			if ($id==$default){ $selected=" selected"; } else { $selected=NULL; } //highlight default fields
			$options.="<option value='".$id."'".$selected.">".$branch_path."</option>\n"; //create the option line
			$options.=$this->node_select_options($leaf,$info,$default,$branch_path); //add the options for the children

		}
		return ($options);
	}
	
	function path_implode($arr){
		$path="";
		foreach ($arr as $id=> $vals){ $path.="/".$vals["node_name"]; }
		return ($path);
	}	
	
	//return the path to a node given a list of parents, false if it fails
	function node_path($node_id){
		if ($node_id==NULL){ return (false); }
		
		$links=$this->parent_nodes($node_id);
		$path=$this->path_implode($links);
		return ($path);
	}
	
	//return the parents for a given node
	function parent_nodes($node_id){
		if ($node_id==NULL){ return (false); }
		
		$links=array();
		
		$i=0;
		while ($node_id!=0 && $i<50){ //$i<50 prevents infinte looping
			$sql="SELECT node_parent,node_name FROM ".$this->db_table_prefix."resource_tree WHERE node_id=".$node_id;
			$rs=$this->db->query($sql);
			if (count($rs)!=1){ fatal("Resource Tree Fault - Could not locate parent node."); }
			
			$node_parent=$rs[0]["node_parent"];
			$node_name=$rs[0]["node_name"];
			
			$links[]=array("node_id"=>$node_id,"node_name"=>$node_name);
			$node_id=$node_parent;
			$i++;
		}
		if ($i==50){ fatal("Could not locate parent node"); }
		
		$links=array_reverse($links);
		return ($links);	
	}
}

// retrieve the list of elements to go in the tree menu
function retrieve_resource_tree($db,$db_table_prefix){
	$tree_data=array();
	$node_map=array("0"=>NULL);

	//load the folders
	$sql="SELECT node_id,node_name,node_parent FROM ".$db_table_prefix."resource_tree ORDER BY node_name ASC";
	$rs=$db->query($sql);
	foreach ($rs as $id => $r){
		//$node_label="n".$r["node_id"];
		$node_label=$r["node_id"];
		//$tree_data[$node_label]=array("title"=>$r["node_name"],"parent"=>"n".$r["node_parent"]);
		$tree_data[$node_label]=array("title"=>$r["node_name"],"parent"=>$r["node_parent"]);
		//$node_map[$node_label]="n".$r["node_parent"];
		$node_map[$node_label]=$r["node_parent"];
	}
	//pre($node_map);
	//load the files (we don't load files, it makes the tree menu too large)
	/*
	$sql="SELECT r.resource_id, v.version_id, v.filename, v.node_id".
		" FROM ".$db_table_prefix."resources AS r, ".$db_table_prefix."resource_versions AS v".
		" WHERE r.current_version=v.version_id".
		" AND r.resource_id=v.resource_id";
	$rs=$db->query($sql);
	foreach ($rs as $id => $r){
		$node_label="r".$r["resource_id"];
		$tree_data[$node_label]=array("title"=>$r["filename"],"parent"=>"n".$r["node_id"]);
		$node_map[$node_label]="n".$r["node_id"];
	}
	*/
		
	//pre($node_map); exit();
	$tree=get_children("0",$node_map); //build the tree
	//pre($tree); exit();
	
	return (array("resources"=>$tree_data,"tree"=>$tree));
}

//trim filenames longer than a specific length
function trim_filename($filename,$filename_length){
	if (strlen($filename)<$filename_length){ return ($filename); }
	
	$bits=explode(".",$filename);
	$ext=array_pop($bits); //get the extension off
	$filename=substr(implode(".",$bits),0,($filename_length-4)); //trim the filename

	return ($filename.".".$ext);
}


?>
