<?
/*
Stronghold CMS - Database Schema Maintainer
Copyright (C) 2008 Scott Barnett

This program is free software; you can redistribute it and/or
modify it under the terms of Version 3 of the GNU General Public
License as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

----------------------------------------------------------------

$options=array(



	test
	drop_unknown_tables
	drop_unknown_constraints
)
*/

function install_schema($db,$txt_tables,$txt_constraints,$txt_functions,$options){
	//error checking
	if (!$db){ echo ("FATAL: Install Schema Fault - no database connector initiated\n"); exit(); }
	if (!is_array($txt_tables)){ echo ("FATAL: Install Schema Fault - txt_tables is not an array\n"); exit(); }
	if (!is_array($txt_constraints)){ echo ("FATAL: Install Schema Fault - txt_constraints is not an array\n"); exit(); }
	if (!is_array($options)){ echo ("FATAL: Install Schema Fault - options is not an array\n "); exit(); }
	//echo ("<pre>");
	
	//print_r($txt_tables);
	//print_r($txt_constraints);
	//print_r($stored_procedures);
	
	//load the information about the current database
	$db_constraints=load_constraints($db); //print_r($db_constraints);
	$db_tables=load_tables($db); //print_r($db_tables); exit();
	$db_functions=load_functions($db); //print_r($db_functions); exit();

	$sqls=array();
	
	// check for unknown tables
	foreach ($db_tables as $db_table_name => $db_table_attr){ //for each actual table
		if (!array_key_exists($db_table_name,$txt_tables)){
			echo ("ALERT: Unknown table ".$db_table_name."\n");
			if ($options["drop_unknown_tables"]){ $sqls[]="DROP TABLE ".$db_table_name; }
		}
	}

	// check for tables and problems with tables
	foreach ($txt_tables as $txt_table_name => $txt_table_attr){ //for each schema table
		$db_table_attr=$db_tables[$txt_table_name];

		if (count($db_table_attr)>1){
	
			//check for missing attributes
			foreach ($txt_table_attr as $txt_attr_title => $txt_attr_properties){
				$db_attr_properties=$db_table_attr[$txt_attr_title];
	
				if (array_key_exists($txt_attr_title,$db_table_attr)){
					foreach ($txt_attr_properties as $txt_attr_property_title => $txt_attr_property_value){
						$db_attr_property_value=strtoupper(trim($db_attr_properties[$txt_attr_property_title])); //sql server returns a space on "no " nullable fields
					
						if (array_key_exists($txt_attr_property_title,$db_attr_properties)){
							if ($db_attr_property_value==$txt_attr_property_value){ //compare the attribute values
								/* perfect! */
							} else {
								if ($db_attr_property_value!="2147483647" && ($db_attr_property_value!="INT" && $txt_attr_property_value!="SERIAL")){ //ignore serial fields and mssql's limit on text fields
									echo ("ALERT: Table [".$txt_table_name."] has attribute [".$txt_attr_title."] with property [".$txt_attr_property_title."=>".$db_attr_property_value."] which should be [".$txt_attr_property_title."=>".$txt_attr_property_value."]\n");
								}

								if ($txt_attr_property_title=="is_nullable"){
									if ($txt_attr_property_value=="YES"){ $sqls[]="ALTER TABLE ".$txt_table_name." ALTER COLUMN ".$txt_attr_title." DROP NOT NULL"; }
									else { $sqls[]="ALTER TABLE ".$txt_table_name." ALTER COLUMN ".$txt_attr_title." SET NOT NULL"; }
								}
							}
						}
					}
				} else {
					// attribute is missing from the table
					$sqls[]="ALTER TABLE ".$txt_table_name." ADD ".$txt_attr_title." ".datatype($options["database_type"],$txt_attr_properties).";";
				}
			}
			
			//check for extra attributes
		} else {
			//create the table if it doesn't exist
			$sqls[]=sql_create_table($options["database_type"],$txt_table_name,$txt_table_attr);
		}
	}
	//print_r($sqls); exit();
	
	//-------------------------------------------------------------------------------------------------------------
	
	
	// check for unknown constraints (drop them)
	$db_constraints_temp=array();
	foreach ($db_constraints as $db_constraint_name => $db_constraint_values){
		if (!array_key_exists($db_constraint_name,$txt_constraints)){ //we can skip named constraints because we can identify them immediately			
			$unknown=true;
			if ($db_constraint_values["constraint_type"]=="PRIMARY KEY"){ 
				if (array_key_exists($db_constraint_values["table_name"]."_pkey",$txt_constraints)){
					$unknown=false;
					//this is a bit of a kludge, we're going to add the constraint back into the array with the correct
					//name to make the logic for when we check for known constraints much much easier but we can't do
					//it without breaking the "for" loop, so we drop them in a temporary array and merge them when we're done
					$db_constraints_temp[$db_constraint_values["table_name"]."_pkey"]=$db_constraint_values;
				}
			}
			
			if ($unknown){
				echo ("ALERT: Unknown constraint ".$db_constraint_name."\n");
				if ($options["drop_unknown_constraints"]){ $sqls[]="ALTER TABLE ".$db_constraint_values["table_name"]." DROP CONSTRAINT ".$db_constraint_name; }
			}
		}
	}
	$db_constraints=array_merge($db_constraints,$db_constraints_temp);
	
	// check for known constraints (add them)
	foreach ($txt_constraints as $txt_constraint_name => $txt_constraint_values){
		if (!array_key_exists($txt_constraint_name,$db_constraints)){
			$sqls[]=sql_create_constraint($txt_constraint_name,$txt_constraint_values);
		}
	}
	
	//add the functions
	foreach ($txt_functions as $label => $sql){
		//functions don't contain data, we drop and recreate them on every run so they're upgraded automatically
		
		if ($options["database_type"]=="mssql"){
			//mssql needs a function for each system cause it doesn't support variable table names (sigh)
			foreach ($options["prefixes"] as $id => $prefix){
			
				$label=$prefix.$label;
				if (in_array($label,$db_functions)){ $sqls[]="DROP FUNCTION ".$label; }
				$sqls[]=str_replace("{db_table_prefix}",$prefix,$sql);
			}
		} else {
			//everything else is just raw SQL
			$sqls[]=$sql;
		}
	}
	
	echo ("\n\n");
	//pre($sqls);
	foreach ($sqls as $id => $sql){
		
		if ($options["test"]==true || $options["test"]===NULL){
			echo ("TEST SQL: ".$sql);	
		} else {
			echo ("EXECUTING SQL: ".$sql);
			$db->query($sql);
			echo (" DONE");
		}
		echo ("\n");
	}
	


}

//**************************************************************************************************************
// Utility functions

// populate tables with data
function populate_tables($db,$populate,$test=true){
	if (!is_array($populate)){ echo ("FATAL: Populate array not specified for populate_tables\n"); exit(); }
	
	foreach ($populate as $table => $vals){
		foreach ($vals as $id => $insert){
			$sql="INSERT INTO ".$table." (".implode(",",array_keys($insert)).") VALUES (".implode(",",$insert).");";
			if ($test){
				echo ("TEST SQL: ".$sql."\n");
			} else {
				echo ("EXECUTING SQL: ".$sql."\n");
				$db->query($sql);
			}
		}
	}
}


//return the datatype SQL String
function datatype($database_type,$properties){
	if ($database_type==NULL){ echo ("FATAL: No database specified for dataytype()\n"); exit(); }
	if (!is_array($properties)){ return (false); }
	
	//primary key (serial) datatypes
	if ($properties["data_type"]=="SERIAL"){
		//mssql and mysql don't support serials
		if ($database_type=="postgres"){ return ("SERIAL"); }
		elseif ($database_type=="oracle"){ return ("SERIAL"); }
		elseif ($database_type=="mssql"){ return ("INT"); }
		elseif ($database_type=="mysql"){ return ("INT"); }
		else { echo ("FATAL: Unknown database type in datatype()\n"); exit(); }
	}

	//set the nulls bit
	if ($properties["is_nullable"]=="NO"){ $nulls=" NOT NULL"; } else { $nulls=" NULL"; }

	//integer data type
	if ($properties["data_type"]=="INT"){ return ("INT".$nulls); }

	//smallint data type	
	if ($properties["data_type"]=="SMALLINT"){ return ("SMALLINT".$nulls); }
	
	//text data type
	if ($properties["data_type"]=="TEXT"){
		$str="TEXT";
		if ($properties["character_maximum_length"]){ $str.=" (".$properties["character_maximum_length"].")"; }
		return ($str.$nulls);
	}

	//char data type
	if ($properties["data_type"]=="CHAR"){
		$str="CHAR";
		if ($properties["character_maximum_length"]){ $str.=" (".$properties["character_maximum_length"].")"; }
		return ($str.$nulls);
	}
	
	//character varying data type
	if ($properties["data_type"]=="VARCHAR"){
		$str="VARCHAR";
		if ($properties["character_maximum_length"]){ $str.=" (".$properties["character_maximum_length"].")"; }
		return ($str.$nulls);
	}

	return (NULL);
}


function load_constraints($db){

	$sql="SELECT c.constraint_name,c.constraint_type,u.table_name,u.column_name,u.constraint_name,u.table_schema
	FROM information_schema.constraint_column_usage AS u, information_schema.table_constraints AS c
	WHERE u.constraint_name=c.constraint_name
	AND u.table_schema!='dbo'
	AND u.table_schema!='public'
	ORDER BY u.constraint_name ASC";
	$rs=$db->query($sql);
	//print_r($rs); exit();
	
	//create the constraints array
	$db_constraints=array();
	foreach ($rs as $id => $r){
		$constraint_name=$r["constraint_name"];
		if (!array_key_exists($constraint_name,$db_constraints)){
			$db_constraints[$constraint_name]=array(
				"constraint_type"=>$r["constraint_type"],
				"table_name"=>$r["table_name"]
			);
		}
		$db_constraints[$constraint_name]["columns"][]=$r["column_name"];
	}

	return ($db_constraints);
}

//return a list of functions
function load_functions($db){
	$sql="SELECT routine_name FROM information_schema.routines
	WHERE routine_type='FUNCTION'
	AND specific_schema!='pg_catalog'
	AND specific_schema!='information_schema'";
	$rs=$db->query($sql);
	$functions=array();
	foreach ($rs as $id => $r){ $functions[]=$r["routine_name"]; }
	return ($functions);
}


function load_tables($db){
	// COLUMNS
	$sql="SELECT table_name,column_name,is_nullable,data_type,character_maximum_length FROM information_schema.columns WHERE table_schema!='dbo'"; //WHERE table_schema='public'
	$rs=$db->query($sql);
	//print_r($rs); exit();

	//create the schema array
	$db_tables=array();
	foreach($rs as $id => $r){
		$table_name=$r["table_name"];
		$column_name=$r["column_name"];
		unset($r["table_name"]);
		unset($r["column_name"]);
		$db_tables[$table_name][$column_name]=$r;
	}
	return ($db_tables);
}

//pre($schema);

//return the SQL required to add a constraint
function sql_create_constraint($name,$values){
	$sql=NULL;
	if ($values["constraint_type"]=="PRIMARY"){ $sql="ALTER TABLE ".$values["table_name"]." ADD PRIMARY KEY (".implode(",",$values["columns"]).")";	}
	elseif ($values["constraint_type"]=="UNIQUE"){ $sql="ALTER TABLE ".$values["table_name"]." ADD CONSTRAINT ".$name." UNIQUE (".implode(",",$values["columns"]).")"; }
	else { echo  ("ALERT: Unkown constraint type -> ".$values["constraint_type"]); }
	return ($sql);
}

//create a table given a database type and the fields
function sql_create_table($database_type,$name,$attr){
	$sql="CREATE TABLE ".$name." (";
	$bits=array();
	foreach ($attr as $attr_name => $properties ){ $bits[]=$attr_name." ".datatype($database_type,$properties); }
	$sql.=implode(",",$bits).")";

	return ($sql);
}

?>