<?
/*
Stronghold CMS - System Core
Copyright (C) 2008 Scott Barnett

This program is free software; you can redistribute it and/or
modify it under the terms of Version 3 of the GNU General Public
License as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
*/

if (php_sapi_name()!="cli"){
	// stronghold libraries
	include ($_SERVER["DOCUMENT_ROOT"]."/_system/error.php"); //error handler
	include ($_SERVER["DOCUMENT_ROOT"]."/config.php"); //configuration information
	include ($_SERVER["DOCUMENT_ROOT"]."/_system/database.php"); //database connector
	
	// PEAR mail libraries
	include ("Mail.php");
	include ("Mail/mime.php");
	include ("Text/Diff.php");
	
	//boot people out if the system isn't online
	if ($CONFIG["system_state"]!=1){
		$allowed=false;
		if (is_array($CONFIG["system_allow"])){
			if (in_array($_SERVER["REMOTE_ADDR"],$CONFIG["system_allow"])){ $allowed=true; } //allow ranges that are specified
		}
	
		if (!$allowed){
			if ($CONFIG["system_state"]==0){ readfile ($_SERVER["DOCUMENT_ROOT"]."/_error/offline.htm"); exit(); }
			if ($CONFIG["system_state"]==2){ readfile ($_SERVER["DOCUMENT_ROOT"]."/_error/comingsoon.htm"); exit(); }
		}
	}

	//database connection
	if ($CONFIG["db_type"]=="postgres"){ $db=new pg($CONFIG["db_server"],$CONFIG["db_database"],$CONFIG["db_username"],$CONFIG["db_password"]); }
	elseif ($CONFIG["db_type"]=="mysql"){ $db=new my($CONFIG["db_server"],$CONFIG["db_database"],$CONFIG["db_username"],$CONFIG["db_password"]); }
	elseif ($CONFIG["db_type"]=="mssql"){ $db=new ms($CONFIG["db_server"],$CONFIG["db_database"],$CONFIG["db_username"],$CONFIG["db_password"]); }
	elseif ($CONFIG["db_type"]=="oracle"){ $db=new ora($CONFIG["db_server"],$CONFIG["db_database"],$CONFIG["db_username"],$CONFIG["db_password"]); }
	else { fatal ("Stronghold System Fault: Unknown db_type"); }
}

//**********************************************************************************************************************

function load_page($db,$url){
	global $CONFIG; //dodgy
	
	$db_table_prefix=$GLOBALS["CONFIG"]["db_table_prefix"];
	$page=array();
	
	// ./index hack <- might be a better way to accomplish this?
	if ($url=="/index"){
		$url="/";
		$page["deindexed"]=true;
	} else {
		if (substr($url,strlen($url)-6,6)=="/index"){ //if the end of the string is /index
			$url=substr($url,0,strlen($url)-6); //ditch the "/index"
			$page["deindexed"]=true;
		}
	}

	// if they're an admin
	if (is_array($_SESSION[$GLOBALS["CONFIG"]["system_session"]."access"])){ $admin=true; } else { $admin=false; }
	
	// load the page
	$loaded_page=retrieve_page($db,$db_table_prefix,$url);
	if (!$loaded_page){ return (404); } //return 404 if it doesn't exist

	//if they're not an admin, don't let them see archived pages
	if (!$admin && ($loaded_page["archive_date"]!=NULL)){ return (404); } else { unset($loaded_page["archive_date"]); }

	// load some extended information (versions, url, navigation)
	$page=array_merge($page,$loaded_page);
	$page["url"]=$url;
	$page["versions"]=retrieve_page_versions($db,$db_table_prefix,$page["module_name"],$page["page_id"]);
	$page["navigation"]=retrieve_flat_nav($db,$db_table_prefix,$admin,$page["page_id"]);
	//pre($page); exit();
	
	// does it have children? (./index hack)
	if (count($page["navigation"])>0 && !$page["deindexed"]){ return (301); }

	//--------------------------------------------------------------------------------------------------
	//work out what version of the page to give them
	if ($admin){ //if they're an admin
		// AN ADMIN - whatever version they ask for, default of the highest version
		$page["version_id"]=prepare_int($_REQUEST["version_id"]); 
		if ($page["version_id"]==NULL){
			$first_version=NULL;
			foreach ($page["versions"] as $version_id => $values){ if ($first_version==NULL){ $first_version=$version_id; } } //find the first element in the array
			if ($first_version==NULL){ fatal ("Could not load document, no versions for page."); }
			$page["version_id"]=$first_version;
		}
	} else {
		// A USER - give them latest version of approved content that hasn't been archived and has an active publishing date
		$page["version_id"]=user_version($page["versions"]);
	}
	if ($page["version_id"]==NULL){ return (404); } //return a 404 if there's no version available

	//--------------------------------------------------------------------------------------------------
	// display the page
	if ($page["version_id"]){
		// breadcrumb trail
		$page["parents"]=parents($db,$page["parent"]);

		// workflow information
		$page["workflows"]=retrieve_workflow($db,$db_table_prefix,$page["workflow_id"]);
		$page["approvals"]=retrieve_page_approvals($db,$db_table_prefix,$page["page_id"],$page["version_id"]);

		// load the module
		include ($_SERVER["DOCUMENT_ROOT"]."/_modules/".$page["module_name"].".php");
		$module=load_module($db,$page);
		
		return(array("page"=>$page,"module"=>$module));
	}

	return(false);
}

//given an array of page versions, return the highest version a user can access
function user_version($versions){
	$now=time();

	foreach ($versions as $version_id => $values){
		//the page for the user needs to meet several criteria
		if ($values["status"]==3){
			$valid=true;
			if ($values["publish_live"]!=NULL){
				if (($values["publish_live"]>$now)){ $valid=false; }
			}
			if ($values["publish_end"]!=NULL){
				if (($values["publish_end"]<$now)){ $valid=false; }
			}
			if ($valid){ return ($version_id); }
		}
	}
	return (false);
}


function parents($db,$page_id){
	global $CONFIG;

	$links=array();
	$stop=false;
	while (! $stop && ($page_id!=NULL)){
		$sql="SELECT parent,link ".
			" FROM ".$GLOBALS["CONFIG"]["db_table_prefix"]."pages ".
			" WHERE page_id=".$page_id;
		//echo $sql."<br>\n";
		$rs=$db->query($sql);
		$parent=$rs[0]["parent"];
		$link=$rs[0]["link"];
		
		if ($parent==$page_id){ $stop=true; } else { $page_id=$parent; }
		$links[]=array("page_id"=>$page_id,"link"=>$link);
		
	}
	$links=array_reverse($links);
	return ($links);
}

//********************************************************************************************
// PAGE functions

// view a specified page
function view_page($db,$page,$module){
	view_module($db,$page,$module);
}

// the EDIT view of a module
function edit_page($db,$page,$module){
	//pre($page);
	//pre($module);
	$db_table_prefix=$GLOBALS["CONFIG"]["db_table_prefix"];
	$db_common_prefix=$GLOBALS["CONFIG"]["db_common_prefix"];
	$user_id=$_SESSION[$GLOBALS["CONFIG"]["system_session"]."userid"];
	
	if (prepare_int($_GET["newversion"])){
		//calculate the highest version of the page, use the next available key
		$max=1;
		foreach ($page["versions"] AS $key => $value){ if ($key>$max){ $max=$key; } }
		$max++;
		$now=time();

		$new_module=array();
		$new_module["page_id"]=$page["page_id"];
		$new_module["version_id"]=$max;
		$new_module["title"]=$module["title"];
		$new_module["short_title"]=$module["short_title"];
		$new_module["body_text"]=$module["body_text"];
		$new_module["status"]=1;
		$new_module["created_date"]=$now;
		$new_module["nav_id"]=$module["nav_id"];
		$new_module["heading"]=$module["heading"];
		$new_module["mod_time"]=$now;
		$new_module["mod_userid"]=$user_id;
		//pre($new_module);
			
		$result=revise_module($db,$db_table_prefix,$new_module); // create a new module
		if ($result!==true){ fatal("Module revision failed - ".$result); }
		header ("Location: ".$page["url"]."?mode=edit&version_id=".$max); exit();
	}

	if (prepare_int($_GET["cancel"])==1){
		$result=cancel_module($db,$page,$module); // cancel an existing module
		if ($result){ log_edit($db,$db_table_prefix,$user_id,"CANCEL",$page["page_id"],$module["version_id"]); }
		header ("Location: ".$page["url"]); exit();
	}
	
	$lock=lock_module($db,$db_table_prefix,$page["page_id"],$module["version_id"],$user_id);
	if ($lock!==true){
		$user=retrieve_user($db,$db_common_prefix,"user_id",$lock);
		$b="<p>Page ".$page["page_id"].".".$module["version_id"]." is currently being edited by ".$user["firstname"]." ".$user["surname"].".</p>";
		$b.="<p>Please try again later.</p>";
		$b.="<input type='button' class='mb' value='Return to Page ".$page["page_id"].".".$module["version_id"]."' onclick='window.location=\"".$page["url"]."\"'><br>\n";
		
		$document["title"]="Version Locked";
		$document["body"]=$b;
		$document["mod_time"]=time();
		include ($_SERVER["DOCUMENT_ROOT"]."/includes/template.php");
		exit();
	}
	edit_module($db,$page,$module);
}

function post_page($db,$page,$module){ post_module($db,$page,$module); }

//create a page (and module), and return an error message if it fails
function create_page($db,$db_table_prefix,$user_id,&$page){
	//check for errors
	//we don't use field_check because we don't know what some of the variables are going to be (they're just passed on to the module)
	if ($user_id==NULL){ return ("Page Creation Fault: No user_id specified"); }
	if (!is_array($page)){ return ("Page Creation Fault: No page array specified"); }
	if ($page["parent"]===NULL){ return ("Page Creation Fault: No parent specified"); }
	
	if (!$page["time_created"]){ return ("Page Creation Fault: No time created specified"); }
	if (!$page["workflow_id"]){ return ("Page Creation Fault: No workflow_id specified"); }
	if (!$page["link"]){ return ("Page Creation Fault: No link specified"); }
	if (!$page["module_id"]){ return ("Page Creation Fault: No module_id specified"); }
	//if (!$page["archive_date"]){ return ("Page Creation Fault: No module_id specified"); }
	if (!$page["title"]){ return ("Page Creation Fault: No title specified"); }
	if (!$page["short_title"]){ return ("Page Creation Fault: No short_title specified"); }

	$now=time();
	
	if ($page["parent"]==0 && php_sapi_name()=="cli"){
		//only allow the cli to create the home page
		$page["url"]="/";
	} else {
		//check for a parent, figure out the URL
		$sql="SELECT url FROM ".$db_table_prefix."pages WHERE page_id=".$page["parent"];
		$rs=$db->query($sql);
		if (count($rs)!=1){ return ("Could not locate parent page."); }
		if ($rs[0]["url"]=="/"){ $page["url"]="/".$page["link"]; } //don't double up on the / with the home page
		else { $page["url"]=$rs[0]["url"]."/".$page["link"]; }
		if ($page["url"]==NULL){ return ("Page Creation Fault: Problems calculating the URL."); }
	}
	
	//see if the page exists (disregard archived pages)
	$sql="SELECT page_id FROM ".$db_table_prefix."pages WHERE archive_date IS NULL AND url='".$page["url"]."'";
	$rs=$db->query($sql);
	if (count($rs)>0){ return ("Page for url '".$page["url"]."' already exists."); }

	//request a new key for the page (atomic insert)
	$page["page_id"]=$db->next_val($db_table_prefix."pages_page_id_seq");
	if (!$page["page_id"]){ fatal ("Page Creation Fault: Could not obtain a page_id"); }

	//insert the page	
	$insert=array();
	$insert["page_id"]=$page["page_id"];
	$insert["time_created"]=$page["time_created"];
	$insert["parent"]=$page["parent"];
	$insert["workflow_id"]=$page["workflow_id"];
	$insert["link"]="'".$page["link"]."'";
	$insert["url"]="'".$page["url"]."'";
	$insert["module_id"]=$page["module_id"];
	$insert["archive_date"]=sql_null($page["archive_date"]);
	$insert["levels"]=url_levels($page["url"]);
	$sql="INSERT INTO ".$db_table_prefix."pages (".implode(",",array_keys($insert)).") VALUES (".implode(",",$insert).")";
	//echo ($sql."\n");
	$rs=$db->query($sql);
	
	//define default values
	$page["version_id"]=1;
	
	//create and the module header and use default values
	$header=array();
	$header["page_id"]=$page["page_id"];
	$header["version_id"]=$page["version_id"];
	$header["title"]=$page["title"];
	$header["short_title"]=$page["short_title"];
	$header["heading"]=1;
	$header["status"]=1;
	$header["created_date"]=$now;
	$header["nav_id"]=1;
	$header["mod_time"]=$now;
	$header["mod_userid"]=$user_id;	
	$result=create_module_header($db,$db_table_prefix,$user_id,$header);
	
	if ($result!==true){ //if module creation fails, roll back the page creation
		$sql="DELETE FROM ".$db_table_prefix."pages WHERE page_id=".$header["page_id"];
		$db->query($sql);
		return ($result);
	} else {
		//create the module instance
		$result=create_module($db,$db_table_prefix,$user_id,$page);
		
		if ($result!==true){ //if module creation fails, roll back the module creation and the page creation
			$sql="DELETE FROM ".$db_table_prefix."module WHERE page_id=".$header["page_id"]." AND version_id=1";
			$db->query($sql);
			$sql="DELETE FROM ".$db_table_prefix."pages WHERE page_id=".$header["page_id"];
			$db->query($sql);
			return ($result);		
		}
	}
	
	return (true);
}

//********************************************************************************************
// MODULE HEADER Functions

function create_module_header($db,$db_table_prefix,$user_id,$module){
	if ($user_id==NULL){ return ("Module Header Fault: No user_id specified."); }
	if (!is_array($module)){ return ("Module Header Fault: No module array specified"); }

	//check the array for errors
	$compulsory=array("page_id","version_id","title","short_title","heading","status","created_date","nav_id","mod_time","mod_userid");
	$optional=array("publish_live","publish_end");
	$not_null=array("page_id","version_id","title","short_title","status","created_date","nav_id","mod_time","mod_userid");
	$result=field_check($module,$compulsory,$optional,$not_null,"Module Header Fault");
	if ($result!==true){ return ($result); }

	//SQLize the variables	
	$module["title"]="'".$module["title"]."'"; //string
	$module["heading"]="'".$module["heading"]."'";
	$module["nav_id"]=$module["nav_id"];
	$module["short_title"]="'".$module["short_title"]."'"; //string
	//$module["notes"]=sql_snull($module["notes"]); //string

	//create the module header
	$sql="INSERT INTO ".$db_table_prefix."module (".implode(",",array_keys($module)).") VALUES (".implode(",",$module).")";
	//echo ($sql); exit();
	$db->query($sql);
	
	//log the edit
	log_edit($db,$db_table_prefix,$user_id,"CREATE",$module["page_id"],$module["version_id"]);
	return (true);
}

//insert into the module header table, return true if ok, error if it fails
function update_module_header($db,$db_table_prefix,$page_id,$version_id,$module){
	$db_common_prefix=$GLOBALS["CONFIG"]["db_common_prefix"];
	$user_id=$_SESSION[$GLOBALS["CONFIG"]["system_session"]."userid"];

	//pre($module); exit();
	if ($page_id==NULL){ return ("Module Header Fault: No page_id specified"); }
	if ($version_id==NULL){ return ("Module Header Fault: No version_id specified"); }
	if (!is_array($module)){ return ("Module Header Fault: No module array specified"); }

	//check the array for errors
	$compulsory=array("title","short_title","heading","nav_id","mod_time","mod_userid");
	$optional=array("status","publish_live","publish_end");
	$not_null=array("title","short_title","heading","nav_id","mod_time","mod_userid");
	$result=field_check($module,$compulsory,$optional,$not_null,"Module Header Fault");
	if ($result!==true){ return ($result); }

	//process approvals
	if ($_POST["approve"]=="Submit for Approval"){ //direct comparison, no need for prepare_str
		$module["status"]=2;
		log_edit($db,$db_table_prefix,$user_id,"SUBMIT",$page_id,$version_id);

		//notify all the officers that an approval has happened
		$sql="SELECT u.email".
			" FROM ".$db_table_prefix."pages AS p, ".$db_table_prefix."workflow AS w, ".$db_table_prefix."group_members as g, ".$db_common_prefix."users AS u".
			" WHERE p.page_id=".$page_id.
			" AND p.workflow_id=w.workflow_id".
			" AND w.group_id=g.group_id".
			" AND g.user_id=u.user_id";
		$rs=$db->query($sql);
		$emails=array();
		foreach ($rs as $id => $r){ $emails[]=$r["email"]; }
		notify_officers($db,$emails,"SUBMIT",$page_id,$version_id);
		//pre($module); exit();
	}

	//SQLize some of the variables
	$module["title"]="'".$module["title"]."'";
	$module["short_title"]="'".$module["short_title"]."'";
	$module["publish_live"]=sql_null($module["publish_live"]);
	$module["publish_end"]=sql_null($module["publish_end"]);

	//update the module header
	$sql="UPDATE ".$db_table_prefix."module SET ";
	$options=array();
	foreach ($module as $key => $value){ $options[]=$key."=".$value; }
	$sql.=implode(",",$options);
	$sql.=" WHERE page_id=".$page_id." AND version_id=".$version_id;
	//echo ($sql); exit();
	$db->query($sql);
	
	delete_approvals($db,$db_table_prefix,$page_id,$version_id);
	
	return (true);
}


function delete_approvals($db,$db_table_prefix,$page_id,$version_id){
	$db_common_prefix=$GLOBALS["CONFIG"]["db_common_prefix"];

	//load the approvals for the given page
	$sql="SELECT g.group_id,u.email"
		." FROM ".$db_table_prefix."approvals AS a,".$db_table_prefix."group_members AS g,".$db_common_prefix."users AS u"
		." WHERE a.page_id=".$page_id." AND a.version_id=".$version_id
		." AND a.group_id=g.group_id"
		." AND g.user_id=u.user_id";
	$rs=$db->query($sql);

	if (count($rs)!=0){
		// email each person in a group that has approved it
		$emails=array();
		foreach ($rs as $id => $r){ $emails[]=$r["email"]; }
		notify_officers($db,$emails,"REVERT",$page_id,$version_id);
		
		// delete all approvals
		$sql="DELETE FROM ".$db_table_prefix."approvals WHERE page_id=".$page_id." AND version_id=".$version_id;
		$db->query($sql);
	}
}


//************************************************************************************************************************************
//******************************************* WORKFLOW functions *********************************************************************
//************************************************************************************************************************************

//return the contents of a workflow for a given workflow id
function retrieve_workflow($db,$db_table_prefix,$workflow_id){
	if ($workflow_id==NULL ){ fatal ("Workflow Retrieval Fault: No workflow_id specified"); }
	
	$sql="SELECT group_id FROM ".$db_table_prefix."workflow WHERE workflow_id=".$workflow_id;
	$rs=$db->query($sql);
	$workflows=array();
	foreach ($rs as $id => $r){ $workflows[]=$r["group_id"]; }
	return ($workflows);
}

//return an array of all the system workflows
function retrieve_workflows($db,$db_table_prefix){
	//$sql="SELECT w.workflow_id,w.group_id,g.title FROM ".$db_table_prefix."workflow AS w,".$db_table_prefix."groups AS g WHERE w.group_id=g.group_id";
	$sql="SELECT workflow_id,group_id FROM ".$db_table_prefix."workflow";
	$rs=$db->query($sql);
	
	$workflows=array();
	foreach ($rs as $id => $r){ $workflows[$r["workflow_id"]][]=$r["group_id"]; }	

	return ($workflows);
}

//return the approvals for a given page version
function retrieve_page_approvals($db,$db_table_prefix,$page_id,$version_id){
	if ($page_id==NULL ){ fatal ("Page Approval Retrieval Fault: No page_id specified"); }
	if ($version_id==NULL ){ fatal ("Page Approval Retrieval Fault: No version_id specified"); }
	
	$sql="SELECT group_id".
		" FROM ".$db_table_prefix."approvals".
		" WHERE page_id=".$page_id." AND version_id=".$version_id;
	$rs=$db->query($sql);
	$approvals=array();
	foreach ($rs as $id => $r){ $approvals[]=$r["group_id"]; }
	return ($approvals);
}


// returns an array of workflows and approvals for a given workflow and resource
/*
function resource_workflows($db,$db_table_prefix,$workflow_id,$resource_id,$version_id){

	if ($workflow_id==NULL ){ fatal ("Workflow Retrieval Fault: No workflow_id specified"); }
	if ($page_id==NULL){ fatal ("Workflow Retrieval Fault: No page_id specified"); }
	if ($version_id==NULL){ fatal ("Workflow Retrieval Fault: No version_id specified"); }
	
	//workflows
	$sql="SELECT group_id FROM ".$db_table_prefix."workflow WHERE workflow_id=".$workflow_id;
	$rs=$db->query($sql);
	$workflows=array();
	for($i=0; $i<count($rs); $i++){ $workflows[]=$rs[$i]["group_id"]; }
	//return ($groups);

	//approvals
	$sql="SELECT group_id FROM ".$db_table_prefix."approvals WHERE page_id=".$page_id." AND version_id=".$version_id;
	$rs=$db->query($sql);
	$approvals=array();
	for($i=0; $i<count($rs); $i++){ $approvals[]=$rs[$i]["group_id"]; }

	return (array("workflows"=>$workflows,"approvals"=>$approvals));
}
*/


// returns true or false depending on whether the page is approved
function workflow_approved($workflows,$approvals){
	//pre($workflows); pre($approvals); echo $group_id;
	if (!is_array($workflows) || !is_array($approvals)){ return (false); }
	foreach ($workflows as $id => $workflow_id){
		if (! in_array($workflow_id,$approvals)){ return (false); }
	}
	return (true);
}

// returns false if any of the user's groups are in the worklows, and they haven't approved it
function workflow_group_approved($workflows,$approvals,$access){
	if (!is_array($workflows) || !is_array($approvals) || !is_array($access)){ return (false); }

	foreach ($access as $group_id => $access_level){
		if (in_array($group_id,$workflows)){
			if (in_array($group_id,$approvals)){ return (true); }
		}
	}
	
	return (false);
}

//**************************************************************************************************************************************************
// DATABASE INTERACTION

//return an object with a navigation tree and an array of pages to link to the keys in the tree
function retrieve_recursive_nav($db,$page_id=1,$levels=5){
	if (!$db){ return (false); }
	if (!$page_id){ return (false); }

	$db_table_prefix=$GLOBALS["CONFIG"]["db_table_prefix"];

	$sql="SELECT p.page_id,p.url,p.link,p.parent,m.version_id,m.title AS short_title".
		" FROM ".$db_table_prefix."pages AS p,".$db_table_prefix."module AS m".
		" WHERE p.page_id=m.page_id".
		" AND m.status=3".
		" AND p.archive_date IS NULL".
		" ORDER BY p.link ASC, m.version_id DESC\n";
	//echo $sql;
	$rs=$db->query($sql);

	//figure out all the pages and their latest version_ids
	$pages=array();
	foreach ($rs as $id => $r){
		$temp_page_id=$r["page_id"];
		if (!array_key_exists($temp_page_id,$pages)){ //only add the page if it hasn't been already
			unset($r["page_id"]);
			$pages[$temp_page_id]=$r;
		}
	}

	$parents=key_array($rs,"page_id","parent");
	$ret=get_children($page_id,$parents);
	
	return(array("pages"=>$pages,"map"=>$ret));
}
function get_children($page_id,$parent_map){
	$nav=array();
	foreach ($parent_map as $map_page_id => $map_parent){
		if ($map_parent==$page_id){ $nav[$map_page_id]=get_children($map_page_id,$parent_map); }
	}
	return ($nav);
}

//return a flat navigation array
function retrieve_flat_nav($db,$db_table_prefix,$active,$parent=1){
	//$active should control the m.status line?
	$sql="SELECT p.url,p.link,m.title AS short_title,m.version_id".
		" FROM ".$db_table_prefix."pages AS p,".$db_table_prefix."module AS m\n".
		" WHERE p.parent=".$parent."\n".
		" AND p.page_id=m.page_id\n".
		" AND m.status=3\n".
		" AND p.archive_date IS NULL\n".
		" ORDER BY p.link ASC, m.version_id ASC";
	$rs=$db->query($sql);
	
	//this works because the last link is written over the top of the preceding links
	$nav=key_array($rs,"url","short_title");

	return($nav);
}

function retrieve_editor_size($db,$db_common_prefix,$user_id){
	if ($user_id==NULL){ fatal ("Editor Size Retrieval Fault: No user_id specified"); }
	$sql="SELECT editor_size FROM ".$db_common_prefix."users WHERE user_id=".$user_id;
	$rs=$db->query($sql);
	if (count($rs)!=1){ fatal ("Editor Size Retrieval Fault: Could not locate user"); }
	
	$bits=explode("x",$rs[0]["editor_size"]);
	$size=array("width"=>$bits[0],"height"=>$bits[1]);
	
	return ($size);
}

//retrieve a specific group
function retrieve_group($db,$db_table_prefix,$group_id){
	if ($group_id==NULL){ return (false); }
	$sql="SELECT group_id,title FROM ".$db_table_prefix."groups WHERE group_id=".$group_id;
	$rs=$db->query($sql);
	if (count($rs)!=1){ return (false); }
	return ($rs[0]);
}

//return a list of all the groups
function retrieve_groups($db,$db_table_prefix){
	$sql="SELECT group_id,title FROM ".$db_table_prefix."groups ORDER BY title ASC";
	$rs=$db->query($sql);

	$groups=array();
	foreach ($rs as $id => $r){ $groups[$r["group_id"]]=$r["title"]; }
	return ($groups);
}

//returns an array of module types in system
function retrieve_modules($db,$db_table_prefix){
	$sql="SELECT module_id,module_desc FROM ".$db_table_prefix."modules ORDER BY module_desc ASC";
	$rs=$db->query($sql);
	$modules=key_array($rs,"module_id","module_desc");
	return ($modules);
}

//retrieve the module name for a given module
function retrieve_module_name($db,$db_table_prefix,$module_id){
	if (!$module_id){ return (false); }
	$sql="SELECT module_name FROM ".$db_table_prefix."modules WHERE module_id=".$module_id;
	$rs=$db->query($sql);
	if (count($rs)!=1){ return (false); }
	return ($rs[0]["module_name"]);
}

//returns a page given a URL
function retrieve_page($db,$db_table_prefix,$url){
	$sql="SELECT p.page_id,p.workflow_id,p.parent,p.archive_date,m.module_name".
		" FROM ".$db_table_prefix."pages AS p,".$db_table_prefix."modules AS m".
		" WHERE p.archive_date IS NULL".
		" AND p.url='".$url."'".
		" AND p.module_id=m.module_id";
//		echo ($sql); exit();
//archive_date IS NOT NULL
	$rs=$db->query($sql);
	if (count($rs)!=1){ return (false); }
	return ($rs[0]);
}

//returns a page given a page_id and version_id
function retrieve_page_by_key($db,$db_table_prefix,$page_id,$version_id){
	if ($page_id==NULL || $version_id==NULL){ fatal ("Page Retrieval Fault: Missing variables"); }
	$sql="SELECT p.url,p.url,m.title".
		" FROM ".$db_table_prefix."pages AS p,".$db_table_prefix."module AS m".
		" WHERE p.page_id=".$page_id.
		" AND p.page_id=m.page_id".
		" AND m.version_id=".$version_id;
	$rs=$db->query($sql);
	if (count($rs)!=1){ return (false); }
	return ($rs[0]);
}

//returns an array pages in a system
function retrieve_pages($db,$db_table_prefix){
	$sql="SELECT page_id,url FROM ".$db_table_prefix."pages".
		" WHERE archive_date IS NULL ".
		" ORDER BY url ASC";
	$rs=$db->query($sql);
	$pages=array();
	for ($i=0; $i<count($rs); $i++){ $pages[($rs[$i]["page_id"])]=$rs[$i]["url"]; }
	return ($pages);
}

//return the highest permission level for a given page and group
function retrieve_permission_effective($db,$db_table_prefix,$page_id,$groups){
	if ($db_table_prefix==NULL){ fatal ("Effective Permissions Fault: No db_table_prefix specified"); }
	if ($page_id==NULL){ fatal ("Effective Permissions Fault: No page_id specified"); }
	if ($groups==NULL || !is_array($groups)){ fatal ("Effective Permissions Fault: No groups specified"); }
	
	$permissions=retrieve_permissions($db,$db_table_prefix,$page_id);
	
	$highest_permission=0;
	
	foreach ($permissions as $page_id => $values){
		foreach ($values["permissions"] as $id => $bits){
			if (array_key_exists($bits["group_id"],$groups)){ $highest_permission=$bits["access"]; }
		}
	}

	return ($highest_permission);
}

//return the permissions for a given page
function retrieve_permissions($db,$db_table_prefix,$page_id){
	if ($db_table_prefix==NULL){ fatal ("Permissions Retrieval Fault: No db_table_prefix specified"); }
	if ($page_id==NULL){ fatal ("Permissions Retrieval Fault: No page_id specified"); }

	$current_page_id=$page_id;
	$permissions=array();

	$i=0;
	$parent=NULL;
	while ($i<50){ //loop until we break, only loop 50 times to prevent infinite looping

		if ($current_page_id==NULL){ fatal ("Permissions Retrieval Fault: No page_id specified for navigation loop"); }

		//load the page information
		$sql="SELECT url,parent FROM ".$db_table_prefix."pages WHERE page_id=".$current_page_id;
		$rs=$db->query($sql);
		//echo ($sql."<br>\n".ptable($rs));
		if (count($rs)!=1){ fatal ("Permissions Retrieval Fault: Could not locate page ".$current_page_id.", please check your tree data."); }
		$page=$rs[0];

		//load the permissions
		$sql="SELECT group_id,access FROM ".$db_table_prefix."acl WHERE page_id=".$current_page_id;
		$rs=$db->query($sql);
		if (count($rs)>0){ $permissions[$current_page_id]=array("url"=>$page["url"],"permissions"=>$rs); }

		$current_page_id=$page["parent"];
		if ($current_page_id==0){ break; }
		$i++;
	}
	if ($i==50){ fatal ("Permissions Retrieval Fault: Infinite loop detected, please check your tree data.");  }

	//$permissions=array_reverse($permissions);
	return ($permissions);
}

//retrieve the approvals for a given resource version
function retrieve_resource_approvals($db,$db_table_prefix,$resource_id,$version_id){
	if ($resource_id==NULL ){ fatal ("Workflow Resource Retrieval Fault: No resource_id specified"); }
	if ($version_id==NULL ){ fatal ("Workflow Resource Retrieval Fault: No version_id specified"); }
	
	$sql="SELECT group_id FROM ".$db_table_prefix."resource_approvals WHERE resource_id=".$resource_id." AND version_id=".$version_id;
	$rs=$db->query($sql);
	$approvals=array();
	foreach ($rs as $id => $r){ $approvals[]=$r["group_id"]; }
	return ($approvals);
}

// return an array of user information for a given user key
function retrieve_user($db,$db_table_prefix,$key,$user){

	if ($user==NULL){ return (false); }
	if ($key!="username" && $key!="user_id"){ return (false); }
	//pre($GLOBALS["CONFIG"]);

	$sql="SELECT * FROM ".$GLOBALS["CONFIG"]["db_common_prefix"]."users WHERE ";
	if ($key=="user_id"){ $sql.=$key."=".$user; }
	if ($key=="username"){ $sql.="UPPER(".$key.")='".strtoupper($user)."'"; }
	$rs=$db->query($sql);
	if (count($rs)!=1){ return (false); }
	return ($rs[0]);
}


function retrieve_useraccess($db,$db_common_prefix){ 
	$sql="SELECT ref_id,ref_val FROM ".$db_common_prefix."refs WHERE ref='useraccess' ORDER BY ref_id ASC";
	$rs=$db->query($sql);
	$access=key_array($rs,"ref_id","ref_val");
	return ($access);
}

function retrieve_user_access($db,$db_table_prefix,$db_common_prefix,$user_id){
	if ($db_table_prefix==NULL || $db_common_prefix==NULL || $user_id==NULL){ return (false); }
	$sql="SELECT g.group_id,g.level,r.ref_val AS level_text".
		" FROM ".$db_table_prefix."group_members AS g".
		" LEFT JOIN ".$db_common_prefix."refs AS r ON (g.level=r.ref_id AND r.ref='useraccess')".
		" WHERE user_id=".$user_id;
	$rs=$db->query($sql);
	if (count($rs)==0){ return (false); }
	$groups=key_array($rs,"group_id");
	return ($groups);
}

// return the group membership of a user given a user_id *****TO BE DEPRECATED*****
function retrieve_user_groups($db,$db_table_prefix,$user_id){
	if ($db_table_prefix==NULL || $user_id==NULL){ return (false); }
	$sql="SELECT group_id,level FROM ".$db_table_prefix."group_members WHERE user_id=".$user_id;
	$rs=$db->query($sql);
	if (count($rs)==0){ return (false); }
	$groups=key_array($rs,"group_id","level");
	return ($groups);
}

// retrieve all users
function retrieve_users($db,$db_table_prefix,$orderby="surname"){
	if (!in_array($orderby,array("surname","firstname","username","email","access"))){ return (false); }
	$sql="SELECT * FROM ".$db_table_prefix."users ORDER BY ".$orderby." ASC";
	$rs=$db->query($sql);
	$users=key_array($rs,"user_id");
	return ($users);
}

//retrieve all the versions of a given page
function retrieve_page_versions($db,$db_table_prefix,$module,$page_id){

	$sql="SELECT version_id,status,publish_live,publish_end FROM ".$db_table_prefix."module WHERE page_id=".$page_id." ORDER BY version_id DESC";
	$rs=$db->query($sql);
	
	$versions=key_array($rs,"version_id");
	//$versions=array();
	//for ($i=0; $i<count($rs); $i++){ $versions[($rs[$i]["version_id"])]=$rs[$i]["status"]; }
	return ($versions);
}

//retrieve the triple bottom line categories
function retrieve_tbls($db,$db_common_prefix){
	if ($db_common_prefix==NULL){ return (false); }
	$sql="SELECT * FROM ".$db_common_prefix."tbl";
	$rs=$db->query($sql);
	if (count($rs)==0){ return array(); }
	//$tbl=array();
	//foreach ($rs as $id => $r){ $tbl[$r["major"]][$r["minor"]]=$r["def"]; }
	return (key_array($rs,"tbl_id"));
}


//*********************************************************************************************
// GENERIC Module Functions - Most modules share behaviours, generic functions hook to these

//approve a module
function generic_approve_module($db,$db_table_prefix,$page,$version_id){
	$user_id=$_SESSION[$GLOBALS["CONFIG"]["system_session"]."userid"];
	
	foreach ($_SESSION[$GLOBALS["CONFIG"]["system_session"]."access"] AS $gid => $value){ /* do nothing */ } //get the group id (dodgy)

	$now=time();

	//if not already approved by this group && if their group is part of the workflow
	if (! in_array($gid,$page["approvals"]) && in_array($gid,$page["workflows"])){ 
		$insert=array();
		$insert["page_id"]=$page["page_id"];
		$insert["version_id"]=$version_id;
		$insert["group_id"]=$gid;
		$insert["timestamp"]=$now;
		$insert["user_id"]=$user_id;
		$sql="INSERT INTO ".$db_table_prefix."approvals (".implode(",",array_keys($insert)).") VALUES (".implode(",",$insert).")";
		$db->query($sql);
		
		log_edit($db,$db_table_prefix,$user_id,"APPROVE",$page["page_id"],$version_id);

		$page["approvals"][]=$gid; //add the approval to the array
		
		$approved=true;
		foreach ($page["workflows"] as $key => $group_id){ if (! in_array($group_id,$page["approvals"])){ $approved=false; } }
		//pre($page["workflows"]); pre($page["approvals"]); exit();
		
		if ($approved){ //if everyone has approved the module
			$now=$now+1; //this is a bit of a hack, increment the time one second so that the edit logging is before the approval logging
			$sql="UPDATE ".$db_table_prefix."module SET status=3,mod_time=".$now.",mod_userid=".$user_id." WHERE page_id=".$page["page_id"]." AND version_id=".$version_id;
			//echo ($sql."<br>\n");
			$db->query($sql);
			log_edit($db,$db_table_prefix,$user_id,"LIVE",$page["page_id"],$version_id);
		}
		unlock_module($db,$db_table_prefix,$page["page_id"],$page["version_id"],$user_id);
		
		header ("Location: ".prepare_str($_SERVER["PHP_SELF"])."?version_id=".$version_id); exit();
	}
}

//Lock a module for editing - returns the user id of the user that holds the lock
function generic_lock_module($db,$db_table_prefix,$page_id,$version_id,$user_id){
	if ($page_id==NULL){ fatal ("Generic Locking Fault: Missing variable page_id."); }
	if ($version_id==NULL){ fatal ("Generic Locking Fault: Missing variable version_id."); }
	if ($user_id==NULL){ fatal ("Generic Locking Fault: Missing variable user_id."); }

	$now=time();
	$expires=$now+300; // 300 seconds = 5 minutes

	//delete any old old locks or locks the user already has on the page (we'll renew)
	$sql="DELETE FROM ".$db_table_prefix."locks ".
		"WHERE timestamp<".$now.
		"OR (page_id=".$page_id." AND version_id=".$version_id." AND user_id=".$user_id.")";
	$db->query($sql);
	
	//find any existing locks
	$sql="SELECT user_id FROM ".$db_table_prefix."locks WHERE page_id=".$page_id." AND version_id=".$version_id;
	$locks=$db->query($sql);
	if (count($locks)>0){
		$lock_user=$locks[0]["user_id"];
		return ($lock_user);
	}
	
	//create the lock
	$sql="INSERT INTO ".$db_table_prefix."locks (page_id,version_id,user_id,timestamp) VALUES (".$page_id.",".$version_id.",".$user_id.",".$expires.")";
	//echo $sql."<br>";
	$db->query($sql);
	
	return (true);
}

//unlock a module
function generic_unlock_module($db,$db_table_prefix,$page_id,$version_id,$user_id){
	if ($page_id==NULL || $version_id==NULL || $user_id==NULL){ fatal ("Cannot unlock module, missing variables."); }
	$sql="DELETE FROM ".$db_table_prefix."locks WHERE page_id=".$page_id." AND version_id=".$version_id." AND user_id=".$user_id;
	$db->query($sql);
	return (true);
}

//*********************************************************************************************
// Information Box Functions

function box_approvals($db,$page,$mod_type,$mod_time){
	$status_codes=array(1=>"New",2=>"Pending",3=>"Approved",4=>"Cancelled");
	$groups=retrieve_groups($db,$GLOBALS["CONFIG"]["db_table_prefix"]);
		
	$info="<table border='0'>\n";
	$info.="<tr><td>Page Key:</td><td>".$page["page_id"].".".$page["version_id"]."</td></tr>\n";
	$info.="<tr><td>Module:</td><td>".$mod_type."</td></tr>\n";
	$info.="<tr><td>Approvals:</td>\n";
	$info.="<td>";

	//echo the group with highlighting depending if it's approved by them or not
	foreach ($page["workflows"] AS $key=>$group_id){
		if (in_array($group_id,$page["approvals"])){ $info.="<nobr><font color='#009900'>".$groups[$group_id]."</font></nobr><br>\n"; }
		else { $info.="<nobr><font color='#ff0000'>".$groups[$group_id]."</font></nobr><br>\n"; }
	}
	
	$info.="</td></tr>\n";
	$info.="<tr><td>Modified:</td><td><nobr>".date("H:i d/m/Y",$mod_time)."</nobr></td></tr>\n";
	if ($page["archive_date"]!=NULL){ $info.="<tr><td><strong>Archived:</strong></td><td><nobr><strong>".date("H:i d/m/Y",$page["archive_date"])."</strong></nobr></td></tr>\n"; }
	$info.="<tr><td>Versions:</td><td>";
	if (prepare_str($_GET["mode"])=="edit"){ $edit="&mode=edit"; } else { $edit=NULL; }

	$version_count=0;
	$version_buffer="";
	foreach ($page["versions"] as $version_id => $values){
		if ($version_count<5){ //only display last 5 versions
			$info.="<a href='".prepare_str($_SERVER["PHP_SELF"])."?version_id=".$version_id.$edit."'>".$version_id." - ".$status_codes[$values["status"]]."</a>";
			if ($page["version_id"]!=$version_id){ $info.=" <a href='".prepare_str($_SERVER["PHP_SELF"])."?version_id=".$page["version_id"]."&compare_id=".$version_id.$edit."'><img src='/stronghold/images/compare.gif' alt='Compare Versions ".$page["version_id"]." and ".$version_id."' border='0'></a>"; }
			$info.="<br>\n";
			$version_count++;
		}
	}
	if ($version_count==5){
		
	}
	$info.="</td></tr>\n";
	$info.="<tr><td colspan='2'><img src='/resources/images/pixel.gif' height='1' width='200'></td></tr>";
	$info.="</table>";			
	return (menu_box("Page Information",$info));
}

function box_history($db,$page_id,$version_id){
	$db_table_prefix=$GLOBALS["CONFIG"]["db_table_prefix"];
	$db_common_prefix=$GLOBALS["CONFIG"]["db_common_prefix"];

	$sql="SELECT l.timestamp,l.event,r.ref_val,u.firstname,u.surname,u.email".
		" FROM ".$db_table_prefix."editlog AS l".
		" LEFT JOIN (SELECT * FROM ".$db_common_prefix."refs AS r WHERE ref='editlog') AS r ON (r.ref_id=l.event)".
		" LEFT JOIN ".$db_common_prefix."users AS u ON (l.user_id=u.user_id)".
		" WHERE page_id=".$page_id.
		" AND version_id=".$version_id.
		" ORDER BY l.timestamp DESC, l.event DESC";
	//echo ($sql);
	$rs=$db->query($sql);
	//echo ptable($rs); exit();

	if (count($rs)>0){
		$history="<table border='1' class='dtable'>\n";
		foreach ($rs as $id => $r){
			$history.="<tr><td>".$r["ref_val"]."</td>\n";
			$history.="<td><nobr><span class='footnote'>".date("d/m/Y H:i:s",$r["timestamp"])."</span></nobr><br>";
			$history.="<nobr> <a href='mailto:".$r["email"]."'>".$r["firstname"]." ".$r["surname"]."</a></nobr></td></tr>\n";
		}
		$history.="</table>";
	} else {
		$history.="<em>None!</em>\n";
	}
		
	$box=menu_box("Version History",$history);
	return ($box);
}

// generic menubox
function menu_box($title,$content){
	$r="<fieldset>\n";
	$r.="<legend>".$title."&nbsp;</legend>\n";
	$r.="<div align='center'>".$content."</div>\n";
	$r.="<img src='/resources/images/pixel.gif' height='1' width='200'>";
	$r.="</fieldset><br>\n";
	return ($r);
}

//*********************************************************************************************
// HTML Rendering Functions

//return a form submit button
function button_submit($form,$name,$label,$icon,$disabled=false){
	if ($disabled){
		return ("<a href='javascript:void(0);' class='buttonDis'><div class='button_padding'><img src='/stronghold/images/".$icon."' border='0' style='position:relative; top:3px;'>&nbsp;".$label."</div></a>\n");
	} else {
		return ("<input type='hidden' name='".$name."' value=''>\n".
				"<a href='javascript:document.".$form.".".$name.".value=\"".$label."\";document.".$form.".submit();' class='button'><div class='button_padding'><img src='/stronghold/images/".$icon."' border='0' style='position:relative; top:3px;' width='16' height='16'>&nbsp;".$label."</div></a>\n");
		//return ("<input type='image' name='".$name."' alt='".$value."' value='".$label."' src='/stronghold/images/pixel.gif' class='button'>");
	}
}

//return a link button
function button_href($label,$location,$icon,$disabled=false){
	if ($disabled){
		return ("<a href='javascript:void(0);' class='buttonDis'><div class='button_padding'><img src='/stronghold/images/".$icon."' border='0' style='position:relative; top:3px;'>&nbsp;".$label."</div></a>");
	} else {
		return ("<a href='".$location."' class='button'><div class='button_padding'><img src='/stronghold/images/".$icon."' border='0' style='position:relative; top:3px;'>&nbsp;".$label."</div></a>\n");
	}
}

function date_picker($label,$value=NULL,$mask="d/m/Y"){

	$r="<input id='".$label."' name='".$label."' class=\"datechooser dc-dateformat='".$mask."' dc-iconlink='/stronghold/images/datechooser.gif' dc-onupdate='chooser_".$label."'\" type='text' value='".$value."'>
	<script type='text/javascript'>
	<!-- //
		function chooser_".$label."(objDate){
			var chooserhandle = document.getElementById('".$label."');
			chooserhandle.DateChooser.updateFields();
			return true;
		}
	// -->
	</script>";
	return ($r);
}

//return an icon for a given filename
function file_icon ($filename){
	$unknown="<img src='/stronghold/images/pixel.gif'>";

	if ($filename==NULL){ return ($unknown); }
	
	$ext=file_type($filename);
	if ($ext==NULL){ return ($unknown); }
	
	$base_path="/stronghold/images/icons/";

	if ($ext=="doc"){ return ("<img src='".$base_path."doc.gif' alt='Microsoft Word Document'>"); }
	if ($ext=="gif"){ return ("<img src='".$base_path."gif.gif' alt='GIF Image'>"); }
	if ($ext=="htm"){ return ("<img src='".$base_path."htm.gif' alt='HTML Document'>"); }
	if ($ext=="jpg"){ return ("<img src='".$base_path."jpg.gif' alt='JPG Image'>"); }
	if ($ext=="pdf"){ return ("<img src='".$base_path."pdf.gif' alt='PDF Document'>"); }
	if ($ext=="png"){ return ("<img src='".$base_path."png.gif' alt='PNG image'>"); }
	if ($ext=="ppt"){ return ("<img src='".$base_path."ppt.gif' alt='Microsoft Powerpoint Presentation'>"); }
	if ($ext=="rtf"){ return ("<img src='".$base_path."doc.gif' alt='Rich Text Document'>"); }
	if ($ext=="tif"){ return ("<img src='".$base_path."tif.gif' alt='TIF Image'>"); }
	if ($ext=="txt"){ return ("<img src='".$base_path."txt.gif' alt='Plain Text Document'>"); }
	if ($ext=="xls"){ return ("<img src='".$base_path."xls.gif' alt='Microsoft Excel Spreadsheet'>"); }
	if ($ext=="zip"){ return ("<img src='".$base_path."zip.gif' alt='ZIP Archive'>"); }

	return ($unknown);
}

//*********************************************************************************************
// Data Checking/Consistency Functions (not Stronghold specific)

//return a validated timestamp given a date
function date_timestamp($str){

	$bits=explode("/",$str);
	if (count($bits)!=3){ return (false); }

	$d=(int)$bits[0];
	$m=(int)$bits[1];
	$y=(int)$bits[2];

	if ($d<1 || $d>31){ return (false); }
	if ($m<1 || $m>12){ return (false); }
	if ($y<1980 || $y>2060){ return (false); }

	return (mktime(0,0,0,$m,$d,$y));
}

//return a validated timestamp given a date and time
function datetime_timestamp($str,$hour,$min){
	if ($hour===NULL){ return (false); }
	if ($min===NULL){ return (false); }
	$hour=(int)$hour;
	$min=(int)$min;

	$bits=explode("/",$str);
	if (count($bits)!=3){ return (false); }

	$d=(int)$bits[0];
	$m=(int)$bits[1];
	$y=(int)$bits[2];

	if ($d<1 || $d>31){ return (false); }
	if ($m<1 || $m>12){ return (false); }
	if ($y<1980 || $y>2060){ return (false); }

	return (mktime($hour,$min,0,$m,$d,$y));
}

//check the structure of an array and return errors if not correct
// $actual = the post array
// $compulsory = fields that must be specified
// $option = optional fields
// $not_null = fields that must not contain a null value
function field_check($actual,$compulsory,$optional,$not_null,$label){

	//check for missing compulsory variables
	foreach ($compulsory as $id => $var){
		if (!array_key_exists($var,$actual)){ return ($label.": Compulsory variable [".$var."] not specfied."); }
	}
	
	//check all the variables given are allowed
	$all=array_merge($compulsory,$optional);
	foreach ($actual as $key => $value){
		if (!in_array($key,$all)){ return  ($label.": Unknown variable [".$key."] specfied."); }
	}

	//check for illegal NULL values
	foreach ($not_null as $id => $var){
		if ($actual[$var]===NULL){ return ($label.": NULL value given for non-null variable [".$var."]."); }
	}

	return (true);
}

//return the file type for a given filename (maybe we should look at meta data?)
function file_type($filename){
	$ext=strtolower(substr($filename,strlen($filename)-3,3));

	if ($ext=="gif"){ return ($ext); } //gif image
	if ($ext=="jpg"){ return ($ext); } //jpg image
	if ($ext=="png"){ return ($ext); } //png image
	if ($ext=="tif"){ return ($ext); } //tiff image
	
	if ($ext=="doc"){ return ($ext); } //word document
	if ($ext=="htm"){ return ($ext); } //html document
	if ($ext=="pdf"){ return ($ext); } //pdf document
	if ($ext=="ppt"){ return ($ext); } //powerpoint
	if ($ext=="rtf"){ return ($ext); } //rich text document
	if ($ext=="txt"){ return ($ext); } //text document
	if ($ext=="xls"){ return ($ext); } //excel document
	
	if ($ext=="zip"){ return ($ext); } //zip archive

	return (NULL);
}

//verify the characters allowed in a link
function illegal_link($str){
//echo $str;
	$legal=array("a","b","c","d","e","f","g","h","i","j","k","l","m","n","o","p","q","r","s","t","u","v","w","x","y","z","-","_","1","2","3","4","5","6","7","8","9","0");
	for ($i=0; $i<strlen($str); $i++){ if (!in_array($str[$i],$legal)){ return(true); } }
	return(false);
}

//return a key from the post or get array with full filtering, allows the sentinel value of "new"
function request_key($label){
	$id=prepare_str($_GET[$label]);
	if ($id==NULL){ $id=prepare_str($_POST[$label]); }
	if ($id!=NULL && $id!="new"){ $id=prepare_int($id); }
	return ($id);
}

//*********************************************************************************************
// Misc Functions

//mssql hack because you can't use dynamic table names in a function
function tree_path_sql($db_type,$db_table_prefix,$node_id){
	if ($db_type=="mssql"){ return ($tree_path="dbo.".$db_table_prefix."tree_path(".$node_id.")"); }
	return ("tree_path(".$node_id.",'".$db_table_prefix."resource_tree')");
}

function adldap_config($CONFIG){
	if (!array_key_exists("adldap_account_suffix",$CONFIG)){ fatal("adLDAP Config Fault: No adldap_account_suffix specified"); }
	if (!array_key_exists("adldap_base_dn",$CONFIG)){ fatal("adLDAP Config Fault: No adldap_base_dn specified"); }
	if (!array_key_exists("adldap_domain_controllers",$CONFIG)){ fatal("adLDAP Config Fault: No adldap_domain_controllers specified"); }
	if (!array_key_exists("adldap_ad_username",$CONFIG)){ fatal("adLDAP Config Fault: No adldap_ad_username specified"); }
	if (!array_key_exists("adldap_ad_password",$CONFIG)){ fatal("adLDAP Config Fault: No adldap_ad_password specified"); }

	$c=array();
	$c["account_suffix"]=$CONFIG["adldap_account_suffix"];
	$c["base_dn"]=$CONFIG["adldap_base_dn"];
	$c["domain_controllers"]=$CONFIG["adldap_domain_controllers"];
	$c["ad_username"]=$CONFIG["adldap_ad_username"];
	$c["ad_password"]=$CONFIG["adldap_ad_password"];
	return ($c);
}

//fatal error handler
function fatal($error){
	$msg=$error."<br><br>\n";
	$msg.="SERVER: ".prepare_str($_SERVER["HTTP_HOST"])."<br>\n";
	$msg.="SCRIPT: ".prepare_str($_SERVER["PHP_SELF"])."<br>\n";
	$msg.="TIME: ".date("d/m/Y H:i:s")."<br>\n";
	$msg.="SESSION: <br>\n";
	if (is_array($_SESSION)){ foreach($_SESSION as $key => $value){ $msg.=$key." - ".$value."<br><br>\n"; } }
	$msg.=$_SERVER["REMOTE_ADDR"]." ".$_SERVER["REMOTE_HOST"]."<br>\n";
	$msg.=$_SERVER["HTTP_USER_AGENT"]."<br><br>\n";
	
	//determine all the emails to report to
	if (!is_array($GLOBALS["CONFIG"]["error_reporting"])){ echo ("FATAL: The error_reporting variable in config.php must be an array"); exit(); }
	$emails=$GLOBALS["CONFIG"]["error_reporting"];
	
	// send the emails
	foreach ($emails as $id => $email){ html_mail($email,"Stronghold Error Handler <".$GLOBALS["CONFIG"]["system_email"].">","Fatal Error",$msg); }

	// bomb out with a pretty error message
	echo ("<font color='#ff0000' size='-1' face='Verdana,Helvetica,Arial'><strong>FATAL: ".$error."</strong></font>");
	exit();
}

//return a correctly formatted string for file sizes
function file_size($path){
	if ($path==NULL){ return (false); }
	return (number_format(round(filesize($path)/1024))."KB");
}

function missing_alt_tags($str){

	
}


function has_access($requires,$level,$acl){
	//$requires is an array of groups
	//$level is the level of access (int)
	//$acl is an array of the users groups where key=group_id and value=level

	foreach ($requires as $key => $group_id){
		if (array_key_exists($group_id,$acl)){
			if ($acl[$group_id]>=$level){ return (true); }
		}
	}
	return (false);
}

//checks if someone has administrator access
function has_admin($access){
	if (!is_array($access)){ return (false); }
	foreach ($access as $group_id => $level){
		if ($level==4){ return(true); }
	}
	return (false);
}

//log an edit, returns true on success, false on failure
function log_edit($db,$db_table_prefix,$user_id,$event,$page_id,$version_id){
	if ($user_id==NULL){ fatal("Missing logging variable [user_id] - event: ".$event); }
	if ($event==NULL){ fatal("Missing logging variable [event] - event: ".$event); }
	if ($page_id==NULL){ fatal("Missing logging variable [page_id] - event: ".$event); }
	if ($version_id==NULL){ fatal("Missing logging variable [version_id] - event: ".$event); }
	
	$event_id=NULL;
	if ($event=="UPDATE"){ $event_id=1; }
	elseif ($event=="ARCHIVE"){ $event_id=2; }
	elseif ($event=="APPROVE"){ $event_id=3; }
	elseif ($event=="LIVE"){ $event_id=4; }
	elseif ($event=="REV_NOTE"){ $event_id=5; }
	elseif ($event=="SUBMIT"){ $event_id=6; }
	elseif ($event=="CREATE"){ $event_id=7; }
	elseif ($event=="CANCEL"){ $event_id=8; }
	if ($event_id==NULL){ fatal ("Undefined log event."); }

	$sql="INSERT INTO  ".$db_table_prefix."editlog (page_id,version_id,user_id,event,timestamp";
	$sql.=") VALUES (";
	$sql.=$page_id.",".$version_id.",".$user_id.",".$event_id.",".time().");";
	$db->query($sql);

	return (true);	
}

// notify officers of an event in the system
function notify_officers($db,$emails,$event,$page_id,$version_id){
	if ($page_id==NULL || $version_id==NULL){ fatal ("Notification Fault: Missing Variables"); }
	if (!is_array($emails)){ return (false); }
	if ($event==NULL){ return (false); }
	
	$page=retrieve_page_by_key($db,$GLOBALS["CONFIG"]["db_table_prefix"],$page_id,$version_id);
	if (!$page){ fatal("Notification Fault: Could not locate page"); }
	
	$domain="https://".$GLOBALS["CONFIG"]["system_domain"];
	$block="<br><table border='0'>\n";
	$block.="<tr><td><strong>System:</strong></td><td>".$GLOBALS["CONFIG"]["system_name"]."</td></tr>\n";
	$block.="<tr><td><strong>System Address:</strong></td><td><a href='".$domain."/'>".$domain."/</a></td></tr>\n";
	$block.="<tr><td><strong>Page ID:</strong></td><td>".$page_id.".".$version_id."</td></tr>\n";
	$block.="<tr><td><strong>Page Title:</strong></td><td>".$page["title"]."</td></tr>\n";
	$block.="<tr><td><strong>Page Location:</strong></td><td><a href='".$domain.$page["url"]."'>".$domain.$page["url"]."</a></td></tr>\n";
	$block.="<tr><td><strong>Username:</strong></td><td>".$_SESSION[$GLOBALS["CONFIG"]["system_session"]."username"]."</td></tr>\n";
	$block.="<tr><td><strong>Time:</strong></td><td>".date("d/m/Y H:i",time())."</td></tr>\n";
	$block.="</table>\n";
	//echo ($block); exit();

	$body=NULL;
	if ($event=="SUBMIT"){
		$subject="New Page Requires Approval";
		$body="A new document has been submitted for approval.<br>\n";
		$body.=$block;
	}
	if ($event=="REVERT"){
		$subject="Existing Page Requires Approval";
		$body="A page with your prior approval has been changed and requires a new approval.<br>";
		$body.=$block;
	}
	if ($body==NULL){ return (false); }
	
	//send the emails
	foreach ($emails as $id => $email){
		$from="Stronghold <".$GLOBALS["CONFIG"]["system_email"].">";
		$b="<html><body><font face='Verdana,Helvetica,Arial' size='-1'>\n";
		$b.=$body;
		$b.="</font></body></html>";
		html_mail($email,$from,$subject,$b);
	}
	
	return (true);
}



//return a formatted revision note string
function rev_note($new,$old,$user){
	return ("\n\n".$new."\n\n"."--- ".$user." - ".date("H:i:s d/m/Y")." ------------------------------".$old);
}

// return a select box given an array
function select_box($arr,$title,$default){
	$r="<select name='".$title."'>\n";
	foreach($arr as $key => $value){
		$r.="<option value='".$key."'";
		if ($default==$key){ $r.=" selected"; }
		$r.=">".$value."</option>\n";
	}	
	$r.="</select>\n";
	return ($r);
}

//return the javascript tag for a tooltip
function tooltip($msg){ return ("<a href=\"javascript:void(0);\" onmouseover=\"return overlib('".$msg."');\" onmouseout=\"return nd();\"><img src='/stronghold/images/help.gif' border='0'></a>"); }

//return the number of levels in a url
function url_levels($url){
	if ($url==""){ fatal("Cannot count levels in a url with a NULL value"); }
	//if ($url=="/" || $url==""){ return (0); }
	if ($url[strlen($url)-1]=="/"){ $url=substr($url,0,strlen($url)-1); } //drop trailing slashes
	if ($url[strlen($url)]=="/"){ $url=substr($url,0,strlen($url)); }
	$levels=0;
	for ($i=0; $i<strlen($url); $i++){ if ($url[$i]=="/"){ $levels++; } } //count the levels
	return ($levels);
}

?>
