<?
/*
Stronghold CMS - Stronghold Database Schema
Copyright (C) 2008 Scott Barnett

This program is free software; you can redistribute it and/or
modify it under the terms of Version 3 of the GNU General Public
License as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.


------------------------------------------------------------------

At this stage, the schema-matic only works with Postgres.

Defining the schema in this way is intended to provide the basis for
supporting multiple RDBMS'

This script can be included multiple times, and isn't meant to be
called directly.
*/

if (!is_array($db_options)){ echo ("FATAL: No database options specified for schema creation"); exit(); }
if ($db_options["db_common_prefix"]===NULL){ echo ("FATAL: No common specified\n"); exit(); }
if ($db_options["prefixes"]===NULL){ echo ("FATAL: No prefixes spedified\n"); exit(); }
if ($db_options["test"]===NULL){ echo ("FATAL: Test flag not spedified\n"); exit(); }
if ($db_options["populate_database"]===NULL){ echo ("FATAL: Population database flag not set\n"); exit(); }
if ($db_options["populate_common"]===NULL){ echo ("FATAL: Population common flag not set\n"); exit(); }
if ($db_options["database_type"]===NULL){ echo ("FATAL: Database type flag not set\n"); exit(); }

//reset the variables
$txt_tables=array();
$txt_constraints=array();
$txt_functions=array();

//******************************************************************************************************************************************
// Common Tables
if ($db_options["populate_common"]){
	$table_name="users";
	$txt_tables[$db_options["db_common_prefix"].$table_name]=array(
		"user_id"		=>array("is_nullable" => "NO", 	"data_type" => "SERIAL", 		"character_maximum_length" => NULL),
		"username"		=>array("is_nullable" => "NO", 	"data_type" => "VARCHAR",		"character_maximum_length" => 100),
		"password"		=>array("is_nullable" => "YES",	"data_type" => "VARCHAR",		"character_maximum_length" => 50),
		"firstname"		=>array("is_nullable" => "NO", 	"data_type" => "VARCHAR",		"character_maximum_length" => 255),
		"surname"		=>array("is_nullable" => "NO", 	"data_type" => "VARCHAR",		"character_maximum_length" => 255),
		"email"			=>array("is_nullable" => "NO", 	"data_type" => "VARCHAR",		"character_maximum_length" => 255),
		"access"		=>array("is_nullable" => "NO", 	"data_type" => "SMALLINT",		"character_maximum_length" => NULL),
		"active"		=>array("is_nullable" => "NO", 	"data_type" => "SMALLINT",		"character_maximum_length" => NULL),
		"editor_size"	=>array("is_nullable" => "NO", 	"data_type" => "VARCHAR",		"character_maximum_length" => 11),
	);
	$txt_constraints[$db_options["db_common_prefix"].$table_name."_pkey"]=array("constraint_type"=>"PRIMARY","table_name"=>$db_options["db_common_prefix"].$table_name,"columns"=>array("user_id"));
	$txt_constraints[$db_options["db_common_prefix"].$table_name."_unique"]=array("constraint_type"=>"UNIQUE","table_name"=>$db_options["db_common_prefix"].$table_name,"columns"=>array("username"));
	
	//-------------------------------------------------------------------------------------------------------------------------
	
	$table_name="refs";
	$txt_tables[$db_options["db_common_prefix"].$table_name]=array(
		"ref"			=>array("is_nullable" => "NO", 	"data_type" => "VARCHAR",		"character_maximum_length" => 10),
		"ref_id"		=>array("is_nullable" => "NO",	"data_type" => "INT",			"character_maximum_length" => NULL),
		"ref_val"		=>array("is_nullable" => "NO",	"data_type" => "VARCHAR",		"character_maximum_length" => 20),
	);
	$txt_constraints[$db_options["db_common_prefix"].$table_name."_unique"]=array("constraint_type"=>"UNIQUE","table_name"=>$db_options["db_common_prefix"].$table_name,"columns"=>array("ref","ref_id"));
	
	//-------------------------------------------------------------------------------------------------------------------------
	
	$table_name="seq";
	$txt_tables[$db_options["db_common_prefix"].$table_name]=array(
		"seq_name"		=>array("is_nullable" => "NO",	"data_type" => "VARCHAR",		"character_maximum_length" => 40),
		"seq_val"		=>array("is_nullable" => "NO",	"data_type" => "INT",			"character_maximum_length" => NULL),
	);
	$txt_constraints[$db_options["db_common_prefix"].$table_name."_unique"]=array("constraint_type"=>"UNIQUE","table_name"=>$db_options["db_common_prefix"].$table_name,"columns"=>array("seq_name"));
	
	//-------------------------------------------------------------------------------------------------------------------------
	
	$table_name="tbl";
	$txt_tables[$db_options["db_common_prefix"].$table_name]=array(
		"tbl_id"		=>array("is_nullable" => "NO", 	"data_type" => "SERIAL",		"character_maximum_length" => NULL),
		"major"			=>array("is_nullable" => "NO",	"data_type" => "SMALLINT",		"character_maximum_length" => NULL),
		"minor"			=>array("is_nullable" => "NO",	"data_type" => "SMALLINT",		"character_maximum_length" => NULL),
		"def"			=>array("is_nullable" => "NO",	"data_type" => "VARCHAR",		"character_maximum_length" => 100),
	);
	$txt_constraints[$db_options["db_common_prefix"].$table_name."_pkey"]=array("constraint_type"=>"PRIMARY","table_name"=>$db_options["db_common_prefix"].$table_name,"columns"=>array("tbl_id"));
}

//******************************************************************************************************************************************
// Tables for each CMS

foreach ($db_options["prefixes"] as $id => $prefix){
	//-------------------------------------------------------------------------------------------------------------------------

	$table_name="acl";
	$txt_tables[$prefix.$table_name]=array(
		"group_id"		=>array("is_nullable" => "NO", 	"data_type" => "INT", 		"character_maximum_length" => NULL),
		"page_id"		=>array("is_nullable" => "NO", 	"data_type" => "INT", 		"character_maximum_length" => NULL),
		"access"		=>array("is_nullable" => "NO", 	"data_type" => "INT", 		"character_maximum_length" => NULL),
	);
	$txt_constraints[$prefix.$table_name."_unique"]=array("constraint_type"=>"UNIQUE","table_name"=>$prefix.$table_name,"columns"=>array("group_id","page_id"));
	
	//-------------------------------------------------------------------------------------------------------------------------
	
	$table_name="approvals";
	$txt_tables[$prefix.$table_name]=array(
		"page_id"		=>array("is_nullable" => "NO", 	"data_type" => "INT", 		"character_maximum_length" => NULL),
		"version_id"	=>array("is_nullable" => "NO", 	"data_type" => "INT", 		"character_maximum_length" => NULL),
		"group_id"		=>array("is_nullable" => "NO", 	"data_type" => "INT", 		"character_maximum_length" => NULL),
		"timestamp"		=>array("is_nullable" => "NO", 	"data_type" => "INT", 		"character_maximum_length" => NULL),
		"user_id"		=>array("is_nullable" => "NO", 	"data_type" => "INT", 		"character_maximum_length" => NULL),
	);
	$txt_constraints[$prefix.$table_name."_unique"]=array("constraint_type"=>"UNIQUE","table_name"=>$prefix.$table_name,"columns"=>array("page_id","version_id","group_id"));
	
	//-------------------------------------------------------------------------------------------------------------------------
	
	$table_name="editlog";
	$txt_tables[$prefix.$table_name]=array(
		"page_id"		=>array("is_nullable" => "NO", 	"data_type" => "INT", 		"character_maximum_length" => NULL),
		"version_id"	=>array("is_nullable" => "NO",	"data_type" => "INT",		"character_maximum_length" => NULL),
		"user_id"		=>array("is_nullable" => "NO",	"data_type" => "INT",		"character_maximum_length" => NULL),
		"timestamp"		=>array("is_nullable" => "NO",	"data_type" => "INT",		"character_maximum_length" => NULL),
		"event"			=>array("is_nullable" => "NO",	"data_type" => "SMALLINT",	"character_maximum_length" => NULL),
	);
	
	//-------------------------------------------------------------------------------------------------------------------------
	
	$table_name="group_members";
	$txt_tables[$prefix.$table_name]=array(
		"group_id"		=>array("is_nullable" => "NO", 	"data_type" => "INT", 		"character_maximum_length" => NULL),
		"user_id"		=>array("is_nullable" => "NO", 	"data_type" => "INT", 		"character_maximum_length" => NULL),
		"level"			=>array("is_nullable" => "NO", 	"data_type" => "SMALLINT",	"character_maximum_length" => NULL),
	);
	$txt_constraints[$prefix.$table_name."_unique"]=array("constraint_type"=>"UNIQUE","table_name"=>$prefix.$table_name,"columns"=>array("group_id","user_id"));
	
	//-------------------------------------------------------------------------------------------------------------------------
	
	$table_name="groups";
	$txt_tables[$prefix.$table_name]=array(
		"group_id"		=>array("is_nullable" => "NO", 	"data_type" => "SERIAL", 	"character_maximum_length" => NULL),
		"title"			=>array("is_nullable" => "NO", 	"data_type" => "VARCHAR",	"character_maximum_length" => 255),
	);
	$txt_constraints[$prefix.$table_name."_pkey"]=array("constraint_type"=>"PRIMARY","table_name"=>$prefix.$table_name,"columns"=>array("group_id"));

	//-------------------------------------------------------------------------------------------------------------------------
	
	$table_name="locks";
	$txt_tables[$prefix.$table_name]=array(
		"page_id"		=>array("is_nullable" => "NO", 	"data_type" => "INT",		"character_maximum_length" => NULL),
		"user_id"		=>array("is_nullable" => "NO",	"data_type" => "INT",		"character_maximum_length" => NULL),
		"version_id"	=>array("is_nullable" => "NO",	"data_type" => "INT",		"character_maximum_length" => NULL),
		"timestamp"		=>array("is_nullable" => "NO",	"data_type" => "INT",		"character_maximum_length" => NULL),
	);
	$txt_constraints[$prefix.$table_name."_unique"]=array("constraint_type"=>"UNIQUE","table_name"=>$prefix.$table_name,"columns"=>array("page_id","user_id","version_id"));

	//-------------------------------------------------------------------------------------------------------------------------
	
	$table_name="module";
	$txt_tables[$prefix.$table_name]=array(
		"page_id"		=>array("is_nullable" => "NO", 	"data_type" => "INT",		"character_maximum_length" => NULL),
		"version_id"	=>array("is_nullable" => "NO", 	"data_type" => "INT",		"character_maximum_length" => NULL),
		"title"			=>array("is_nullable" => "YES",	"data_type" => "VARCHAR",	"character_maximum_length" => 255),
		"mod_time"		=>array("is_nullable" => "NO", 	"data_type" => "INT",		"character_maximum_length" => NULL),
		"mod_userid"	=>array("is_nullable" => "YES",	"data_type" => "INT",		"character_maximum_length" => NULL),
		"status"		=>array("is_nullable" => "NO",	"data_type" => "SMALLINT",	"character_maximum_length" => NULL),
		"created_date"	=>array("is_nullable" => "YES", "data_type" => "INT",		"character_maximum_length" => NULL),
		"short_title"	=>array("is_nullable" => "YES", "data_type" => "VARCHAR",	"character_maximum_length" => 20),
		"nav_id"		=>array("is_nullable" => "NO",  "data_type" => "SMALLINT",	"character_maximum_length" => NULL),
		"heading"		=>array("is_nullable" => "YES", "data_type" => "SMALLINT",	"character_maximum_length" => NULL),
		"publish_live"	=>array("is_nullable" => "YES", "data_type" => "INT",		"character_maximum_length" => NULL),
		"publish_end"	=>array("is_nullable" => "YES", "data_type" => "INT",		"character_maximum_length" => NULL),
	);
	$txt_constraints[$prefix.$table_name."_unique"]=array("constraint_type"=>"UNIQUE","table_name"=>$prefix.$table_name,"columns"=>array("page_id","version_id"));
	
	//-------------------------------------------------------------------------------------------------------------------------
	
	$table_name="module_plugin";
	$txt_tables[$prefix.$table_name]=array(
		"page_id"		=>array("is_nullable" => "NO", 	"data_type" => "INT",		"character_maximum_length" => NULL),
		"version_id"	=>array("is_nullable" => "NO", 	"data_type" => "INT",		"character_maximum_length" => NULL),
		"include_path"	=>array("is_nullable" => "YES",	"data_type" => "TEXT",		"character_maximum_length" => NULL),
		"notes"			=>array("is_nullable" => "YES", "data_type" => "TEXT",		"character_maximum_length" => NULL),
	);
	$txt_constraints[$prefix.$table_name."_unique"]=array("constraint_type"=>"UNIQUE","table_name"=>$prefix.$table_name,"columns"=>array("page_id","version_id"));
	
	//-------------------------------------------------------------------------------------------------------------------------
	
	$table_name="module_sitemap";
	$txt_tables[$prefix.$table_name]=array(
		"page_id"		=>array("is_nullable" => "NO", 	"data_type" => "INT",		"character_maximum_length" => NULL),
		"version_id"	=>array("is_nullable" => "NO", 	"data_type" => "INT",		"character_maximum_length" => NULL),
		"text_before"	=>array("is_nullable" => "YES",	"data_type" => "TEXT",		"character_maximum_length" => NULL),
		"text_after"	=>array("is_nullable" => "YES",	"data_type" => "TEXT",		"character_maximum_length" => NULL),
	);
	$txt_constraints[$prefix.$table_name."_unique"]=array("constraint_type"=>"UNIQUE","table_name"=>$prefix.$table_name,"columns"=>array("page_id","version_id"));
	
	//-------------------------------------------------------------------------------------------------------------------------
	
	$table_name="module_text";
	$txt_tables[$prefix.$table_name]=array(
		"page_id"		=>array("is_nullable" => "NO", 	"data_type" => "INT",		"character_maximum_length" => NULL),
		"version_id"	=>array("is_nullable" => "NO", 	"data_type" => "INT",		"character_maximum_length" => NULL),
		"body_text"		=>array("is_nullable" => "YES",	"data_type" => "TEXT",		"character_maximum_length" => NULL),
		"notes"			=>array("is_nullable" => "YES", "data_type" => "TEXT",		"character_maximum_length" => NULL),
	);
	$txt_constraints[$prefix.$table_name."_unique"]=array("constraint_type"=>"UNIQUE","table_name"=>$prefix.$table_name,"columns"=>array("page_id","version_id"));
	
	//-------------------------------------------------------------------------------------------------------------------------

	$table_name="modules";
	$txt_tables[$prefix.$table_name]=array(
		"module_id"		=>array("is_nullable" => "NO", 	"data_type" => "INT", 		"character_maximum_length" => NULL),
		"module_name"	=>array("is_nullable" => "NO",	"data_type" => "VARCHAR",	"character_maximum_length" => 20),
		"module_desc"	=>array("is_nullable" => "NO",	"data_type" => "VARCHAR",	"character_maximum_length" => 255),
		"time_installed"=>array("is_nullable" => "NO",	"data_type" => "INT",		"character_maximum_length" => NULL),
	);
	$txt_constraints[$prefix.$table_name."_pkey"]=array("constraint_type"=>"PRIMARY","table_name"=>$prefix.$table_name,"columns"=>array("module_id"));

	//-------------------------------------------------------------------------------------------------------------------------
	
	$table_name="pages";
	$txt_tables[$prefix.$table_name]=array(
		"page_id"		=>array("is_nullable" => "NO", 	"data_type" => "SERIAL", 	"character_maximum_length" => NULL),
		"time_created"	=>array("is_nullable" => "NO", 	"data_type" => "INT", 		"character_maximum_length" => NULL),
		"parent"		=>array("is_nullable" => "NO",	"data_type" => "INT", 		"character_maximum_length" => NULL),
		"workflow_id"	=>array("is_nullable" => "NO", 	"data_type" => "INT", 		"character_maximum_length" => NULL),
		"link"			=>array("is_nullable" => "YES",	"data_type" => "VARCHAR",	"character_maximum_length" => 255),
		"url"			=>array("is_nullable" => "NO", 	"data_type" => "VARCHAR",	"character_maximum_length" => 255),
		"module_id"		=>array("is_nullable" => "NO", 	"data_type" => "INT", 		"character_maximum_length" => NULL),
		"archive_date"	=>array("is_nullable" => "YES",	"data_type" => "INT", 		"character_maximum_length" => NULL),
		"levels"		=>array("is_nullable" => "NO", 	"data_type" => "SMALLINT",	"character_maximum_length" => NULL),
	);
	$txt_constraints[$prefix.$table_name."_pkey"]=array("constraint_type"=>"PRIMARY","table_name"=>$prefix.$table_name,"columns"=>array("page_id"));
	
	//-------------------------------------------------------------------------------------------------------------------------
	
	$table_name="resources";
	$txt_tables[$prefix.$table_name]=array(
		"resource_id"		=>array("is_nullable" => "NO", 	"data_type" => "SERIAL",	"character_maximum_length" => NULL),
		"current_version"	=>array("is_nullable" => "NO", 	"data_type" => "INT", 		"character_maximum_length" => NULL),
		"time_created"		=>array("is_nullable" => "NO",	"data_type" => "INT", 		"character_maximum_length" => NULL),
		"workflow_id"		=>array("is_nullable" => "NO",	"data_type" => "INT", 		"character_maximum_length" => NULL),
		"archive_date"		=>array("is_nullable" => "YES",	"data_type" => "INT", 		"character_maximum_length" => NULL),
	);
	$txt_constraints[$prefix.$table_name."_pkey"]=array("constraint_type"=>"PRIMARY","table_name"=>$prefix.$table_name,"columns"=>array("resource_id"));

	//-------------------------------------------------------------------------------------------------------------------------
	
	$table_name="resource_approvals";
	$txt_tables[$prefix.$table_name]=array(
		"resource_id"	=>array("is_nullable" => "NO", 	"data_type" => "INT", 		"character_maximum_length" => NULL),
		"version_id"	=>array("is_nullable" => "NO", 	"data_type" => "INT", 		"character_maximum_length" => NULL),
		"group_id"		=>array("is_nullable" => "NO", 	"data_type" => "INT", 		"character_maximum_length" => NULL),
		"timestamp"		=>array("is_nullable" => "NO", 	"data_type" => "INT", 		"character_maximum_length" => NULL),
		"user_id"		=>array("is_nullable" => "NO", 	"data_type" => "INT", 		"character_maximum_length" => NULL),
	);
	$txt_constraints[$prefix.$table_name."_unique"]=array("constraint_type"=>"UNIQUE","table_name"=>$prefix.$table_name,"columns"=>array("resource_id","version_id","group_id"));

	//-------------------------------------------------------------------------------------------------------------------------

	$table_name="resource_log";
	$txt_tables[$prefix.$table_name]=array(
		"resource_id"	=>array("is_nullable" => "NO", 	"data_type" => "INT",		"character_maximum_length" => NULL),
		"version_id"	=>array("is_nullable" => "NO",	"data_type" => "INT",		"character_maximum_length" => NULL),
		"user_id"		=>array("is_nullable" => "NO",	"data_type" => "INT",		"character_maximum_length" => NULL),
		"timestamp"		=>array("is_nullable" => "NO",	"data_type" => "INT",		"character_maximum_length" => NULL),
		"event"			=>array("is_nullable" => "NO",	"data_type" => "SMALLINT",	"character_maximum_length" => NULL),
	);
	
	//-------------------------------------------------------------------------------------------------------------------------
	
	$table_name="resource_versions";
	$txt_tables[$prefix.$table_name]=array(
		"resource_id"		=>array("is_nullable" => "NO", 	"data_type" => "INT", 		"character_maximum_length" => NULL),
		"version_id"		=>array("is_nullable" => "NO", 	"data_type" => "INT", 		"character_maximum_length" => NULL),
		"node_id"			=>array("is_nullable" => "NO", 	"data_type" => "INT", 		"character_maximum_length" => NULL),
		"filename"			=>array("is_nullable" => "NO", 	"data_type" => "VARCHAR",	"character_maximum_length" => 100),
		"resource_name"		=>array("is_nullable" => "YES",	"data_type" => "VARCHAR",	"character_maximum_length" => 50),
		"file_type"			=>array("is_nullable" => "NO",	"data_type" => "CHAR",		"character_maximum_length" => 3),
		"time_created"		=>array("is_nullable" => "NO",	"data_type" => "INT", 		"character_maximum_length" => NULL),
		"status"			=>array("is_nullable" => "NO",	"data_type" => "SMALLINT",	"character_maximum_length" => NULL),
		"md5_sum"			=>array("is_nullable" => "NO",	"data_type" => "CHAR", 		"character_maximum_length" => 32),
		"notes"				=>array("is_nullable" => "NO",	"data_type" => "TEXT",		"character_maximum_length" => NULL),
	);
	$txt_constraints[$prefix.$table_name."_unique"]=array("constraint_type"=>"UNIQUE","table_name"=>$prefix.$table_name,"columns"=>array("resource_id","version_id"));
	
	//-------------------------------------------------------------------------------------------------------------------------

	$table_name="resource_tree";
	$txt_tables[$prefix.$table_name]=array(
		"node_id"		=>array("is_nullable" => "NO", 	"data_type" => "SERIAL", 		"character_maximum_length" => NULL),
		"node_name"		=>array("is_nullable" => "NO", 	"data_type" => "VARCHAR",		"character_maximum_length" => 50),
		"node_parent"	=>array("is_nullable" => "NO",	"data_type" => "INT", 			"character_maximum_length" => NULL),
	);
	$txt_constraints[$prefix.$table_name."_pkey"]=array("constraint_type"=>"PRIMARY","table_name"=>$prefix.$table_name,"columns"=>array("node_id"));
	
	//-------------------------------------------------------------------------------------------------------------------------
	
	$table_name="workflow";
	$txt_tables[$prefix.$table_name]=array(
		"workflow_id"	=>array("is_nullable" => "NO", 	"data_type" => "INT", 			"character_maximum_length" => NULL),
		"group_id"		=>array("is_nullable" => "NO", 	"data_type" => "INT",			"character_maximum_length" => NULL),
	);
	$txt_constraints[$prefix.$table_name."_unique"]=array("constraint_type"=>"UNIQUE","table_name"=>$prefix.$table_name,"columns"=>array("workflow_id","group_id"));
	
	//-------------------------------------------------------------------------------------------------------------------------
}

//********************************************************************************************************************************
// Initial data population


//****************************************************************************************************************
/*
I'm pretty resigned to the fact that stored procedures aren't going to be consistent across different
database technologies. If you can contribute to other technologies in this area, please email your
procedures to scott@strongholdcms.org. Thanks.
*/

if ($db_options["database_type"]=="postgres"){

$txt_functions["tree_path"]="
CREATE FUNCTION tree_path(INT, VARCHAR) RETURNS VARCHAR
    AS 'DECLARE
top_node_id ALIAS FOR $1;
db_table ALIAS FOR $2;
path varCHAR(500);
node int;
i int;
i_node_name varCHAR(255);
i_node_parent int;
itemrecord RECORD;

BEGIN

   i:=1;
   node:=top_node_id;
   path:='''';


   WHILE i<100 LOOP

      i_node_name:=NULL;
      i_node_parent:=NULL;

      FOR itemrecord IN EXECUTE (''SELECT node_name,node_parent FROM ''|| db_table ||'' AS t WHERE node_id=''||node) LOOP
         i_node_name:=itemrecord.node_name;
         i_node_parent:=itemrecord.node_parent;
      END LOOP;

      IF i_node_name IS NULL THEN
         RETURN (''busted'');
      END IF;

      path:=''/'' || i_node_name || path;

      IF i_node_parent=0 THEN
         RETURN (path);
      END IF;

      node := i_node_parent;

      i := i+1;
   END LOOP;

   RETURN path;

END '
    LANGUAGE plpgsql;
";
}
//------------------------------------------------------------------------------------------------------------------------
if ($db_options["database_type"]=="mssql"){

$txt_functions["tree_path"]="
CREATE FUNCTION {db_table_prefix}tree_path (
	@top_node_id INT
) RETURNS VARCHAR(100)
AS
BEGIN

	DECLARE @i INT
	DECLARE @node_id INT
	DECLARE @ret_val VARCHAR(100)
	DECLARE @r_node_id INT
	DECLARE @r_node_parent INT
	DECLARE @r_node_name VARCHAR(100)
	
	SET @i=0
	SET @node_id=@top_node_id
	SET @ret_val=''

	WHILE (@i<50) BEGIN
	
		DECLARE c CURSOR FOR SELECT node_id,node_name,node_parent FROM {db_table_prefix}resource_tree WHERE node_id=@node_id
		OPEN c
		FETCH NEXT FROM c INTO @r_node_id,@r_node_name,@r_node_parent

		SET @ret_val='/'+@r_node_name+@ret_val
	
		IF @r_node_parent = 0 BEGIN
			RETURN @ret_val
		END
	
		CLOSE c
		DEALLOCATE c

		SET @node_id=@r_node_parent
		SET @i = @i + 1
	END
	
	RETURN 'busted'
END
";
}



?>