<?
/*
Stronghold CMS - Installer
Copyright (C) 2008 Scott Barnett

This program is free software; you can redistribute it and/or
modify it under the terms of Version 3 of the GNU General Public
License as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
-------------------------------------------------------------------

This script has been setup at the CLI level because we can provide
more functionality, and you do not need to have a preconfigured apache
environment to get started.

*/
//limit this script to running from the console
if (php_sapi_name()!="cli"){ echo ("FATAL: This script cannot be invoked with this method.\n"); exit(); }

//fix the document root so the libraries work
$_SERVER["DOCUMENT_ROOT"]=dirname(dirname(realpath($_SERVER["PWD"]."/".$_SERVER["PHP_SELF"])));

//libraries
include ($_SERVER["DOCUMENT_ROOT"]."/config.php");
include ($_SERVER["DOCUMENT_ROOT"]."/_system/stronghold.php");
include ($_SERVER["DOCUMENT_ROOT"]."/_system/database.php");
//include ($_SERVER["DOCUMENT_ROOT"]."/_system/schema-matic.php"); //included by install_db.php
include ($_SERVER["DOCUMENT_ROOT"]."/_system/extended.php");
include ($_SERVER["DOCUMENT_ROOT"]."/_system/adLDAP.php");

//the user's path plus a couple of extras
$paths=$_SERVER["PATH"]."/usr/sbin/httpd:/usr/local/apache/bin:/usr/local/apache2/bin:/usr/local/apache2/modules:/usr/local/apache/modules:/usr/lib/apache2/modules";
$paths=array_unique(explode(":",$paths));

//read the config file into memory, we're going to overwrite it
$full_config=file($_SERVER["DOCUMENT_ROOT"]."/config.php");

//process the command line switches
foreach ($_SERVER["argv"] as $id => $val){
	if (strpos($val,"=")!=false){
		$bits=explode("=",prepare_str($val));
		if(count($bits)==2 && array_key_exists($bits[0],$CONFIG)){ $CONFIG[$bits[0]]=$bits[1]; }
		elseif ($bits[0]=="adldap_domain_controller"){ $CONFIG["adldap_domain_controllers"][0]=$bits[1]; }
	}
}

//print_r($CONFIG); exit();

$c="<?\n";

echo ("---------------------------------------------------------------------------\n");
echo ("Welcome to the Stronghold CMS Installer\n\n");

echo ("This program is free software; you can redistribute it and/or\n");
echo ("modify it under the terms of Version 3 of the GNU General Public\n");
echo ("License as published by the Free Software Foundation.\n\n");
echo ("A full copy of this licence can be found in the LICENCE.txt file\n");
echo ("in the root folder of this distribution, or type \"licence\" to\n");
echo ("read it now.\n");

//make them accept the licence agreement or bomb out
$answer="";
while ($answer!="yes"){
	if ($answer=="licence"){ readfile($_SERVER["DOCUMENT_ROOT"]."/LICENCE.txt"); }
	$answer=question("Do you accept the terms if this licence?\nyes, no, licence","no");
	if ($answer=="no"){ echo ("Sorry, you must accept the licence to use or distribute this software.\n"); exit(); }
}


//************************************************************************************************************
// let the user know what the installer is going to do
echo ("\nThis installer will attempt to:\n");
echo (" - Check your system dependencies\n");
echo (" - Collect your configuration settings (and test if applicable)\n");
echo (" - Write your configuration to disk\n");
echo (" - Install your database\n");
echo (" - Provide you with your apache configuration\n");
echo ("If you would like to perform these tasks manually, please check the\n");
echo ("installation documentation on www.strongholdcms.org\n\n");
echo (" *** This installer will give you default options. ***\n");
echo (" ***    Hit enter to accept the default option.    ***\n");

$answer=question("Would you like to continue? yes, no","no");
if ($answer!="yes"){ echo ("No worries.\n"); exit(); }


//************************************************************************************************************
echo ("\nChecking system dependencies.\n\n");

$missing_dependancy=false;

//write access to config.php file
echo ("*** Write access to config.php... ");
if (!is_writeable($_SERVER["DOCUMENT_ROOT"]."/config.php")){ echo ("FAILED\nI need write access to config.php\nYou could try 'chmod u+r ".$_SERVER["DOCUMENT_ROOT"]."/config.php'\n"); $missing_dependancy=true; } else { echo ("OK\n"); }

//apache binary
echo ("*** Apache... ");
$apache_binary=find_file("httpd",$paths); //try "httpd" first
if ($apache_binary===false){ $apache_binary=find_file("apache2",$paths); } //try "apache2" for debian
if ($apache_binary===false){ echo ("NOT FOUND\n"); $missing_dependancy=true; } else { echo ("FOUND - ".$apache_binary."\n"); }

//mod_auth_ntlm_winbind
echo ("*** mod_auth_ntlm_winbind... ");
$manw_binary=find_file("mod_auth_ntlm_winbind.so",$paths);
if ($manw_binary===false){ echo ("NOT FOUND\n"); $missing_dependancy=true; } else { echo ("FOUND - ".$manw_binary."\n"); }

//the php binary (kinda pointless, it has to be running to run this script!)
echo ("*** PHP... ");
$php_binary=find_file("php",$paths);
if ($php_binary===false){ echo ("NOT FOUND\n"); $missing_dependancy=true; } else { echo ("FOUND - ".$php_binary."\n"); }

//php compiled with ldap support
echo ("*** PHP LDAP Support ... ");
if (!function_exists("ldap_bind")){ echo ("NOT FOUND\n"); $missing_dependancy=true; } else { echo ("FOUND\n"); }

/*
echo ("OpenSSL... ");
$ssl_binary=find_file("openssl",$paths);
if ($ssl_binary===false){ echo ("NOT FOUND\n"); $missing_dependancy=true; } else { echo ("FOUND - ".$ssl_binary."\n"); }
*/

if ($missing_dependancy){ echo ("\nSorry, you will need to install any missing dependencies before you can continue.\n"); exit(); }

//************************************************************************************************************

echo ("---------------------------------------------------------------------------\n");
echo ("DATABASE SETTINGS\n");

//prompt them to setup their database environment
echo ("\nBefore I begin, you will need to create the empty database for\n");
echo ("me to connect to and set the permissions appropriately. I specifically\n");
echo ("won't do this for you for security reasons (and so I don't break\n");
echo ("anything if you accidently put in the wrong details).\n");

$db_type=question("Is your database setup and you're ready to continue?\n(If it's not I'm happy to wait!)\nyes, no","yes");
if ($answer!="yes"){ echo ("No worries, re-run me when you're ready to continue.\n"); exit(); }

$db_type=question("Please specify the type of database you'd like to use.\npostgres, mysql, orcale, mssql",$CONFIG["db_type"]);
set_option("db_type",$db_type,$full_config);

// We only check database dependencies when they select their DB type
$missing_dependancy=false;
if ($db_type=="postgres"){
	if (!function_exists("pg_connect")){ echo ("Sorry, your PHP installation isn't compiled with PostgreSQL support.\n"); $missing_dependancy=true; }
} elseif ($db_type=="mssql"){
	if (!function_exists("mssql_connect")){ echo ("Sorry, your PHP installation isn't compiled with Microsoft SQL Server support.\n"); $missing_dependancy=true; }
} elseif ($db_type=="mysql"){
	if (!function_exists("mysql_connect")){ echo ("Sorry, your PHP installation isn't compiled with MySQL support.\n"); $missing_dependancy=true; }
} elseif ($db_type=="oracle"){
	if (!function_exists("oci_connect")){ echo ("Sorry, your PHP installation isn't compiled with Oracle support.\n"); $missing_dependancy=true; }
} else {
	echo ("Unsupported database type (did you spell the option correctly?)."); exit();
}

if ($missing_dependancy){ echo ("\nSorry, you will need to correct any problems before you can continue.\n"); exit(); }

// get all their database settings and keep them looping until they get it right or overide the script 
$continue=false;
$db_server=$CONFIG["db_server"];
$db_database=$CONFIG["db_database"];
$db_username=$CONFIG["db_username"];
$db_password=$CONFIG["db_password"];

while (!$continue){
	$db_server=question("Please enter the network address of your database server",$db_server);
	$db_database=question("Please enter the name of the database",$db_database);
	$db_username=question("Please enter the database username",$db_username);
	$db_password=question("Please enter the database password",$db_password);
	
	echo ("\n*** Testing your database connection, this may take a moment... ");

	$db=false;
	if ($db_type=="postgres"){
		$db=new pg($db_server,$db_database,$db_username,$db_password);
	} elseif ($db_type=="mssql"){
		$db=new ms($db_server,$db_database,$db_username,$db_password);
	} elseif ($db_type=="mysql"){
		$db=new my($db_server,$db_database,$db_username,$db_password);
	} elseif ($db_type=="oracle"){
		$db=new ora($db_server,$db_database,$db_username,$db_password);
	}

	if (!$db->db){
		echo ("Connection failed, please check your database settings and try again\n");
	} else {
		echo ("Connection Succeeded!\n");
		set_option("db_server",$db_server,$full_config);
		set_option("db_database",$db_database,$full_config);
		set_option("db_username",$db_username,$full_config);
		set_option("db_password",$db_password,$full_config);
		$continue=true;
	}
}

$db_table_prefix=question("Please enter the system's table prefix",$CONFIG["db_table_prefix"]);
set_option("db_table_prefix",$db_table_prefix,$full_config);

$db_common_prefix=question("Some database resources in Stronghold are shared between domains.\nPlease enter a table prefix for the common resources",$CONFIG["db_common_prefix"]);
set_option("db_common_prefix",$db_common_prefix,$full_config);


echo ("---------------------------------------------------------------------------\n");
echo ("SYSTEM SETTINGS\n");

$system=array();
$security=array();

$system["system_name"]=question("Please enter the name of your system",$CONFIG["system_name"]);
set_option("system_name",$system["system_name"],$full_config);

$system["system_contact"]=question("Please enter the name of the system technical contact",$CONFIG["system_contact"]);
set_option("system_contact",$system["system_contact"],$full_config);

$system["system_email"]=question("Please enter the email address of the technical contact",$CONFIG["system_email"]);
set_option("system_email",$system["system_email"],$full_config);

$system["system_session"]=question("Please enter the prefix to use for session variables",$db_table_prefix);
set_option("system_session",$system["system_session"],$full_config);

$system["system_domain"]=question("Please enter the HTTP domain of the system",$CONFIG["system_domain"]);
set_option("system_domain",$system["system_domain"],$full_config);
set_option("system_path","/opt/stronghold/".$system["system_domain"],$full_config); //set the path as well

$system["system_tech"]=question("Please enter the name of a technical contact used to refer users,\neg. 'encounterd \"blah\" error, please contact \"system_tech\".'",$CONFIG["system_tech"]);
set_option("system_tech",$system["system_tech"],$full_config);

$detected_ip=server_ip();
if (!$detected_ip){ $detected_ip="127.0.0.1"; }
$ip_address=question ("Please enter the IP address you want to use for your SSL host",$detected_ip);

/*
$answer=question("Stronghold uses a datastore on the filesystem to store binaries (eg. pdf's, jpg's, etc)\nPlease enter the path for your datastore.",$CONFIG["system_datastore"]);
set_option("system_datastore",$answer,$full_config);
*/

echo ("---------------------------------------------------------------------------\n");
echo ("ACTIVE DIRECTORY SETTINGS\n");


$security["security_auth"]=question("Stronghold can use several different methods for authenticating users.
   ntlm - Seamless authentication with Active Directory, and uses adldap for fallback
   adldap - Prompted authentication with Active Directory
   database - Prompted authentication with the database
Please enter your prefered authentication type",$CONFIG["security_auth"]);
set_option("security_auth",$security["security_auth"],$full_config);


if ($security["security_auth"]=="adldap" || $security["security_auth"]=="ntlm"){
	$security["security_domain"]=question("Please enter the name of your Microsoft Windows domain (eg. MYDOMAIN)",$CONFIG["security_domain"]);
	set_option("security_domain",$security["security_domain"],$full_config);
	
	/*
	adLDAP doesn't support graceful failure yet (sigh), which is a bit dodgy cause it's my code.
	Until it does this script is just going to bomb out if the connection fails.
	*/
	
	//$continue=false;
	//while (!$continue){
	
		if ($CONFIG["adldap_account_suffix"]=="@mydomain.local"){ $CONFIG["adldap_account_suffix"]="@".strtolower($security["security_domain"]).".local"; } //if unconfigured
		$security["account_suffix"]=question("Please enter the suffix used on your Active Directory usernames (eg. @mydomain.local)",$CONFIG["adldap_account_suffix"]);
		$security["base_dn"]=question("Please enter the your Active Directory base DN (eg. DC=mydomain,DC=local)","DC=".implode(",DC=",explode(".",strtolower(substr($security["account_suffix"],1)))));
		$security["domain_controller"]=question("Please enter a domain controller (eg. dc01.mydomain.local)",$CONFIG["adldap_domain_controllers"][0]);
		$security["ad_username"]=question("I need an normal AD user account to set as a service account to query AD.\nPlease enter the service account username",$CONFIG["adldap_ad_username"]);
		$security["ad_password"]=question("Please enter the service account password",$CONFIG["adldap_ad_password"]);
		
		$options=array(
			"account_suffix"=>$security["account_suffix"],
			"base_dn"=>$security["base_dn"],
			"domain_controllers"=>array($security["domain_controller"]),
			"ad_username"=>$security["ad_username"],
			"ad_password"=>$security["ad_password"],
		);
	//pre($options);
		$adldap=new adldap($options);
		if (!$adldap){
			echo ("Connection failed, please check your AD settings and try again\n");
		} else {
			echo ("AD Connection Succeeded!\n");
			set_option("adldap_account_suffix",$security["account_suffix"],$full_config);
			set_option("adldap_base_dn",$security["base_dn"],$full_config);
			set_option("adldap_domain_controllers","array(\"".$security["domain_controller"]."\")",$full_config);
			set_option("adldap_ad_username",$security["ad_username"],$full_config);
			set_option("adldap_ad_password",$security["ad_password"],$full_config);
			$continue=true;
		}
	//}
}

$answer=question("I can setup a super-secret-squirrel password so you can login as any user\n(very handy for debugging).\nLeave this blank to disable it.",$CONFIG["security_pass"]);
set_option("security_pass",$answer,$full_config);


echo ("---------------------------------------------------------------------------\n");
echo ("ADMINISTRATOR ACCOUNT\n");

echo ("\nI need to setup your first account for you so you can login to the system.\n");
echo ("*** This needs to be *your* Active Directory username or I can't log you in.***\n");
echo ("You can setup more accounts once you've logged in.\n");

//defaults
$admin_account=array();
$admin_account["username"]="admin";
$admin_account["firstname"]="System";
$admin_account["surname"]="Adminstrator";
$admin_account["email"]=$system["system_email"];

//collect the information
$admin_account["username"]=question("Please enter your username",$admin_account["username"]);
$admin_account["firstname"]=question("Please enter your firstname",$admin_account["firstname"]);
$admin_account["surname"]=question("Please enter your surname",$admin_account["surname"]);
$admin_account["email"]=question("Please enter your email address",$admin_account["email"]); //****************************

/*
// version 1 does not support manaul password authentication, you must have a windows domain
//set the password
$continue=false;
while (!$continue){
	$password1=question("Please enter a password for your admin account (attempt 1)",NULL);
	$password2=question("Please enter a password for your admin account (attempt 2)",NULL);
	if ($password1!=$password2){ echo ("Passwords do not match, please try again.\n"); }
	elseif (strlen($password1)<6){ echo ("Password must be at least 6 characters long, please try again.\n"); }
	elseif (strlen($password1)<6){ echo ("Password must be at least 6 characters long, please try again.\n"); }
	elseif ($password1!=prepare_str($password1)){ echo ("Your password contains illegal characters, please try again.\n"); }
	else { $continue=true; }
}
$admin_account["password"]=$password1;
*/
$admin_account["password"]="";

echo ("\nI need to setup a group account. You can add more groups or change the\n");
echo ("name of this group at any time.");
$group_name=question("Please enter a group account name",$system["system_tech"]);



echo ("---------------------------------------------------------------------------\n");
echo ("INSTALLING STRONGHOLD\n\n");

//write the config to disk
echo ("Writing your configuration to disk... ");
$length=file_put_contents("../config.php",$full_config);
if (!$length){ echo ("Failed (check your file system permissions?)\n"); exit(); } //false or 0, writing nothing is no good either
echo ("Done\n");

//--------------------------------------------------------------------------------
// Database

if (common_exists($db,$db_common_prefix)){ $populate_common=false; } else { $populate_common=true; }
$db_options=array(
	"prefixes"=>array($db_table_prefix),
	"db_common_prefix"=>$db_common_prefix,
	"test"=>false,
	"drop_unknown_tables"=>false,
	"drop_unknown_constraints"=>false,
	"database_type"=>$db_type,
	"populate_database"=>true,
	"populate_common"=>$populate_common,
);
include ("install_db.php");

//--------------------------------------------------------------------------------
// Datastore
echo ("Setting permissions...\n");

$command="mkdir /opt/stronghold/".$system["system_domain"]."/_datastore";
echo ($command."\n"); system ($command);
$command="chmod 755 /opt/stronghold/".$system["system_domain"]." -R";
echo ($command."\n"); system ($command);
$command="chgrp www-data /opt/stronghold/".$system["system_domain"]." -R";
echo ($command."\n"); system ($command);
$command="chmod 775 /opt/stronghold/".$system["system_domain"]."/_datastore -R";
echo ($command."\n"); system ($command);
$command="chgrp \"".$security["security_domain"]."+Domain Users\" /opt/stronghold/".$system["system_domain"]."/portal -R";
echo ($command."\n"); system ($command);

echo ("DONE\n");

//--------------------------------------------------------------------------------
// Apache

echo ("---------------------------------------------------------------------------\n");
echo ("APACHE CONFIGURATION\n\n");


include ("apache-config.php");
$http_config=apache_http_config("*",$admin_account["email"],$_SERVER["DOCUMENT_ROOT"],$system["system_domain"],$system["system_session"]);
$ssl_config=apache_ssl_config($ip_address,$admin_account["email"],$_SERVER["DOCUMENT_ROOT"],$system["system_domain"],$system["system_session"]);
$full_config=$http_config."\n\n".$ssl_config;

echo ($full_config);

$configured=false;
$hosts_conf="/etc/apache2/hosts.conf";
if (!is_writeable("/etc/apache2")){
	echo ("*** Apache configuration directory is not writeable. ***\n");
	echo ("*** You will need to save your Apache configuration manually. ***\n");
} else {
	if (file_exists($hosts_conf)){
		echo ("*** Apache hosts.conf already exists. ***\n");
		echo ("*** You will need to save your Apache configuration manually. ***\n");
	} else {
		//write the file to disk
		$result=write_file($hosts_conf,$full_config);
		
		/*
		DONT FORGET NameVirtualhost *
		*/
		if ($result===true){
			//we only do this if we've configured a system or we can break installations
			if (strpos(file_get_contents("/etc/apache2/httpd.conf"),$hosts_conf)===false){
				system ("echo \"Include ".$hosts_conf."\" >> /etc/apache2/httpd.conf");
				echo ("Apache configured successfully!\n");
				$configured=true;
				
				echo ("Attempting to restart Apache...\n");
				system ("apachectl restart");
			} else {
				echo ("*** hosts.conf reference already in httpd.conf ***\n");
			}
		} else {
			echo ($result);
		}
	}
}

//write a file to disk, returns true or an error message on fail.
function write_file($file,$contents){

	if (!$handle = fopen($file,'w')){ return ("*** Cannot open file (".$file.") ***\n"); } //open the file
	if (fwrite($handle,$contents) === FALSE){ return ("*** Cannot write to file (".$file.") ***\n"); } //write it
	fclose($handle);
	
	return (true);
}

//write the file to disk
$filename="/tmp/apache.config";
if (!$handle = fopen($filename, 'w')){ echo ("*** Cannot open file (".$filename.") ***\n"); exit; } //open the file
if (fwrite($handle,$full_config) === FALSE){ echo ("*** Cannot write to file (".$filename.") ***\n"); exit(); } //write it
fclose($handle);

//****************************************************************************************************

echo ("\n---------------------------------------------------------------------------\n");
echo ("\nThankyou for installing Stronghold CMS!\n\n");

if (!$configured){
	echo ("**********************************************************************\n");
	echo ("You need to add the Apache VirtualHost's to your Apache configuration\n");
	echo ("file/s and once done, you can restart Apache with 'apachectl restart'.\n");
	echo ("**********************************************************************\n");
}
echo ("Your apache configuration may need to be tweaked for your individual\n");
echo ("environment. Incase you're running on a console and can't scroll\n");
echo ("backwards, I've written a copy to \"/tmp/apache.config\".\n\n");

echo ("You can access your CMS at:\n");
echo ("https://".$system["system_domain"]."/stronghold/login.php\n\n");

//****************************************************************************************************


//function check if the common area already exists
function common_exists($db,$db_common_prefix){
	//just check for the refs table
	$sql="SELECT table_name FROM information_schema.tables WHERE table_name='".$db_common_prefix."refs'";
	$rs=$db->query($sql);
	if (count($rs)==1){ return (true); }
	return (false);
}


//set an option in the file string, returns false if it fails
function set_option($label,$value,&$file){
	$c=count($file);
	
	//check each line for the string and update it
	for ($i=0; $i<$c; $i++){
		$line=$file[$i];
		if (strpos($line,"\$CONFIG[\"".$label."\"]")!==false){
			$comment=substr($line,strpos($line,";")); //preserve the comment
			if (substr($value,0,5)=="array"){ //if it's an array, leave out the string quotation marks
				$file[$i]="\$CONFIG[\"".$label."\"]=".$value.$comment;
			} else {
				$file[$i]="\$CONFIG[\"".$label."\"]=\"".$value."\"".$comment;
			}
			return (true);	
		}
	}
	
	return(false);
}

//returns the path if found or false
function find_file($binary,$paths){
	foreach ($paths as $id => $path){
		if (file_exists($path."/".$binary)){ return ($path."/".$binary); }
	}
	return (false);
}

//ask a question, return the answer
function question ($question,$default){
	//fwrite(STDOUT,$question." [".$default."] => "
	echo ("\n".$question." [".$default."] => ");
	$response=fgets(STDIN);
	//$f=popen("read; echo \$REPLY","r");
	//$response=fgets($f,100);
	//pclose($f);
	$response=trim($response);
	if ($response==NULL){ return ($default); }
	return ($response);
}

function server_ip(){
	$result=trim(exec ("ifconfig |grep \"inet addr\" |grep -v 127.0.0.1"));
	if (!$result){ return (false); }
	$result=explode(" ",$result);

	foreach ($result as $id => $val){
		if (substr($val,0,5)=="addr:"){
			$ip=substr($val,5);
			if (count(explode(".",$ip))==4){ return ($ip); }
		}
	}	
	return (false);
}
?>
