<?
/*
Stronghold CMS - Stronghold Database Installer
Copyright (C) 2008 Scott Barnett

This program is free software; you can redistribute it and/or
modify it under the terms of Version 3 of the GNU General Public
License as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

-------------------------------------------------------------------
MUST BE RUN FROM THE COMMAND LINE (security reasons)

*/

//fix the document root so the libraries work
//$_SERVER["DOCUMENT_ROOT"]=dirname(dirname(realpath($_SERVER["PWD"]."/".$_SERVER["PHP_SELF"])));

//main config file
//include ($_SERVER["DOCUMENT_ROOT"]."/config.php");

// You shouldn't need to edit anything below this line
//*****************************************************************************************************************************

//fix the document root so the libraries work
$_SERVER["DOCUMENT_ROOT"]=dirname(dirname(realpath($_SERVER["PWD"]."/".$_SERVER["PHP_SELF"])));

//set the default options if they haven't specified any
if (!is_array($db_options)){
	include ($_SERVER["DOCUMENT_ROOT"]."/config.php");
	$db_options=array(
		"prefixes"=>array($CONFIG["db_table_prefix"]),
		"db_common_prefix"=>$CONFIG["db_common_prefix"],
		"test"=>true,
		"drop_unknown_tables"=>false,
		"drop_unknown_constraints"=>false,
		"database_type"=>$CONFIG["db_type"],
		"populate_database"=>false,
		"populate_common"=>false,
	);
	
	//allow command line switches
	foreach ($_SERVER["argv"] as $id => $val){
		if (strpos($val,"=")!=false){
			$bits=explode("=",$val);
			if(count($bits)==2 && array_key_exists($bits[0],$db_options)){ $db_options[$bits[0]]=$bits[1]; }
		}
	}
}

print_r($db_options);

//connect to the database if not already initiated
if (!$db){
	include ($_SERVER["DOCUMENT_ROOT"]."/_system/database.php");
	//database connection
	if ($CONFIG["db_type"]=="postgres"){ $db=new pg($CONFIG["db_server"],$CONFIG["db_database"],$CONFIG["db_username"],$CONFIG["db_password"]); }
	elseif ($CONFIG["db_type"]=="mysql"){ $db=new my($CONFIG["db_server"],$CONFIG["db_database"],$CONFIG["db_username"],$CONFIG["db_password"]); }
	elseif ($CONFIG["db_type"]=="mssql"){ $db=new ms($CONFIG["db_server"],$CONFIG["db_database"],$CONFIG["db_username"],$CONFIG["db_password"]); }
	elseif ($CONFIG["db_type"]=="oracle"){ $db=new ora($CONFIG["db_server"],$CONFIG["db_database"],$CONFIG["db_username"],$CONFIG["db_password"]); }
	else { echo ("FATAL: Database Installer Fault: Unknown db_type\n"); exit(); }
}

//limit this script to running from the console
if (php_sapi_name()!="cli"){ echo ("FATAL: This script cannot be invoked with this method.\n"); exit(); }

//libraries
//include_once ($_SERVER["DOCUMENT_ROOT"]."/_system/stronghold.php"); //(nasty include once!)
include ($_SERVER["DOCUMENT_ROOT"]."/_system/schema-matic.php");
include ($_SERVER["DOCUMENT_ROOT"]."/_system/resource.php");
include ($_SERVER["DOCUMENT_ROOT"]."/_modules/text.php"); //we need the text module for the home page

$now=time();

//-----------------------------------------------------------------------------
//load the schema
echo ("Loading the database schema...\n");
include ($_SERVER["DOCUMENT_ROOT"]."/_tools/db_schema.php");
echo ("DONE\n");

//-----------------------------------------------------------------------------
//install the database
echo ("Installing database...\n");
install_schema($db,$txt_tables,$txt_constraints,$txt_functions,$db_options);
echo ("DONE\n");

//-----------------------------------------------------------------------------
//populate the common tables

if ($db_options["populate_common"]){
	$populate=array();
	echo ("Populating common...\n");
	
	//editlog definitions
	$populate[$db_common_prefix."refs"][]=array("ref"=>"'editlog'","ref_id"=>"1","ref_val"=>"'UPDATE'");
	$populate[$db_common_prefix."refs"][]=array("ref"=>"'editlog'","ref_id"=>"2","ref_val"=>"'ARCHIVE'");
	$populate[$db_common_prefix."refs"][]=array("ref"=>"'editlog'","ref_id"=>"3","ref_val"=>"'APPROVE'");
	$populate[$db_common_prefix."refs"][]=array("ref"=>"'editlog'","ref_id"=>"4","ref_val"=>"'LIVE'");
	$populate[$db_common_prefix."refs"][]=array("ref"=>"'editlog'","ref_id"=>"5","ref_val"=>"'REV_NOTE'");
	$populate[$db_common_prefix."refs"][]=array("ref"=>"'editlog'","ref_id"=>"6","ref_val"=>"'SUBMIT'");
	$populate[$db_common_prefix."refs"][]=array("ref"=>"'editlog'","ref_id"=>"7","ref_val"=>"'CREATE'");
	$populate[$db_common_prefix."refs"][]=array("ref"=>"'editlog'","ref_id"=>"8","ref_val"=>"'CANCEL'");
	
	//user access definitions
	$populate[$db_common_prefix."refs"][]=array("ref"=>"'useraccess'","ref_id"=>"1","ref_val"=>"'Editor'");
	$populate[$db_common_prefix."refs"][]=array("ref"=>"'useraccess'","ref_id"=>"2","ref_val"=>"'Approver'");
	$populate[$db_common_prefix."refs"][]=array("ref"=>"'useraccess'","ref_id"=>"3","ref_val"=>"'Custodian'");
	$populate[$db_common_prefix."refs"][]=array("ref"=>"'useraccess'","ref_id"=>"4","ref_val"=>"'Administrator'");

	//status definitions
	$populate[$db_common_prefix."refs"][]=array("ref"=>"'status'","ref_id"=>"1","ref_val"=>"'New'");
	$populate[$db_common_prefix."refs"][]=array("ref"=>"'status'","ref_id"=>"2","ref_val"=>"'Pending'");
	$populate[$db_common_prefix."refs"][]=array("ref"=>"'status'","ref_id"=>"3","ref_val"=>"'Approved'");
	$populate[$db_common_prefix."refs"][]=array("ref"=>"'status'","ref_id"=>"4","ref_val"=>"'Cancelled'");

	//common sequences (emulated via an SQL function)
	$populate[$db_common_prefix."seq"][]=array("seq_name"=>"'".$db_common_prefix."users_user_id_seq'","seq_val"=>"0");
	
	//insert the data
	populate_tables($db,$populate,$db_options["test"]);
}

//-----------------------------------------------------------------------------

if ($db_options["populate_database"]){

	foreach ($db_options["prefixes"] as $id => $prefix){
		$populate=array();
		//installed modules
		$populate[$prefix."modules"][]=array("module_id"=>1,"module_name"=>"'text'","module_desc"=>"'Generic Text Module'","time_installed"=>$now);
		$populate[$prefix."modules"][]=array("module_id"=>2,"module_name"=>"'plugin'","module_desc"=>"'Plugin Module'","time_installed"=>$now);
		
		//sequences for the system (ignored in postgres and oracle)
		$populate[$db_common_prefix."seq"][]=array("seq_name"=>"'".$db_table_prefix."groups_group_id_seq'","seq_val"=>"0");
		$populate[$db_common_prefix."seq"][]=array("seq_name"=>"'".$db_table_prefix."pages_page_id_seq'","seq_val"=>"0");
		$populate[$db_common_prefix."seq"][]=array("seq_name"=>"'".$db_table_prefix."resource_tree_node_id_seq'","seq_val"=>"0");
		$populate[$db_common_prefix."seq"][]=array("seq_name"=>"'".$db_table_prefix."resources_resource_id_seq'","seq_val"=>"0");
		
		populate_tables($db,$populate,$db_options["test"]);
		
		//admin account
		echo ("Creating the admin account... ");
		$sql="SELECT user_id FROM ".$db_common_prefix."users WHERE UPPER(username)='".strtoupper($admin_account["username"])."'";
		$rs=$db->query($sql);
		if (count($rs)!=0){
			//already exists
			$admin_id=$rs[0]["user_id"];
		} else {
			//create a new account
			$fields=array(
				"username"=>$admin_account["username"],
				"firstname"=>$admin_account["firstname"],
				"surname"=>$admin_account["surname"],
				"email"=>$admin_account["email"],
				"access"=>4,
				"active"=>1,
				"editor_size"=>"700x400",
			);
			
			$admin_id=user_create($db,$db_common_prefix,$fields);
			if (!$admin_id){ echo ("FAILED\n"); exit(); } else { echo ("DONE\n"); }
		}
		echo ("DONE\n");
		
		
		//setup the first group
		echo ("Creating a group... ");
		$group_id=group_create($db,$prefix,$group_name);
		if (!$group_id){ echo ("FAILED\n"); exit(); } else { echo ("DONE\n"); }
		
		//setup the first group
		echo ("Adding the admin account to the group... ");
		$result=group_add_user($db,$prefix,$group_id,$admin_id,4);
		if ($result===false){ echo ("FAILED\n"); exit(); } else { echo ("DONE\n"); }
		
		//setup the first workflow
		echo ("Creating the default workflow... ");
		$result=workflow_add_group($db,$prefix,1,1);
		if ($result===false){ echo ("FAILED\n"); exit(); } else { echo ("DONE\n"); }

		//create the resource tree folders
		echo ("Creating the resource tree default structure... ");
		$resource=new resource($db,$prefix);
		$result=$resource->node_create("resources",0,$node_id);
		if ($result!==true){ echo ("FATAL: Could not create resources node - ".$result); exit(); }
		$result=$resource->node_create("docs",1,$node_id);
		if ($result!==true){ echo ("FATAL: Could not create docs node - ".$result); exit(); }
		$result=$resource->node_create("images",1,$node_id);
		if ($result!==true){ echo ("FATAL: Could not create images node - ".$result); exit(); }
		$resource=NULL; //all done, delete the object
		echo ("DONE\n");

		//create the home page
		echo ("Creating the home page... ");
		$result=create_page($db,$prefix,$admin_id,home_page());
		if ($result!==true){ echo ("FAILED - ".$result."\n"); exit(); } else { echo ("DONE\n"); }

	}
}

//just returns an array for the home page
function home_page(){

	$page=array(	
		"time_created"=>time(),
		"parent"=>"0",
		"workflow_id"=>1,
		"link"=>"NULL",
		"module_id"=>1,
		"archive_date"=>"NULL",
		"title"=>"Home Page",
		"short_title"=>"Home",
		"body_text"=>"
<p>Thankyou for installing Stronghold CMS! Some things you may like to know:</p>

<ul>
<li><strong>Website Skin</strong><br>
Stronghold CMS is really designed to be as visually flexible as possible. As such, the default website
skin for a new CMS is very basic, it&#39;s designed to be a blank slate for you to get creative with. :)
In the &quot;includes&quot; folder are two files, template.php and style.css. They provide some basic and
easy to understand examples for you to work from.
<br><br></li>

<li><strong>Default Workflow</strong><br>
The default workflow has been established as just your group. You should give some thought to which groups in your
organisation should be needed to approve content. In most organisations, it&#39;s both Public Relations and the IT department.
<br><br></li>

<li><strong>Backups</strong><br>
The installer has not established a backup routine for you. You should look into this as part of your organisational backup
scheme. You can find information that may assist in the
<a href=\"http://www.strongholdcms.org/wiki/doku.php?id=administration:backups\" target=\"_blank\">Administration section of the Stronghold CMS Manual</a>.
<br><br></li>

<li><strong>Database Access</strong><br>
If you&#39;ve used the automated installation, PostgreSQL is accessible via <a href=\"/phpPgAdmin/\" target=\"_blank\">phpPgAdmin</a>.<br>
Username: postgres<br>
Password: (your database password)
<br><br></li>

<li><strong>Colouring Outside the Square</strong><br>
<a href=\"http://www.strongholdcms.org/wiki/\" target=\"_blank\">The Stronghold CMS Manual</a> is a great place to look for
formal information and structured information, you may even be able to help us improve it :)<br>
<a href=\"http://www.strongholdcms.org/wiki/doku.php?id=administration:backups\" target=\"_blank\">The Stronghold CMS Forums</a>
are a great place to discuss everything Content Management with system owners, developers, and graphic artists.
<br><br></li>
</ul>
"
	);
	return ($page);
}

?>