<?
/*
Stronghold CMS - Resource Importer
Copyright (C) 2008 Scott Barnett

This program is free software; you can redistribute it and/or
modify it under the terms of Version 3 of the GNU General Public
License as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
*/

$base_path="";
// automatically apply approvals to the resource, set to false not to disable
// must be an array of [group_id] => [user_id]
$approvals=array(1=>1,2=>4);

//if you want the importer to automatically create folders that don't exist
$create_folders=true;

//permissions to set on the imported files when you're done
$permissions="apache.apache";

//**************************************************************************************************
$argv=$_SERVER["argv"];

if (php_sapi_name()!="cli"){ echo ("FATAL: This script cannot be invoked with this method.\n"); exit(); }
if ((count($argv)<2) || in_array("--help",$argv)){
	echo ("Usage: resource_import.php [options] [resource tree path] [filenames]\n");
	echo ("       -c           Create folders that don't exist (except the resource tree path)\n");
	echo ("       -p           Create the resource tree path if it doesn't exist\n");
	echo ("       -rp          Recursive, preserve paths - imports into the corresponding folders\n");
	echo ("       -rf          Recursive, flat - imports into a single folder\n");
	echo ("       -d           Delete successfully imported files\n\n");
	echo ("Options must be specified individually. eg, [-rp -d] is ok, [-rpd] is not supported yet\n");
	echo ("Default is a flat, non-recursive import.\n\n");
	echo ("          This resource_import does not have Super Cow Powers\n\n");
	exit();
}

//fix the document_root so the libraries work
$base_path=dirname(dirname($_SERVER["PHP_SELF"]));
$_SERVER["DOCUMENT_ROOT"]=$base_path; 

//libraries
include ($_SERVER["DOCUMENT_ROOT"]."/_system/stronghold.php");
include ($_SERVER["DOCUMENT_ROOT"]."/_system/resource.php");

//create a resource handler
$resource=new resource($db,$CONFIG["db_table_prefix"]);

//set some modes based on command line options
$options=array();
$options["recursive_preserve"]=check_option("-rp",$argv);
$options["recursive_flat"]=check_option("-rf",$argv);
$options["delete_files"]=check_option("-d",$argv);
$options["create_folders"]=check_option("-c",$argv);
$options["create_path"]=check_option("-p",$argv);

//pre($options); pre($argv); exit();
//-------------------------------------------------------------------------------------------------

//get rid of the first element, it's the script name (resource_import.php)
unset ($argv[0]);

//the resource tree path is the next element in the array
$resource_tree_path=array_shift($argv);

//figure out the path to the resource
if (strpos($resource_tree_path,"/resources")!==0){ echo ("FATAL: Invalid import container (have you specified the resource tree path?) -> ".$resource_tree_path."\n"); exit(); }

//locate the node
$resource_tree_node_id=$resource->path_node($resource_tree_path);
if (!$resource_tree_node_id){
	if ($options["create_path"]){
		//echo ($resource_tree_path."!!!!!"); exit();
		$resource_tree_node_id=locate_node($resource,$resource_tree_path,true);
		
	} else {
		echo ("FATAL: Import container not found -> ".$resource_tree_path."\n"); exit();
	}
}

//$resource_tree_node_id=locate_node($resource,$resource_tree_path,false); //we don't create the parent intentionally incase they mispell it
//

//add all the files to the list of paths
$paths=array();
foreach ($argv as $id => $path){
	$path=prepare_str($path);
	if ($path[0]!="/"){ $path=$_SERVER["PWD"]."/".$path; } //detect relative paths and change them to absolute

	//run some basic checks
	if (!file_exists($path)){ echo ("WARNING: Cannot find file, ignoring -> ".$path."\n"); }
	elseif (!is_file($path)){
		//if it's a folder, add it if we're using recursion
		if ($options["recursive_flat"] || $options["recursive_preseve"]){ $paths=array_merge($paths,import_files($path)); }
	} 
	elseif (!file_type($path)) { echo ("WARNING: Ignoring file with unknown filetype -> ".$path."\n"); }
	else { $paths[]=$path; }
}
//pre($paths); exit();

//create all the resources
foreach ($paths as $id => $source_file){
	
	//the "corporate user" provision, trim filenames with more than 50 characters
	$filename=substr($source_file,(strrpos($source_file,"/")+1)); //ditch the path
	if (strlen($filename)>100){
		echo ("WARNING: Filename is longer than 100 chars, will be shortened\n");
		echo ("  Orginal -> ".$filename."\n");
		echo ("  Short   -> ".trim_filename($filename)."\n");
	}

	//locate the parent, create's it if the option is specified
	$node_id=$resource_tree_node_id;
	if ($options["recursive_preseve"]){ $node_id=locate_node($resource,$source_file,$options["create_folders"]); }

	//create the resource
//	echo ($node_id." ".$source_file); exit();
	$result=$resource->create($node_id,$source_file,$notes,"shell");
	if ($result!==true){ //if it returned an error
		echo ("WARNING: Resource creation failed - ".$result."\n");
	} else {
		$resource_id=$resource->new_resource_id;
		if (!$resource_id){ echo ("FATAL: Could not determine new resource id\n"); exit(); }
		echo ("Successfully created new resource -> ".$resource_id."\n");
		
		//update any link references (this is left here commented out for importing legacy systems)
		/*
		echo ("Replacing references to old document - ");
		$search=substr($source_file,strpos($source_file,"/resources"));
		$replace="/resources/".$resource_id.".".file_type($source_file);
		$sql="UPDATE ".$CONFIG["db_table_prefix"]."module_text SET body_text=REPLACE(body_text,'".$search."','".$replace."') WHERE body_text LIKE '%".$search."%'";
		$db->query($sql);
		echo ($sql."\n");
		*/
		
		//approve the resource
		if ($approvals){
			foreach ($approvals as $group_id => $user_id){
				$result=$resource->approve($resource_id,1,$group_id,$user_id);
				if (!$result){ echo ("FATAL: Resource approval failed\n"); exit(); }
				echo ("Approved resource ".$resource_id.".1 for group ".$group_id.", user ".$user_id."\n");
			}
		}
		
		//delete the files if specified
		if ($delete_files){ echo ("WARNING: Deleted file - ".$filename."\n"); }
	}
}

//apply permissions
$chown_string="chown ".$permissions." ".$CONFIG["system_datastore"]." -R";
echo ($chown_string."\n");
system ($chown_string);


//*******************************************************************************************************


/*
find the node id, create it if it doesn't exist
 $resource = the handle to the resource object
 $path = the full path of the node in the resource tree
 $create_folders = whether to create the folder if it doesn't exist (true or false)
*/
function locate_node($resource,$path,$create_folders){

	//break up the path and check it's ok
	$bits=explode("/",$path);
	$stop=array_search("resources",$bits);
	if ($stop===false){ echo ("FATAL: Could not locate resource folder in folder verification\n"); exit(); }

	//create the relative path (disregard what we don't need)
	$import_path=array();
	$bit_count=count($bits)-1; //ditch the last token, it's the file name
	for ($i=$stop; $i<$bit_count; $i++){ $import_path[]=$bits[$i]; }
	$relative_path="/".implode("/",$import_path);

	//try and locate the node id, return if we do (should happen the majority of the time)
//echo ($relative_path); exit();
	$node_id=$resource->path_node($relative_path);
	if ($node_id){ return ($node_id); }
	//pre($import_path); exit();

	//we're dealing with a node that doesn't exist
	if ($create_folders){
		//create the folder/s recursively
		$current_path="";
		
		foreach ($import_path as $id => $node_name){
			$current_path.="/".$node_name;
			$node_id=$resource->path_node($current_path); //find the node for the path
			if ($node_id===false){ //if node doesn't exist
				$result=$resource->node_create($node_name,$parent_node,$created_node_id);
				if ($result!==true){ echo ("FATAL: Node creation failed -> ".$current_path."\n"); exit(); }
				echo ("WARNING: Created folder -> ".$current_path."\n");
				$node_id=$created_node_id;
			}
			$parent_node=$node_id;
		}
		
	} else {
		//bomb out cause it doesn't exist
		echo ("FATAL: Path does not exist -> ".$path."\n"); exit();
	}

	//find the node
	$node_id=$resource->path_node($relative_path);
	
	return ($node_id);
}

//return true or false if the option exists, and remove it from the array
function check_option($option,&$argv){
	$pos=array_search($option,$argv);
	if ($pos!==false){
		unset($argv[$pos]); //remove the option from the array
		return (true);
	}
	return (false);
}

//return an array of files in a folder (recursive)
function import_files($folder){

	$list=array();
	if (is_dir($folder)) {
		if ($dh = opendir($folder)) {
			while (($file = readdir($dh)) !== false) {
				$path=$folder."/".$file;
				if ($file[0]!="."){ //ignore dir references and hidden files
					if (is_dir($path)){
						$list=array_merge($list,import_files($path));
					} else {
						$list[]=$path;
					}
				} 
			}
			closedir($dh);
		} else {
			echo ("FATAL: Failed to open folder for reading -> ".$folder."\n");
		}
	}
	
	return ($list);
}


//system ($_SERVER["_"]." ".$_SERVER["PHP_SELF"]." /resources/docs ".$path."/* > /tmp/import-".randomstring().".txt &");
?>
