<?
/*
Stronghold CMS - Site Import Spider (Version 1.0a)
Copyright (C) 2008 Scott Barnett

This program is free software; you can redistribute it and/or
modify it under the terms of Version 3 of the GNU General Public
License as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

*********************************************************************

This script is intended to be run from the console, and by a Stronghold developer
or by any PHP developer with a bit of proactive problem solving ability. It's not
a magic button, and was never intended to be (there's no such thing). Each import
has its own unique circumstances that will require individual tweaking.

FOR BEST PERFORMANCE, RUN IT ON THE SOURCE WEBSERVER (Duh?)

*/

$parent_url="www.mydomain.com";

//$throttle="20000"; //microseconds
$throttle="0"; //microseconds

$include_types=array("htm","html","php");

$db_table_prefix="stronghold_";
$db_common_prefix="common_"; // (read only)

$system_path="/opt/cms";

$content_start="<!-- Begin Content -->";
$content_stop="<!-- End Content -->";

$purge=true; //it will probably take several tries to get the import right, this will purge the content tables before importing

// End of configurable options
//******************************************************************************************************
// The actual application

if (php_sapi_name()!="cli"){ echo ("FATAL: This script cannot be invoked with this method.\n"); exit(); }

//dns and port settings
$server_domain = $parent_url;
$server_port = getservbyname('www', 'tcp');
$server_ip = gethostbyname($parent_url);
echo ($server_domain." - ".$server_ip.":".$server_port."\n");

//load the libraries
include ("../_system/stronghold.php");
include ("../_modules/text.php");

set_time_limit(0);

if ($purge){
	//echo ("Purging database => ");
	$sql="DELETE FROM ".$db_table_prefix."pages;";
	$db->query($sql);
	$sql="DELETE FROM ".$db_table_prefix."module;";
	$db->query($sql);
	$sql="DELETE FROM ".$db_table_prefix."module_text;";
	$db->query($sql);
	$sql="DELETE FROM ".$db_table_prefix."editlog;";
	$db->query($sql);
	$sql="SELECT setval('".$db_table_prefix."pages_page_id_seq',1)";
	$db->query($sql);
	//echo ("DONE\n");
}
//exit();

$url_cache=new url_cache();

while ($url=$url_cache->next_url()){

	$full_url="http://".$parent_url.$url;

	//display the header to the console	
	echo ("-------------------------------------------------------------\n");
	echo ("(".$url_cache->pending_count." - ".$url_cache->spidered_count.") Fetching: ".custom_url_encode($full_url)." => ");

	//retrieve the page
	$stuff=socket_retrieve($server_ip,$server_domain,$server_port,$url);
	if (strpos($stuff,"\n")===false){ echo ("FATAL: Corrupt header response"); exit(); }
	$http_response=substr($stuff,0,strpos($stuff,"\n")); //get the first line
	$code=substr($http_response,0,3);

	//display the header response
	echo ($http_response."\n");
	
	if ($code=="301"){ //redirect
		$url=substr($http_response,5,strlen($http_response));
		$pending_urls[]=$url; //add it to the spider list	
	}
	if ($code=="200"){ // OK
		//extract and cache an urls in the page	
		$url_list=extract_urls($stuff,$url,$parent_url,$include_types);
		print_r($url_list);
		foreach ($url_list as $id => $new_url){ $url_cache->add_url($new_url); }

		//insert the page into the database
		$result=spider_create_page($db,$db_table_prefix,$url,$stuff,$content_start,$content_stop);
		if ($result!==true){ echo ("WARNING: Creating page failed: ".$result."\n"); }
	}
	
	usleep($throttle);
}





//******************************************************************************************************
// Classes and funtions

//create a page, return an error message if page creation fails
function spider_create_page($db,$db_table_prefix,$url,$content,$content_start,$content_stop){
	$now=time();
	
	$url=cms_url($url); //process a url
	if ($url==NULL){ return ("Problem with the URL"); }

	$body=snaffle($content,$content_start,$content_stop);
	if ($body==false){ return ("Could not locate document body"); }
	
	$title=snaffle($content,"<title>","</title>");
	if ($title==false){ return ("Could not locate title"); }
	
	//work out the link
	$link=explode("/",$url);
	$link=$link[count($link)-1]; //it's the last token
	
	//check there's a parent
	if ($url=="/"){ //ignore the doc root		
		$parent_id="NULL";
	} else {
		$parent_url=parent_url($url);
		$sql="SELECT page_id FROM ".$db_table_prefix."pages WHERE url='".$parent_url."'";
		$rs=$db->query($sql);
		if (count($rs)!=1){ return ("Parent '".$parent_url."' does not exist"); }
		$parent_id=$rs[0]["page_id"];
	}	
	
	//check the page doesn't already exits
	$sql="SELECT page_id FROM ".$db_table_prefix."pages WHERE url='".$url."'";
	//echo ($sql."\n");
	$rs=$db->query($sql);
	if (count($rs)>0){ return ("Document already exists"); }
	

	$page=array();
	$page["time_created"]=$now;
	$page["parent"]=$parent_id;
	$page["workflow_id"]=1;
	$page["link"]=sql_snull($link);
	$page["url"]="'".$url."'";
	$page["module_id"]=1;
	$page["archive_date"]="NULL";
	$page["levels"]=url_levels($url);
	
	//create the page
	echo ("Creating Page '".$url."' => ");
	$sql="INSERT INTO ".$db_table_prefix."pages (".implode(",",array_keys($page)).") VALUES (".implode(",",$page).")";
	$db->query($sql);
	//echo ($sql."\n");
	echo ("DONE\n");

	$page_id=max_value($db,"page_id",$db_table_prefix."pages");
	$title=prepare_str($title);
	$short_title=substr($title,0,20);
	$body=strongholdify($body);
	
	//create the module header
	$header=array();
	$header["page_id"]=$page_id;
	$header["version_id"]=1;
	$header["title"]=$title;
	$header["short_title"]=$short_title;
	$header["heading"]=1;
	$header["status"]=1;
	$header["created_date"]=$now;
	$header["nav_id"]=1;
	$header["mod_time"]=$now;
	$header["mod_userid"]=1;
	echo ("Creating Module => ");
	$result=create_module_header($db,$db_table_prefix,1,$header);
	if ($result!==true){ echo ($result."\n"); } else { echo ("DONE\n"); }
	
	//$module["data"]="'".addslashes($body)."'";
	
	//create the module instance
	$module=array();
	$module["page_id"]=$page_id;
	$module["version_id"]=1;
	$module["body_text"]=$body;
	$module["notes"]="'".rev_note("Created by Stronghold Import Spider",NULL,"SPIDER")."'";
	echo ("Creating Module => ");
	$result=create_module($db,$db_table_prefix,$user_id,$module);
	if ($result!==true){ echo ($result."\n"); } else { echo ("DONE\n"); }
	
	//create the log entry
	/*
	$log=array();
	$log["page_id"]=$page_id;
	$log["version_id"]=1;
	$log["user_id"]=1;
	$log["timestamp"]=$now;
	$log["event"]=7;
	echo ("Logging the CREATE => ");
	$sql="INSERT INTO ".$db_table_prefix."editlog (".implode(",",array_keys($log)).") VALUES (".implode(",",$log).");";
	$db->query($sql);
	echo ("DONE\n");
	*/
	
	return (true);
}

//******************************************************************************************************
// URL cache object

class url_cache{
	var $pending_urls=array();
	var $spidered_urls=array();
	var $pending_count=0;
	var $spidered_count=0;

	function url_cache(){ //default constructor
		//define our nasty cache object, surely no site will be more than 20 levels deep???
		$max_depth=20;
		for ($i=0; $i<$max_depth; $i++){
			$this->pending_urls[$i]=array();
			$this->spidered_urls[$i]=array();
		}
		$this->pending_urls[0][]="/";
		$this->pending_count++;
	}
	
	//return the next URL in the cache, always the one with the least levels (create parents first)
	function next_url(){
		//print_r($this->pending_urls);
		foreach ($this->pending_urls as $level => $urls){
			if (count($urls)>0){
				$url=array_pop($this->pending_urls[$level]);
				$this->spidered_urls[$level][]=$url; //add it to the spidered array
				$this->spidered_count++;
				return ($url);
			}
		}
		return (false);
	}
	
	//add a url to be spidered
	function add_url($url){
		$levels=url_levels($url); //how many levels in the url

		if (!in_array($url,$this->pending_urls[$levels]) && !in_array($url,$this->spidered_urls[$levels])){ // if it isn't already pending or spidered
			echo ("    Found URL: ".$url."\n");
			$this->pending_urls[$levels][]=$url; //add it to the pending array
			$this->pending_count++;
		}
		return (true);
	}
}


//***********************************************************************************************************************
// Utility functions

function strongholdify($str){
	$str=str_replace("/images/","/resources/images/",$str);
	return($str);
}

//return the parent url for a given url
function parent_url($url){
	$parent_url=substr($url,1,strlen($parent_url)); //ditch the first slash
	$parent_url=explode("/",$url);
	unset($parent_url[count($parent_url)-1]); //ditch the last token
	$parent_url=implode("/",$parent_url);
	if ($parent_url[0]!="/"){ $parent_url="/".$parent_url; }
	return ($parent_url);
}

//return a url for the CMS, remove trailing slashes and .htm extensions
function cms_url($url){
	if ($url=="/"){ return ($url); } //ignore the doc root
	if (strpos($url,"?")!==false){ return (NULL); } //bail out on dynamic URL's
	
	//$url=realpath($url); //ditch .././ references

	$ext=file_extension($url);
	if (in_array($ext,array("htm","html","php"))){ $url=substr($url,0,(strlen($url)-strlen($ext)-1)); } //ditch the file extension
	
	if ($url[strlen($url)-1]=="/"){ $url=substr($url,0,strlen($url)-1); } //trim the last slash if it exists
	
	return ($url);
}

//retuns the text between 2 block elements
function snaffle($txt,$block_start,$block_stop){
	$snaf=NULL;
	
	//figure out the starting position
	$pos_start=strpos($txt,$block_start);
	if ($pos_start===false){ return (false); }
	$pos_start=$pos_start+strlen($block_start);
	
	//figure out the length
	$pos_stop=strpos($txt,$block_stop);
	if ($pos_stop===false){ return (false); }
	$length=$pos_stop-$pos_start;
	
	//grab the block of text
	$snaf=substr($txt,$pos_start,$length);
	
	return ($snaf);
}

// remove illegal characters from urls
function custom_url_encode($url){
	$illegal=array('?','=',' ','(',')','&','@');
	$legal=array('%3F','%3D','%20','%28','%29','%26','%40');
	$url=str_replace($illegal,$legal,$url);
	return($url);
}

//retrieve a page using sockets
function socket_retrieve($ip,$domain,$port,$url){

	// create our HTTP header
	$in="GET ".custom_url_encode($url)." HTTP/1.1\r\n";
	$in.="Host: ".$domain."\r\n";
	$in.="Connection: close\r\n";
	$in.="User-Agent: Stronghold Import Spider Ver 1.0\r\n\r\n";
	//echo ($in);
	$out="";
	$response="";
	
	$socket = socket_create(AF_INET, SOCK_STREAM, SOL_TCP); // Create a TCP/IP socket
	if ($socket<0){echo "socket_create() failed: reason: " . socket_strerror($socket) . "\n"; exit(); }
	$result = socket_connect($socket, $ip, $port); // Make the connection
	if ($result < 0) { echo "socket_connect() failed.\nReason: ($result) " . socket_strerror($result) . "<br>\n"; exit(); }
	socket_write($socket, $in, strlen($in)); // send the header
	while ($out = socket_read($socket, 2048)) { $response.=$out; } // read the response <------------------------- delays occur here
	socket_close($socket); //close the socket


	if (strpos($response,"HTTP/1.")===false){
		//the server doesn't send a 200 response header after a 301, accounting for this
		$http_code=200;
		$http_response="OK (no header)";

	} else {
		$response=str_replace("\r","\n",$response); //DO NOT GET RID OF THIS!!!!! DAMM \r
		$first_line=substr($response,0,strpos($response,"\n")); //grab the first line
		$first_bits=explode(" ",$first_line); //break it up
		$http_code=$first_bits[1]; //the http code is the second token
		unset ($first_bits[0]); unset ($first_bits[1]); //remove the http version and code
		$http_response=implode(" ",$first_bits); //implode the rest into the full response string
	}
	
	$return_value=$http_code.": ".$http_response."\n";

	if ($http_code=="301"){ //moved permanently
		$header_lines=explode("\n",$response);
		$h_count=count($header_lines);
		for ($i=0; $i<count($header_lines); $i++){ //for each line in the header
			if (substr($header_lines[$i],0,9)=="Location:"){
				$location=substr($header_lines[$i],10,strlen($header_lines[$i])-1); //drop the "Location: "
				$location=explode("/",$location);
				//print_r($location);
				unset($location[0]);
				unset($location[1]);
				$location[2]=NULL;
				$location=implode("/",$location);
				$i=$h_count;
			}
		}
		$return_value=$http_code.": ".$location."\n"; //overide the default 301
	}
		
	elseif ($http_code=="200"){ // OK
		$temp=explode("<html>",$response);
		$return_value.="<html>".$temp[1];
	} else{
		//echo ($response);
	}

	
	return ($return_value);
}

//return all the urls in a given html page
// $str = body document
// $url = domain name
// $parent_url = 
function extract_urls($str,$url,$parent_url,$include_types){

	//echo ("=> ".$url." + ".$parent_url."\n");
	//remove all the carriage returns
	$str=str_replace("\n","",$str);
	$str=str_replace("\r","",$str);
	$url=trim($url);

	if ($url[strlen($url)-1]=="/"){ //if the last character is a slash
		$directory=$url;
	} else {
		$directory=dirname($url);
		//echo ("!".$directory[strlen($directory)-1]."!\n");
		if ($directory!="/" && $directory[strlen($directory)-1]!="/"){ $directory.="/"; } //if the last character is a slash
	}
	//echo ("=> ".$directory."\n");

	$url_list=array();
	
	$str=explode("<",$str); //break the string up into fragments
	
	foreach ($str as $id => $current_foo){
		if ($current_foo[0]!="/"){ //ignore closing tags
			$pos=strpos($current_foo,">");

			if ($pos!==false){ //ignore lines that don't have a complete tag
				$tag_contents=substr($current_foo,0,$pos); //drop everything after the >
				$tag_contents=single_space($tag_contents); //format the tag properly

				$tag=substr($tag_contents,0,strpos($tag_contents," ")); //get the tag name, from 0 to the first space

				if (in_array($tag,array("a"))){ //if it's a tag we're interested in
					$tag_elements=tag_array($tag_contents); //break up the tag into elements

					$location=NULL;
					if (strtoupper($tag)=="A"){ //process an A tag
						if (array_key_exists("href",$tag_elements)){ $location=$tag_elements["href"]; }
					}
					
					// We have a location, do generic link checking
					if ($location!=NULL){ // we have a valid link
	//echo ("Start: ".$location."!!\n");
						if (strpos($location,"#")!==false){ $location=NULL; } //ignore named anchor references
						elseif (strpos($location,"http:")!==false){ $location=relative_url($location,$parent_url); } //get a relative url for any domain references
						elseif (strpos($location,"https:")!==false){ $location=relative_url($location,$parent_url); } //get a relative url for any domain references
						elseif (strpos($location,"file:")!==false){ $location=NULL; } // file links
						elseif (strpos($location,"javascript:")!==false){ $location=NULL; } // file links

						if (strpos($location,".")!==false && !in_array(file_extension($location),$include_types)){ $location=NULL; } //check the file type if we can
						if ($location[0]!="/"){ $location=$directory.$location; }
						$url_list[]=absolute_path($location);
					}
				}
			}
		}
	}
	$url_list=array_unique($url_list);
	//pre($url_list);
	
	return ($url_list);
}

function relative_url($str,$parent_url){ // not implemented yet!!!
	return ($url);
}


// return an absolute path given a relative path
// [/path/../images/image.jpg] returns [/images/image.jpg]
function absolute_path($path) {
	if ($path=="/"){ return ("/"); } //ignore the doc root
	if ($path[0]!="/"){ return (false); } //return false if it's not a good url
	
	$path=substr($path,1,strlen($path)-1); //ditch the first /
	
	$relative_bits=explode("/",$path);
	$absolute_bits=array();
	
	foreach ($relative_bits as $id => $bit){
		if ($bit==".."){
			unset($absolute_bits[count($absolute_bits)-1]);
		} else {
			$absolute_bits[count($absolute_bits)]=$bit;
		}
	}
	$path="/".implode("/",$absolute_bits);

	return ($path);
}

//remove multiple spaces from text strings (makes tags easier to explode() )
function single_space($str){
	$bits=explode (" ",$str);
	$final_str="";
	
	for ($i=0; $i<count($bits); $i++){
		$bit=$bits[$i];
		if ($bits[$i]=="="){
			$final_str=substr($final_str,0,strlen($final_str)-1); //trim off a space
			$final_str.="=";
		} else{
			$final_str.=$bit." ";
		}
	}
	$final_str=substr($final_str,0,strlen($final_str)-1); //trim off a space
	return ($final_str);
}

//split the contents of a tag into a formatted array
function tag_array($tag){
	$return_array=array();


	if (strlen($tag)>0){

		$tag=str_replace("\"","'",$tag); //so we only have to deal with 1 set of quotes
		$tag=single_space($tag);
		$arr=explode ("'",$tag);
		//print_r($arr);
		
		foreach ($arr as $key => $value){
			$value=trim($value);
			
			if (strpos($value,"=")!==false){
				$bits=explode(" ",$value);
				//print_r($bits);
				for ($i=0; $i<count($bits)-1; $i++){ // foreach except the last
					$return_array[trim($bits[$i])]=1;
				} 

				$last_bit=trim($bits[count($bits)-1]);
				$last_bit=trim(substr($last_bit,0,strlen($lastbit)-1));
				$return_array[$last_bit]=$arr[$key+1];
			}
			
		}
	}
	

	return ($return_array);
}



//return the file extension for a give file or url
function file_extension($str){
	$files=explode("/",$str);
	if (count($files)==0){ return (false); }
	$file_name=$files[count($files)-1]; //get the last token
	
	if (strpos($file_name,"?")!==false){ return (NULL); } //it's a dynamic url, leave it alone
	
	$file_parts=explode(".",$str);
	if (count($file_parts)==0){ return (NULL); } //no file extension

	$ext=$file_parts[count($file_parts)-1]; //get the file extension
	return ($ext);
}

//remove quotation marks from text strings
function no_quotes($str){
	$str=str_replace('"',"",$str); //remove quotation marks
	$str=str_replace("'","",$str); //remove quotation marks
	$str=trim($str);
	return ($str);
}

?>
