<?
/*
Stronghold CMS - Site Transfer Script
Copyright (C) 2008 Scott Barnett

This program is free software; you can redistribute it and/or
modify it under the terms of Version 3 of the GNU General Public
License as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

*********************************************************************

This script is intended to be run from the console, and by a Stronghold
developer or by anyone with a bit of proactive problem solving ability.

At this moment this will only transfer sites in the same database. The workaround
for this is to create a schema in the database for the new site, transfer the data, 
then export the tables using pg_dump

THIS WILL PURGE THE DATA FROM THE DESTINATION DATABASE/DATASTORE
*/

//permissions to set on the imported files when you're done
$permissions="apache.apache";

$test=false;

// End of configurable options
//******************************************************************************************************
// The actual application

// error checking
if (php_sapi_name()!="cli"){ echo ("FATAL: This script cannot be invoked with this method.\n"); exit(); }

if ((count($argv)!=3) || in_array("--help",$argv)){
	echo ("Usage: site_transfer.php [source config.php] [destination config.php]\n");
	exit();
}

//fix the doc root variable so the libraries work
$_SERVER["DOCUMENT_ROOT"]=dirname(dirname(realpath($_SERVER["PWD"]."/".$_SERVER["PHP_SELF"])));

//database library
include ($_SERVER["DOCUMENT_ROOT"]."/_system/database.php");

//check the config files exist
$source_config=$argv[1];
$destin_config=$argv[2];
if (!file_exists($source_config)){ echo ("FATAL: Could not locate source configuration file - ".$source_config."\n"); exit(); }
if (!file_exists($destin_config)){ echo ("FATAL: Could not locate destination configuration file - ".$destin_config."\n"); exit(); }
//load the source configuration
$CONFIG=array();
include ($source_config);
$config_source=$CONFIG;
//load the destination configuration
$CONFIG=array();
include ($destin_config);
$config_destin=$CONFIG;

//establish the database connections
$source_db=database_connector($config_source);
$destin_db=database_connector($config_destin);
if (!$source_db){ echo ("FATAL: Could not establish connection to source database\n"); exit(); }
if (!$destin_db){ echo ("FATAL: Could not establish connection to destination database\n"); exit(); }

//table prefixes
$source_prefix=$config_source["db_table_prefix"];
$destin_prefix=$config_destin["db_table_prefix"];
$prefixes=array($source_prefix,$destin_prefix); // the prefixes array is used by db_schema.php
//echo ($destin_prefix); exit();

//libraries (include them AFTER you've loaded the configs)
//include ($_SERVER["DOCUMENT_ROOT"]."/_system/stronghold.php");

$db_common_prefix="foo"; //just fudge it, we don't replicate the common anyways
$populate_database=false;
$database_type="postgres";
include ($_SERVER["DOCUMENT_ROOT"]."/_tools/db_schema.php");
//pre($txt_tables); exit();

$serials=array();

echo ("\n--------------------------------------------------------------\n");
echo ("Replicating Tables\n\n");

//process the tables
foreach ($txt_tables as $table_name => $attr){
	//only process the appropriate tables

	if (strpos($table_name,$source_prefix)!==false){

		$dest_table=$destin_prefix.(substr($table_name,strlen($source_prefix))); //work out the destination table

		echo ("   ".$table_name." -> ".$dest_table);
		//load the table data
		$sql="SELECT * FROM ".$table_name;
		$rs=$source_db->query($sql);

		//purge the destination table
		$sql="DELETE FROM ".$dest_table;
		if (!$test){ $destin_db->query($sql); }

		//insert the data
		if (count($rs)>0){
			$keys=array_keys($rs[0]);
			$keys_text=implode(",",$keys);
			
			foreach ($rs as $id => $r){ //for each row
				$values=array();
				foreach ($attr as $key => $attributes){ $values[$key]=sql_type($r[$key],$attributes["data_type"],$attributes["is_nullable"]); }
				
				$sql_options=array();
				foreach ($keys as $id => $label_key){ $sql_options[]=$values[$label_key]; }
				$sql="INSERT INTO ".$dest_table." (".$keys_text.") VALUES (".implode(",",$sql_options).")";
				if (!$test){ $destin_db->query($sql); }
			}
		}
		
		//find the sequences		
		foreach ($attr as $key => $attributes){ if ($attributes["data_type"]=="serial"){ $serials[]=$table_name."_".$key."_seq"; } }
		
		echo ("  DONE\n");
	}
}

echo ("\n--------------------------------------------------------------\n");
echo ("Updating Sequences\n\n");

//process all the sequences
foreach ($serials as $id => $serial){
	$sql="SELECT last_value AS val FROM ".$serial;
	echo ("   ".$sql."\n");
	$rs=$source_db->query($sql);
	$value=$rs[0]["val"];
	
	$des_serial=str_replace($source_prefix,$destin_prefix,$serial);
	$sql="SELECT setval('".$des_serial."',".$value.")";
	echo ("   ".$sql."\n");
	if (!$test){ $destin_db->query($sql); }
}
//pre($serials);

//------------------------------------------------------------------------------------------------
//process the datastore

//fetch the paths
$source_path=$config_source["system_datastore"];
$destin_path=$config_destin["system_datastore"];
if ($source_path==NULL){ echo ("FATAL: No source path specified for datastore transfer"); exit(); }
if ($destin_path==NULL){ echo ("FATAL: No destination path specified for datastore transfer"); exit(); }



echo ("\n--------------------------------------------------------------\n");
echo ("Purging the destination datastore\n\n");
//rm has a limit, we need to list and process each folder in a loop
$list=file_ls($destin_path);
foreach ($list as $id => $l){
	$command="rm -Rf ".$destin_path."/".$l;
	if (!$test){ system ($command); }
	//echo ("   ".$command."\n");
	echo (".");
}
echo ("   DONE\n");

echo ("\n--------------------------------------------------------------\n");
echo ("Copying source datastore to destination data store (this may take some time)\n\n");
//cop has a limit, we need to list and process each folder in a loop
$list=file_ls($source_path);
foreach ($list as $id => $l){
	$command="cp -rf ".$source_path."/".$l." ".$destin_path."/";
	if (!$test){ system ($command); }
	//echo ("   ".$command."\n");
	echo (".");
}
echo ("   DONE\n");

echo ("\n--------------------------------------------------------------\n");
echo ("Applying permissions\n\n");
$command="chown ".$permissions." ".$CONFIG["system_datastore"]." -R";
echo ("   ".$command."\n");
if (!$test){ system ($command); }
echo ("   DONE\n");

echo ("\nTransfer complete!\n\n");

//------------------------------------------------------------------------------------------------

//return an sql string for a given data type
function sql_type($var,$type,$nulls){
	if ($type==NULL){ echo ("blank data type - ".$type."\n"); exit(); }
	//echo ($var." - ".$type." - ".$nulls);
	if ($var==NULL && $nulls=="YES"){ return ("NULL"); }
	
	if (in_array($type,array("integer","smallint","serial"))){ return ($var); } //numbers
	if (in_array($type,array("character","character varying"))){ return ("'".$var."'"); }
	if ($type=="text"){ return ("'".addslashes($var)."'"); }
	
	echo ("unknown data type - ".$type."\n"); exit();
}

?>
