<?
/*
Stronghold CMS - Document Proxy
Copyright (C) 2008 Scott Barnett

This program is free software; you can redistribute it and/or
modify it under the terms of Version 3 of the GNU General Public
License as published by the Free Software Foundation.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.
*/

include ($_SERVER["DOCUMENT_ROOT"]."/_system/stronghold.php");

include ($_SERVER["DOCUMENT_ROOT"]."/_system/session.php");

$req=prepare_str($_GET["request"]);

if ($req){
	//-- User Request --//
	
	$r=explode(".",$req);
	if (count($r)!=2){ not_found(); } //if we don't have 2 bits (might change this later)
	if (strlen($r[1])!=3){ not_found(); } //only support 3 char file extensions

	//figure out the extension
	$ext=file_type(prepare_str($r[1]));
	if ($ext==NULL){ not_found(); }

	//resource library
	include ($_SERVER["DOCUMENT_ROOT"]."/_system/resource.php");
	$res=new resource($db,$CONFIG["db_table_prefix"]);

	//figure out the resource id
	if (is_numeric($r[0])){
		$resource_id=$r[0];
	} else {
		//find a resource id by a given filename and file type
		$resource_id=$res->retrieve_resource_id($r[0],$r[1]);
		if (!$resource_id){ not_found(); }
	}

	//retrieve the resource
	$r=$res->retrieve($resource_id);
	if (!$r){ not_found(); }

	//determine the version to serve
	$latest_version=NULL;
	if (($_SERVER["HTTPS"]=="on") && $_SESSION[$CONFIG["system_session"]."username"]!=NULL){ // if they're logged in
		//admin request, latest version
		$versions=array_keys($r["versions"]);
		$latest_version=array_shift($versions);
	} else {
		//user request, latest approved version
		foreach($r["versions"] as $version_id => $version){
			if ($latest_version==NULL && $version["status"]==3){ $latest_version=$version_id; }
		}
	}
	if ($latest_version==NULL){ not_found(); }

//pre($r); exit();
	//all ok, setup the download
	$download_path=$CONFIG["system_datastore"]."/".$resource_id."/".$resource_id."-".$latest_version;
	$download_filename=$resource_id.".".$ext;
	
	
} else { //----------------------------------------------------------------------

	//-- Admin Request --//
	$resource_id=prepare_int($_GET["resource_id"]);
	$version_id=prepare_int($_GET["version_id"]);
	if ($resource_id==NULL || $version_id==NULL){ not_found(); } //block malformed requests
	
	//check they're an admin
	$requires_access=1;
	//include ($_SERVER["DOCUMENT_ROOT"]."/_system/session.php");

	//retrieve the resource
	include ($_SERVER["DOCUMENT_ROOT"]."/_system/resource.php");
	$res=new resource($db,$CONFIG["db_table_prefix"]);
	$r=$res->retrieve_version($resource_id,$version_id);

	//all ok, setup the download
	$ext=$r["file_type"];
	$download_path=$CONFIG["system_datastore"]."/".$resource_id."/".$resource_id."-".$version_id;
	$download_filename=$resource_id.".".$ext;
}

//all done with the database, close the connection
$db=NULL;

//*************************************************************************************************************
// Send a file

// check the file exists in the datastore
if (!file_exists($download_path)){ fatal("Resource retrieval fault: Could not locate resource - ".$download_path); }

//load the mime type
$mime=mime_type($ext);
if (!$mime){ fatal ("Unsuported document type - ".$ext); }

//echo "!!!"; exit();

//header('Last-Modified: '.gmdate('D, d M Y H:i:s', filemtime($path)).' GMT', true, 200);
header("Pragma: anytextexeptno-cache",true);
if ($_SERVER["HTTPS"]=="on"){ //tweak for SSL connections
	header('Pragma: private');
	header('Cache-control: private, must-revalidate');
}
header("Content-Type: ".$mime);
header("Content-Disposition: inline; filename=\"".$download_filename."\"");
header("Content-Length: ".filesize($download_path));
readfile($download_path);

//*************************************************************************************************************
// Utility functions

//send a 404 and terminate the execution
function not_found(){
	header("HTTP/1.0 404 Not Found");
	$last_modified=time();

	$document=array();
	$document["title"]="Document Not Found!";
	$document["body"]=file_get_contents($_SERVER["DOCUMENT_ROOT"]."/_error/404.htm");
	include ($_SERVER["DOCUMENT_ROOT"]."/includes/template.php");
	exit();
}

//return the mime type of a given file extension
function mime_type($ext){
	//to add file types to this list, please email scott@strongholdcms.org
	//most common resources are listed at the top for performance
	if ($ext=="pdf"){ return ("application/pdf"); }
	if ($ext=="jpg"){ return ("image/jpeg"); }
	if ($ext=="gif"){ return ("image/gif"); }
	if ($ext=="png"){ return ("image/png"); }

	//------------------------------------------------------------------
	if ($ext=="asf"){ return ("video/x-ms-asf"); }
	if ($ext=="avi"){ return ("video/x-msvideo"); }
	if ($ext=="bmp"){ return ("image/bmp"); }
	if ($ext=="crt"){ return ("application/x-x509-ca-cert"); }
	if ($ext=="dll"){ return ("application/x-msdownload"); }
	if ($ext=="doc"){ return ("application/msword"); }
	if ($ext=="dot"){ return ("application/msword"); }
	if ($ext=="eps"){ return ("application/postscript"); }
	if ($ext=="exe"){ return ("application/octet-stream"); }
	//gif ^
	if ($ext=="gz"){ return ("application/x-gzip"); }
	if ($ext=="hlp"){ return ("application/winhlp"); }
	if ($ext=="htm"){ return ("text/html"); }
	//if ($ext=="html"){ return ("text/html"); } //4 chars unsupported
	if ($ext=="ico"){ return ("image/x-icon"); }
	if ($ext=="jpe"){ return ("image/jpeg"); }
	//jpg ^
	//if ($ext=="jpeg"){ return ("image/jpeg"); }  //4 chars unsupported
	if ($ext=="js"){ return ("application/x-javascript"); }
	if ($ext=="mov"){ return ("video/quicktime"); }
	if ($ext=="mp2"){ return ("video/mpeg"); }
	if ($ext=="mp3"){ return ("audio/mpeg"); }
	//if ($ext=="mpeg"){ return ("video/mpeg"); }   //4 chars unsupported
	if ($ext=="mpg"){ return ("video/mpeg"); }
	if ($ext=="mpp"){ return ("application/vnd.ms-project"); }
	// pdf ^
	// png ^
	if ($ext=="ppt"){ return ("application/vnd.ms-powerpoint"); }
	if ($ext=="ppt"){ return ("application/vnd.ms-powerpoint"); }
	if ($ext=="ps"){ return ("application/postscript"); }
	if ($ext=="pub"){ return ("application/x-mspublisher"); }
	if ($ext=="qt"){ return ("video/quicktime"); }
	if ($ext=="rtf"){ return ("application/rtf"); }
	if ($ext=="svg"){ return ("image/svg+xml"); }
	if ($ext=="swf"){ return ("application/x-shockwave-flash"); }
	if ($ext=="tar"){ return ("application/x-tar"); }
	if ($ext=="tgz"){ return ("application/x-compressed"); }
	if ($ext=="tif"){ return ("image/tiff"); }
	//if ($ext=="tiff"){ return ("image/tiff"); } //4 chars unsupported
	if ($ext=="txt"){ return ("text/plain"); }
	if ($ext=="xls"){ return ("application/vnd.ms-excel"); }
	if ($ext=="z"){ return ("application/x-compress"); }
	if ($ext=="zip"){ return ("application/zip"); }

	return (false);
}
?>
